abt.auto <-
function (form, data, ... , cols = "blue", pred = 1, mono = TRUE, size = TRUE, 
	seed = sample(1:100, 1), monitor = FALSE, method = c("pred","imp")[1], 
	np = c(5, 3)[1], long = TRUE, plot=TRUE) 
{
    fit <- abt(form, data, monitor = monitor, ...)
    pred.err <- pred.err(fit)
    cat("\nPredicted error\n")
    print(unlist(pred.err))
    xnames <- fit[[1]]$var.names
    nxs <- length(xnames)

	vmono <- rep(0,nxs)
    if (nxs > 1) {
    	cat("\nPREDICTOR SELECTION USING ")
    	if (method == "imp") {
			cat("IMPORTANCE\n")
	   		sumry <- summary(fit, ord=FALSE, plot=FALSE)
    		perrs <- min.imp <- rep(NA,nxs)
			perrs[1] <- pred.err(fit)[[1]]
    		stop.now <- FALSE
    		i <- 1
    		while(!stop.now) {
		    		min.imp[i] <- min(sumry[,2]) 
		    		j <- (1:nrow(sumry))[sumry[,2]==min.imp[i]]
		            if (long) 
		              cat("\nTest drop", xnames[j], "\n")
		            tempform <- formula(paste("~.-", xnames[j], collapse = ""))
		            dropform <- update.formula(form, tempform)
		            vmono.drop <- vmono[-j]
		            if (long)   print(dropform)
		            fit <- abt(dropform, data, monitor = F, seed.stratify = seed,
		            var.mono = vmono.drop, ...)
		    		sumry <- summary(fit, ord=FALSE, plot=FALSE)
					perrs[i+1] <- min(sumry[,2])
		            if (perrs[i+1] > perrs[i]) stop.now <- TRUE
		        	else {
  		        		cat("\nDropping", xnames[j], "\n")
		        		form <- dropform
		        		xnames <- xnames[-j]
		        		vmono <- vmono.drop
		        		i <- i+1
		        	}
		        }
    	}
		else if (method=="pred") {
			cat("PREDICTION ERROR\n")
		    best.pred.err <- pred.err[[2]]
		    stop.now <- FALSE
		    best <- 0
		    xnames <- fit[[1]]$var.names
		    while (!stop.now && length(xnames) > 1) {
		        cat("\nCurrent pedictors:", xnames, "\n")
		        perrs <- list()
		        for (i in 1:nxs) {
		            if (long) 
		              cat("\nTest drop", xnames[i], "\n")
		            tempform <- formula(paste("~.-", xnames[i], collapse = ""))
		            dropform <- update.formula(form, tempform)
		            vmono.drop <- vmono[-i]
		            if (long)   print(dropform)
		             fit <- abt(dropform, data, monitor = monitor, seed.stratify = seed,
		            var.mono = vmono.drop, ...)
		            perrs[[i]] <- pred.err(fit)
		            if (long) {
		              cat("Predicted error\n")
		              print(unlist(perrs[[i]]))
		            }
		        }
		        perrs1 <- sapply(perrs, "[[", 2)
		        best <- min(perrs1)
		        nbest <- ((1:nxs)[perrs1 == best])[1]
		        if (best < best.pred.err) {
		            best.pred.err <- best
		            stop.now <- FALSE
		            newform <- formula(paste("~.-", xnames[nbest], 
		              collapse = ""))
		            vmono <- vmono[-nbest]
		            form <- update.formula(form, newform)
	        		cat("\nDropping", xnames[nbest], "\n")
		            xnames <- xnames[-nbest]
		            nxs <- length(xnames)
		        }
		        else stop.now <- TRUE
		    }
		}
    }


    if (size) {
        cat("\nEVALUATE BEST TREE SIZE\n\n")
        best.size <- abt.size(form, data, monitor = monitor, seed.stratify = seed, 
           ..., cols=cols)$best.size
        fit <- abt(form, data, int = best.size, monitor = monitor, 
            seed.stratify = seed, ...)
    }
    if (size) 
        fit <- abt(form, data, int = best.size, monitor = monitor, 
            seed.stratify = seed, ...)
    else fit <- abt(form, data, monitor = monitor, seed.stratify = seed, ...)
    pred.err <- pred.err(fit)
    cat("\nPredicted error\n")
    print(unlist(pred.err))

	best.size <- fit[[1]]$interaction.depth

    if (mono) {
    	cat("\nMONOTONICITY CHECKS\n")
        if (np == 5) 
            levs <- c(2, 4, 6, 8, 10)
        else if (np == 3) 
            levs <- c(2, 6, 10)
        vtype <- fit[[1]]$var.type
        nxs <- length(vtype)
       
        for (i in 1:nxs) {
            if (vtype[i] == 0) {
                xdata <- data.frame(fit[[1]]$var.levels[[i]][levs])
                names(xdata) <- fit[[1]]$var.names[i]
                ord <- order(peffects.abt(fit, data = xdata, 
                  ret = T)[, 2])
                if (all(ord == (1:np))) 
                  vmono[i] <- 1
                else if (all(ord == (np:1))) 
                  vmono[i] <- -1
            }
        }
        cat("\nMonotone predictors", vmono, "\n")
        if (any(vmono != 0)) {
            if (size) 
                fit <- abt(form, data, int = best.size, monitor = F, 
                  var.mono = vmono, seed.stratify = seed, ...)
            else fit <- abt(form, data, var.mono = vmono, monitor = F, 
                seed.stratify = seed, ...)
        }
    }

	fit <- abt(form, data, monitor = monitor, seed.stratify = seed,
                var.mono = vmono, int = best.size, ...)
    if (plot) plot.all(fit, se=2)
    cat("\nFinal model:\n")
    print(form)
    if (mono) cat("Monotone predictors", vmono,"\n")    
    if (size) cat("Interaction depth = ",best.size,"\n")
    fit
}

