abt.drop.perm <-
function(abt.fit, ..., drop=T, perm=T, reps=100, verbose=F) {
	trms <- attr(abt.fit[[1]]$Terms,"term.labels")
	nt <- length(trms)
	n <- length(abt.fit[[1]]$mdata[,1])
	res <- matrix(0,nrow=1+perm+2*drop,ncol=nt)
	if (drop) drop.names <- c("Drop Error","Drop Error(%)") else drop.names <- NULL
	if (perm) perm.names <- "Perm P" else perm.names <- NULL
	dimnames(res) <- list(c("Influence(%)",drop.names,perm.names),trms)
	res[1,] <- summary(abt.fit,ord=F,plotit=F)[,2]
	err1 <- loss(abt.fit)

	if (drop) {
		yname <- names(abt.fit[[1]]$mdata)[1]
		form <- abt.fit[[1]]$Terms
		n.trees <- abt.fit[[1]]$n.trees 
		distribution <- abt.fit[[1]]$distribution 
	  	shrinkage <- abt.fit[[1]]$shrinkage 
		bag.fraction <- abt.fit[[1]]$bag.fraction
		var.monotone <- abt.fit[[1]]$var.monotone 		 
		n.minobsinnode <- abt.fit[[1]]$n.minobsinnode		
		interaction.depth <- abt.fit[[1]]$interaction.depth
		for (i in 1:nt) { 
			dropform <- formula(paste(yname,paste(trms[-i],collapse = "+"),sep="~"))
			fit <- abt(dropform,data=abt.fit[[1]]$mdata, n.trees=n.trees,
			distribution=distribution, shrinkage=shrinkage, bag.fraction=bag.fraction,
			n.min=n.minobsinnode, int=interaction.depth, var.m=var.monotone[-1],monitor=F)
			res[2,i] <- loss(fit)
			res[3,i] <- 100*(loss(fit)/err1-1)
		}
	}

	if (perm) {
		mres <- matrix(NA,nrow=reps,ncol=nt)
		if (verbose) cat("Permuting predictors: ")
		for (i in 1:nt) {
			if (verbose) cat("\n",trms[i]," ")
			for (j in 1:reps) {
				if (verbose) cat(j," ")
				temp <- abt.fit[[1]]$mdata
				temp[,trms[i]] <- temp[sample(n),trms[i]] 
				ppreds <- predict2.abt(abt.fit,olddata=temp)
				mres[j,i] <- loss(abt.fit,f=ppreds)
			}
			res[3+drop,i] <- mean(err1[[1]] > mres[,i])
		}
	}
	
	cat("\nPredicted error =",signif(err1,4),"\n") 
	print(round(res,3))
	cat("\n")
	if (perm) invisible(list(err=err1,table=res,mres=mres))
	if (!perm) invisible(list(err=err1,table=res))
}

