abt.more <- function(object, n.new.trees = 100, data = NULL, weights = NULL, offset = NULL, verbose = NULL)
{
   if (any(class(object)=="single")) {
        if(is.null(object$Terms) && is.null(object$data))  {
            stop("The abt model was fit using abt.fit (rather than abt) and keep.data was set to FALSE. abt.more cannot locate the dataset.")
        }
        else if(is.null(object$data) && is.null(data)) {
            stop("keep.data was set to FALSE on original abt call and argument 'data' is NULL")
        }
        else if(is.null(object$data)) {
            m <- eval(object$m, parent.frame())
            Terms <- attr(m, "terms")
            a <- attributes(Terms)
        
            y <- as.vector(model.extract(m, response))
            offset <- model.extract(m,offset)
            x <- model.frame(delete.response(Terms),
                            data,
                            na.action=na.pass)
        
            w <- weights
            if(length(w)==0) w <- rep(1, nrow(x))
            w <- w*length(w)/sum(w) # normalize to N
        
            if(is.null(offset) || (offset==0))
            {
                offset <- NA
            }
            Misc <- NA
        
            if(object$distribution == "coxph")
            {
                Misc <- as.numeric(y)[-(1:cRows)]
                y <- as.numeric(y)[1:cRows]
        
                # reverse sort the failure times to compute risk sets on the fly
                i.train <- order(-y[1:object$nTrain])
                i.test <- order(-y[(object$nTrain+1):cRows]) + object$nTrain
                i.timeorder <- c(i.train,i.test)
        
                y <- y[i.timeorder]
                Misc <- Misc[i.timeorder]
                x <- x[i.timeorder,,drop=FALSE]
                w <- w[i.timeorder]
                if(!is.na(offset)) offset <- offset[i.timeorder]
                object$fit <- object$fit[i.timeorder]
            }
        
            # create index upfront... subtract one for 0 based order
            if(ncol(x) > 1)
            {
                x.order <- apply(x[1:object$nTrain,],2,order,na.last=FALSE)-1
            }
            else
            {
                x.order <- order(x[1:object$nTrain,],na.last=FALSE)-1
            }
            x <- data.matrix(x)
            cRows <- nrow(x)
            cCols <- ncol(x)
        }
        else
        {
            y           <- object$data$y
            x           <- object$data$x
            x.order     <- object$data$x.order
            offset      <- object$data$offset
            Misc        <- object$data$Misc
            w           <- object$data$w
            
            cRows <- length(y)
            cCols <- length(x)/cRows
            if(object$distribution == "coxph")
            {
                i.timeorder <- object$data$i.timeorder
                object$fit <- object$fit[i.timeorder]
            }
        }
        
        if(is.null(verbose))        {
            verbose <- object$verbose
        }
        
        x <- as.vector(x)
        
        abt.obj <- .Call("gbm",
                        Y = as.double(y),
                        Offset = as.double(offset),
                        X = as.double(x),
                        X.order = as.integer(x.order),
                        weights = as.double(w),
                        Misc = as.double(Misc),
                        cRows = as.integer(cRows),
                        cCols = as.integer(cCols),
                        var.type = as.integer(object$var.type),
                        var.monotone = as.integer(object$var.monotone),
                        distribution = as.character(object$distribution),
                        rf.nvar=as.integer(object$rf.nvar), 
                        n.trees = as.integer(n.new.trees),
                        interaction.depth = as.integer(object$interaction.depth),
                        n.minobsinnode = as.integer(object$n.minobsinnode),
						n.classes = as.integer(object$num.classes),
                        shrinkage = as.double(object$shrinkage),
                        bag.fraction = as.double(object$bag.fraction),
                        nTrain = as.integer(object$nTrain),
                        fit.old = as.double(object$fit),
                        n.cat.splits.old = length(object$c.splits),
                        n.trees.old = as.integer(object$n.trees),
                        verbose = as.integer(verbose),
                        PACKAGE = "abt")
        names(abt.obj) <- c("initF","fit","train.error","valid.error",
                            "oobag.improve","trees","c.splits")
        
        abt.obj$initF         <- object$initF
        abt.obj$train.error   <- c(object$train.error, abt.obj$train.error)
        abt.obj$valid.error   <- c(object$valid.error, abt.obj$valid.error)
        abt.obj$oobag.improve <- c(object$oobag.improve, abt.obj$oobag.improve)
        abt.obj$trees         <- c(object$trees, abt.obj$trees)
        abt.obj$c.splits      <- c(object$c.splits, abt.obj$c.splits)
        
        # cv.error not updated when using abt.more
        abt.obj$cv.error      <- object$cv.error
        
        abt.obj$n.trees        <- length(abt.obj$trees)
        abt.obj$distribution   <- object$distribution
        abt.obj$train.fraction <- object$train.fraction
        abt.obj$shrinkage      <- object$shrinkage
        abt.obj$bag.fraction   <- object$bag.fraction
        abt.obj$var.type       <- object$var.type
        abt.obj$var.monotone   <- object$var.monotone
        abt.obj$var.names      <- object$var.names
        abt.obj$interaction.depth <- object$interaction.depth
        abt.obj$n.minobsinnode    <- object$n.minobsinnode
        abt.obj$nTrain            <- object$nTrain
        abt.obj$Terms             <- object$Terms
        abt.obj$var.levels        <- object$var.levels
        abt.obj$verbose           <- verbose
        
        abt.obj$mdata <- object$m
        abt.obj$rf.nvar <- object$rf.nvar
        abt.obj$num.classes <- object$num.classes
        abt.obj$nsamps <- object$nsamps
        abt.obj$fitted <- object$fitted
        abt.obj$type <- object$"cv"
        abt.obj$m <- object$m.keep
        
        if(object$distribution == "coxph")
        {
            abt.obj$fit[i.timeorder] <- abt.obj$fit
        }
        if(!is.null(object$data))
        {
            abt.obj$data <- object$data
        }
        else
        {
            abt.obj$data <- NULL
            abt.obj$m <- object$m
        }
        
   }     

   else {
        valid.err <- 0
        nsamps <- object[[1]]$nsamps
        n <- length(nsamps)
        abt.obj <- list()
        
        for (i in 1:length(object)) {
          abt.obj[[i]] <- abt.more(object[[i]], n.new.trees = n.new.trees, 
          data = data, weights = weights, offset = offset, verbose = verbose)
          valid.err <- valid.err + abt.obj[[i]]$valid.error*sum(nsamps==i)
        }

        valid.err <- valid.err/n
        n.trees <- abt.obj[[1]]$n.trees
        valid.err <- smooth.spline(1:n.trees, valid.err)$y
        bb <- valid.err == min(valid.err)
        abt.obj[[1]]$best <- best <- ((1:n.trees)[bb])[1]
        abt.obj[[1]]$best.error <- best.error <- valid.err[best]
        abt.obj[[1]]$mdata <- object[[1]]$mdata
        abt.obj[[1]]$type <- "cv"        
        fitted <- rep(NA,n)
        for (i in 1:length(object)) fitted[nsamps==i] <- predict(abt.obj[[i]], 
        newdata = abt.obj[[1]]$mdata[nsamps==i, , drop = F], n.trees = best) 
        abt.obj[[1]]$fitted <- fitted
    }
    class(abt.obj) <- "abt"
    return(abt.obj)
   
}

