plot.abt <-
function (x, i.var = 1, npts, trim.grid = FALSE, return.grid = FALSE, 
    center = TRUE, cvalue, cfun = mean, add.zero = FALSE, xlab, ylab, 
    zlab = "Partial Dependence", clip = TRUE,  type = c("link", "response")[1], 
    ptype = c("contour", "persp", "lines", "trellis")[1], pcol = "vary",
    fpcol = heat.colors(50), d = 6, theta = 40, n.trees, use = c("best", "all")[1],
    add.legend = TRUE, se = 2, se.smooth = c("smooth", "mean","none")[1], 
    lty = rep(1,20), se.fill = FALSE, ylim, xrange = 0.025, col = rep(c("blue3","grey","firebrick2","forestgreen","chocolate4","darkorchid4","blue","darkred"),10),
	pts = T, rug.add=TRUE, leg.pos = "topright", leg.just = c(0, 1), ...) 
	{
    require(lattice)
    if (one.abt <- any(class(x)=="single")) {
        if (missing(n.trees)) {
            utype = c("best", "all")
            use <- utype[pmatch(use, utype)]
            if (use == "best") 
                n.trees <- x$best
            else n.trees <- x$n.trees
            if (is.null(n.trees)) 
                n.trees <- x$n.trees
        }
        se <- FALSE
    }
    else {
        tree.list <- x
        nlist <- length(tree.list)
        x <- tree.list[[1]]
        if (missing(n.trees)) {
            utype <- c("best", "all")
            use <- utype[pmatch(use, utype)]
            if (use == "best") 
                n.trees <- rep(x$best, nlist)
            else n.trees <- rep(x$n.trees, nlist)
        }
        else if (length(n.trees)==1) n.trees <- rep(n.trees, nlist)
    }
    if (!is.numeric(i.var)) {
        i.var <- unique(match(i.var, x$var.names))
    }
    i.var <- i.var[order(x$var.type[i.var] > 0)]
    if ((min(i.var) < 1) || (max(i.var) > length(x$var.names))) {
        warning("i.var must be between 1 and ", length(x$var.names))
    }
    if (length(i.var) > 3) {
        warning("plot.abt creates up to 3-way interaction plots.\n
        plot.abt will only return the plotting data structure.")
        return.grid = TRUE
    }
    se.sm <- c("smooth", "mean", "none")
    se.smooth <- se.sm[pmatch(se.smooth, c("smooth", "mean", "none"))]
	if (all(is.na(se.smooth))) warning("se.smooth not recognised!\n")
    pty <- c("contour", "persp", "lines", "trellis")
    ptype <- pty[pmatch(ptype, pty)]
	if (all(is.na(ptype))) cat("ptype not recognised!\n")
    ty = c("link", "response")
    type <- ty[pmatch(type, ty)]
	if (all(is.na(type))) warning("type not recognised!\n")
	if (length(lty)<5) lty <- rep(lty,5)
	if (length(col)<5) col <- rep(col,5)
    distrib <- x$distribution$name
    if (missing(npts)) {
        if ((ptype == "contour" | ptype == "persp") & (length(i.var) <  3)) 
            npts <- rep(50, length(i.var))
        else npts <- c(50, rep(5, length(i.var) - 1))
    }
    if (length(trim.grid) == 1) 
        trim.grid <- rep(trim.grid, length(i.var))
 	    grid.levels <- vector("list", length(i.var))
    for (i in 1:length(i.var)) {
        if (is.numeric(x$var.levels[[i.var[i]]])) {
            grid.levels[[i]] <- seq(min(x$var.levels[[i.var[i]]]), 
                max(x$var.levels[[i.var[i]]]), length = npts[i])
            if (trim.grid[i]) 
                grid.levels[[i]] <- grid.levels[[i]][-c(1, npts[i])]
        }
        else {
            grid.levels[[i]] <- as.numeric(factor(x$var.levels[[i.var[i]]], 
                levels = x$var.levels[[i.var[i]]])) - 1
        }
    }
    X <- expand.grid(grid.levels)
    names(X) <- paste("x", 1:length(i.var), sep = "")
    x.num <- (x$var.type == 0)[i.var]

	if (clip && (length(i.var) > 1) && (sum(x.num) == 1)) {
        sub <- function(x, y) x[(x[, 1] > y[1] & x[, 1] < y[2]), 
            , drop = F]
        fv <- (x$var.type[i.var] > 0)
        nv <- (x$var.type[i.var] == 0)
        fact <- x$var.names[i.var][fv]
        numb <- x$var.names[i.var][nv]
        res <- lapply(split(x$mdata[, numb, drop = F], x$mdata[, 
            fact, drop = F]), range)
        Xs <- split(X, X[, !x.num, drop = F])
        for (i in 1:length(Xs)) Xs[[i]] <- sub(Xs[[i]], res[[i]])
        X <- do.call("rbind", Xs)
        row.names(X) <- 1:nrow(X)
    }
    else if (clip && (length(i.var) == 2) && all(x.num)) {
		xpdbox <- function(x, delta = 0.1) {
            x + delta * t(t(x) - apply(x, 2, mean))
        }
        poly.in <- function(xy, poly) {
            if (ncol(poly) == 2) 
                poly <- poly.tst(poly)
            n <- nrow(xy)
            np <- nrow(poly)
            nnp <- rep(n, np)
            check1 <- xor(xy[, 1] >= rep(poly[, 1], nnp), xy[, 
                1] > rep(poly[, 3], nnp))
            check2 <- rep(poly[, 2], nnp) + rep(poly[, 4], nnp) * 
                xy[, 1] > xy[, 2]
            as.vector(matrix(check1 & check2, n, np) %*% rep.int(1, 
                np)%%2 > 0)
        }
        poly.tst <- function(xy) {
            if (is.list(xy)) 
                xy <- as.data.frame(xy)
            if (!is.matrix(xy) || ncol(xy) != 2 || !is.numeric(xy[, 
                1]) || !is.numeric(xy[, 2])) 
                stop("xy must by nX2 numeric matrix or data.frame")
            xy <- as.matrix(xy)
            poly <- matrix(c(xy, xy[c(2:nrow(xy), 1), ]), ncol = 4)
            poly <- poly[poly[, 1] != poly[, 3], ]
            poly[, 4] <- (poly[, 4] - poly[, 2])/(poly[, 3] - 
                poly[, 1])
            poly[, 2] <- poly[, 2] - poly[, 1] * poly[, 4]
            poly
        }
        XX <- x$mdata[, x$var.names[i.var][1:2]]
        XX <- apply(XX, 2, function(x) {
            x[is.na(x)] <- mean(x, na.rm = T)
            x
        })
        hpts <- chull(XX)
        XXP <- xpdbox(XX[c(hpts, hpts[1]), ])
        ins <- poly.in(as.matrix(X), XXP)
    }
	else if (pts && (length(i.var) > 1) && (sum(x.num) == 2)) {
        XX <- x$mdata[, x$var.names[i.var][1:2]]
	}
    
	if (one.abt) {
        y <- .Call("gbm_plot", X = as.double(data.matrix(X)), 
            cRows = as.integer(nrow(X)), cCols = as.integer(ncol(X)), 
            n.class = as.integer(x$num.classes),
            i.var = as.integer(i.var - 1), n.trees = as.integer(n.trees), 
            initF = as.double(x$initF), trees = x$trees, c.splits = x$c.splits, 
            var.type = as.integer(x$var.type), PACKAGE = "abt")
    }
    else if (!se) {
        ytemp <- 0
        for (i in 1:nlist) {
            ytemp <- ytemp + .Call("gbm_plot", X = as.double(data.matrix(X)), 
                cRows = as.integer(nrow(X)), cCols = as.integer(ncol(X)), 
                n.class = as.integer(x$num.classes),
                i.var = as.integer(i.var - 1), n.trees = as.integer(n.trees[i]), 
                initF = as.double(tree.list[[i]]$initF), trees = tree.list[[i]]$trees, 
                c.splits = tree.list[[i]]$c.splits, var.type = as.integer(x$var.type), 
                PACKAGE = "abt")
        }
        y <- ytemp/nlist
    }
    else {
        ytemp <- matrix(NA, nrow = nrow(X), ncol = nlist)
        for (i in 1:nlist) {
            ytemp[, i] <- .Call("gbm_plot", X = as.double(data.matrix(X)), 
                cRows = as.integer(nrow(X)), cCols = as.integer(ncol(X)), 
                n.class = as.integer(x$num.classes),
                i.var = as.integer(i.var - 1), n.trees = as.integer(n.trees[i]), 
                initF = as.double(tree.list[[i]]$initF), trees = tree.list[[i]]$trees, 
                c.splits = tree.list[[i]]$c.splits, var.type = as.integer(x$var.type), 
                PACKAGE = "abt")
        }
        y <- apply(ytemp, 1, mean, na.rm = T)
        y.se <- apply(ytemp, 1, var, na.rm = T)
        if (length(i.var) == 1 && is.numeric(x$var.levels[[i.var]])) {
            if (se.smooth == "smooth") 
                y.se <- runmed(y.se,k=1 + 2 * min((npts-1)%/% 2, ceiling(0.1*npts)))
			else if (se.smooth == "mean") 
                y.se <- rep(mean(y.se), length(y.se))
            if (!one.abt) 
                y.se <- y.se * (nlist - 1)
        }
        y.se <- sqrt(y.se)
    }


   if (x$distribution$name=="multinomial")  {
      ## Put result into matrix form
      X$y <- matrix(y, ncol = x$num.classes)
      colnames(X$y) <- x$classes
      if (se) {
		  X$se <- matrix(y.se, ncol = x$num.classes)
		  colnames(X$se) <- x$classes
		  X$hi <- matrix(X$y + se * X$se, ncol = x$num.classes)
		  colnames(X$hi) <- x$classes
		  X$lo <- matrix(X$y - se * X$se, ncol = x$num.classes)
		  colnames(X$lo) <- x$classes
	  }
      ## Use class probabilities
      if (type=="response") {
         X$y <- exp(X$y)
         X$y <- X$y / matrix(rowSums(X$y), ncol=ncol(X$y), nrow=nrow(X$y))
      }
   }
   else {
   	X$y <- y
   	if (se) {
	   	X$se <- y.se
		X$hi <- X$y + se * X$se
		X$lo <- X$y - se * X$se
	}
   if (distrib == "bernoulli" && type == "response") {
        X$y <- 1/(1 + exp(-X$y))
		    if (se) {
				X$hi <- 1/(1 + exp(-X$hi))
				X$lo <- 1/(1 + exp(-X$lo))
		    }
	    }
   else if (distrib == "poisson" && type == "response") {
        X$y <- exp(X$y)
		    if (se) {
				X$hi <- exp(X$hi)
				X$lo <- exp(X$lo)
		    }
	    }
	}

	if (center) {
        if (missing(cvalue)) cvalue  <- cfun(X$y,na.rm=T)	
        X$y <- X$y - cvalue         
	    if (se) {
		    X$hi <- X$hi - cvalue
	        X$lo <- X$lo - cvalue
        }    
	}
    if (clip && (length(i.var) == 2) && all(x.num)) 
        X$y[!ins] <- NA


    f.factor <- rep(FALSE, length(i.var))
    for (i in 1:length(i.var)) {
        if (!is.numeric(x$var.levels[[i.var[i]]])) {
            X[, i] <- factor(x$var.levels[[i.var[i]]][X[, i] + 
                1], levels = x$var.levels[[i.var[i]]])
            f.factor[i] <- TRUE
        }
    }

    if (return.grid) {
        names(X)[1:length(i.var)] <- x$var.names[i.var]
        if (center) attr(X,"cent") <- cvalue
        return(X)
    }

    nrows <- nrow(x$mdata)
    if (missing(ylim)) {
        if (se) 
            ylim <- c(min(X$lo, na.rm = T), max(X$hi, na.rm = T))
        else ylim <- range(X$y, na.rm = T)
        if (xrange) {
            rnge <- diff(ylim)
            ylim[1] <- ylim[1] - xrange * rnge
            ylim[2] <- ylim[2] + xrange * rnge
        }
    }
    if (length(i.var) == 1) {
        if (missing(xlab)) 
            xlab <- x$var.names[i.var]
        if (missing(ylab)) 
            ylab <- paste("f(", x$var.names[i.var], ")", sep = "")
        if (!f.factor) {
         if (x$distribution$name == "multinomial") {
                if ( type == "response" ){
         ylabel <- "Predicted class probability"
	     }
	     else { ylabel <- paste("f(",x$var.names[i.var],")",sep="") }
           plot(range(X$x1), range(X$y), type = "n", xlab = x$var.names[i.var], 
                ylab = ylabel)
   
           for (ii in 1:x$num.classes){
              lines(X$x1,X$y[,ii],
                     xlab=x$var.names[i.var],
                     ylab=paste("f(",x$var.names[i.var],")",sep=""),
                     col = ii, ...)
            }
         }
		else {
            plot(X$x1, X$y, type = "n", ylim = ylim, xlab = xlab, 
                ylab = ylab, ...)
            if (se) {
                if (!se.fill) {
                  lines(X$x1, X$hi, col = col[2], lty = lty[2])
                  lines(X$x1, X$lo, col = col[2], lty = lty[2])
                }
                else polygon(c(X$x1, rev(X$x1)), c(X$hi, rev(X$lo)), border = NA, 
                col=col[2])
            }
            if (add.zero) abline(h=0,lty=lty[2],col=col[2])
            lines(X$x1, X$y, col = col[1], ...)
            if(rug.add) rug(x$var.levels[[i.var]])
        }
       }
       else {
   		  if (x$distribution$name == "multinomial") {
	      nX <- length(X$x1)
	      dim.y <- dim(X$y)
  		  if (type == "response" ) {
    	  ylabel <- "Predicted probability"
          }
	      else{ ylabel <- paste("f(",x$var.names[i.var],")",sep="") }
      
	      plot(c(0,nX), range(X$y), axes = FALSE, type = "n", 
                xlab = x$var.names[i.var], ylab = ylabel)
          axis(side = 1, labels = FALSE, at = 0:nX)
          axis(side = 2)
          mtext(as.character(X$x1), side = 1, at = 1:nX - 0.5, cex=par()$cex)
          segments(rep(1:nX - 0.75, each = dim.y[2]), as.vector(t(X$y)), 
          rep(1:nX - 0.25, each=dim.y[2]), as.vector(t(X$y)), col = 1:dim.y[2])         
          box()
          }
       else {
          if (!se) {
            barplot(X$y, names = x$var.levels[[i.var[i]]], 
              ylim = ylim, xlab = xlab, ylab = ylab, col = col[2], 
              xpd = FALSE, ...)
            abline(h = 0)
            box()
          }
          else {
            yerr <- (X$hi - X$y) * ifelse(X$y < 0, -1, 
              1)
            z <- barplot(X$y, names = x$var.levels[[i.var[i]]], 
              ylim = ylim, xlab = xlab, ylab = ylab, col = col[2], 
              xpd = FALSE, ...)
            abline(h = 0)
            segments(z, X$y, z, X$y + yerr)
            zerr <- 0.35 * (z[2] - z[1])
            segments(z + zerr, X$y + yerr, z - zerr, X$y + yerr)
            box()
          }
        }
      }
    }
    else if (length(i.var) == 2) {
        if (!f.factor[1] && !f.factor[2]) {
            cp <- (ptype[1] == "persp") | (ptype[1] == "contour")
            if (missing(xlab)) 
                xlab <- x$var.names[i.var[1]]
            if (missing(ylab) & cp) 
                ylab <- x$var.names[i.var[2]]
            if (missing(ylab) & !cp) 
                ylab <- paste("f(", x$var.names[i.var[1]], ",", 
                  x$var.names[i.var[2]], ")", sep = "")
            if (ptype[1] == "contour") {
                z <- matrix(X$y, ncol = length(grid.levels[[2]]))
                image(grid.levels[[1]], grid.levels[[2]], z, 
                  xlab = xlab, ylab = ylab, zlim = range(z, na.rm = T), 
                  xaxs = "r", yaxs = "r", col = fpcol, ...)
                contour(grid.levels[[1]], grid.levels[[2]], z, 
                  zlim = range(z, na.rm = T), add = T, col = col[2], 
                  ...)
                if (pts) 
                  points(XX)
            }
            else if (ptype[1] == "persp") {
                z <- matrix(X$y, ncol = length(grid.levels[[2]]))
                if (pcol == "vary") 
                  pcol <- gcols(z, FUNCOL = fpcol)
                persp(grid.levels[[1]], grid.levels[[2]], z, 
                  zlab = zlab, xlab = xlab, ylab = ylab, zlim = range(z, 
                    na.rm = T), tick = "detailed", col = pcol, 
                  theta = theta, d = d, ...)
            }
            else if (ptype[1] == "lines") {
                X.lst <- split(X, X$x2)
                plot(X$x1, X$y, type = "n", xlab = xlab, ylab = ylab, 
                  ylim = ylim, ...)
                for (i in 1:length(X.lst)) lines(X.lst[[i]]$x1, 
                  X.lst[[i]]$y, col = col[i], lty=lty[i],...)
                if (add.legend) {
                  legend(leg.pos, as.character(signif(unique(sort(X$x2)),2)),
                   lty = lty, xjust = leg.just[1], yjust = leg.just[2],
                    col = col, cex = 0.8, bty = "n", title=x$var.names[i.var[2]])
                }
            }
            else if (ptype[1] == "trellis") 
                if (!se) {
                  print(xyplot(y ~ x1 | x2, data = X, xlab = xlab, 
                    ylab = ylab, type = "l", ylim = ylim, col = col[1], ...))
                }
                else {
                  X$yhi <- X$y + se * X$se
                  X$ylo <- X$y - se * X$se
                  print(xyplot(y + yhi + ylo ~ x1 | x2, data = X, type = "l", 
                    allow.mult = T, xlab = xlab, ylab = ylab, 
                    col = c(col[1], col[2], col[2]), lty = lty, ylim = ylim, ...))
                }
        }
        else if (f.factor[1] && !f.factor[2]) {
            if (missing(xlab)) 
                xlab <- x$var.names[i.var[2]]
            if (missing(ylab)) 
                ylab <- paste("f(", x$var.names[i.var[2]], "|", 
                  x$var.names[i.var[1]], ")", sep = "")
            if (ptype[1] == "lines") {
                X.lst <- split(X, X$x1)
                plot(X$x2, X$y, type = "n", xlab = xlab, ylab = ylab, 
                  ylim = ylim, ...)
                for (i in 1:length(X.lst)) lines(X.lst[[i]]$x2, 
                  X.lst[[i]]$y, col = col[i], lty=lty[i],...)
                if (add.legend) 
                  legend(leg.pos, levels(X$x1), lty = lty, 
                  col = col, cex = 0.8, xjust = leg.just[1],
				  yjust = leg.just[2], bty = "n")
            }
            else if (!se) 
                print(xyplot(y ~ x2 | x1, data = X, xlab = xlab, ylab = ylab, 
                  type = "l", col = col[1], ylim = ylim, ...))
            else {
                X$yhi <- X$y + se * X$se
                X$ylo <- X$y - se * X$se
                print(xyplot(y + yhi + ylo ~ x2 | x1, data = X, type = "l", 
                  allow.mult = T, xlab = xlab, ylab = ylab, col = c(col[1], 
                    col[2], col[2]), lty = 1, ylim = ylim, ...))
            }
        }
        else if (!f.factor[1] && f.factor[2]) {

    	if (missing(xlab)) 
                xlab <- x$var.names[i.var[1]]
            if (missing(ylab)) 
                ylab <- paste("f(", x$var.names[i.var[1]], "|", 
                  x$var.names[i.var[2]], ")", sep = "")
            if (ptype[1] == "lines") {
                X.lst <- split(X, X$x2)
                plot(X$x1, X$y, type = "n", xlab = xlab, ylab = ylab, 
                  ylim = ylim, ...)
                for (i in 1:length(X.lst)) lines(X.lst[[i]]$x1, 
                  X.lst[[i]]$y, col = col[i], lty=lty[i])
                if (add.legend) 
                  legend(leg.pos, levels(X$x2), lty = lty, 
                  col = col, cex = 0.8,  xjust = leg.just[1],
				  yjust = leg.just[2], bty = "n")
            }
            else if (!se) 
                print(xyplot(y ~ x1 | x2, data = X, xlab = xlab, ylab = ylab, 
                  type = "l", col = col[1], ylim = ylim, ...))
            else {
                X$yhi <- X$y + se * X$se
                X$ylo <- X$y - se * X$se
		        print(xyplot(y + yhi + ylo ~ x1 | x2, data = X, type = "l", 
                  allow.mult = T, xlab = xlab, ylab = ylab, col = c(col[1], 
                    col[2], col[2]), lty = lty, ylim = ylim, ...))
            }
        }
        else {
            if (missing(xlab)) 
                xlab <- x$var.names[i.var[i[1]]]
            if (missing(ylab)) 
                ylab <- paste("f(", paste(x$var.names[i.var[1:3]], 
                  collapse = ","), ")", sep = "")
            if (ptype[1] == "trellis") 
                barchart(x1 ~ y | x2, data = X, ylab = paste("f(~", 
                  x$var.names[i.var[1]], "*", x$var.names[i.var[2]], 
                  ")", sep = ""), orig = 0, ...)
            else {
                if (!se) {
                  barplot(X$y, names = outer(substring(x$var.levels[[i.var[1]]], 
                    1, 2), substring(x$var.levels[[i.var[2]]], 
                    1, 2), FUN = paste, sep = "\n"), ylim = ylim, 
                    xlab = xlab, ylab = ylab, col = col, xpd = FALSE, ...)
                  abline(h = 0)
                  box()
                }
                else {
                  yerr <- X$y + se * X$se * ifelse(X$y < 0, -1, 
                    1)
                  z <- barplot(X$y, names = outer(substring(x$var.levels[[i.var[1]]], 
                    1, 2), substring(x$var.levels[[i.var[2]]], 
                    1, 2), FUN = paste, sep = "\n"), ylim = ylim, 
                    xlab = xlab, ylab = ylab, 
                    col = rainbow(n = length(unlist(x$var.levels[i.var[1]]))), 
                    xpd = FALSE, ...)
                  abline(h = 0)
                  segments(z, X$y, z, yerr)
                  zerr <- 0.35 * (z[2] - z[1])
                  segments(z + zerr, yerr, z - zerr, yerr)
                  box()
                }
            }
        }
    }
    else if (length(i.var) == 3) {
        if (missing(xlab)) 
            xlab <- x$var.names[i.var[1]]
        if (missing(ylab)) 
            ylab <- paste("f(", paste(x$var.names[i.var[1:3]], 
                collapse = ","), ")", sep = "")
        if (sum(f.factor) < 3) {
            if (!se) {
                print(xyplot(y ~ x1 | x2 * x3, data = X, xlab = xlab, 
                  ylab = ylab, type = "l", ylim = ylim, ...))
            }
            else {
                X$yhi <- X$y + se * X$se
                X$ylo <- X$y - se * X$se
                print(xyplot(y + yhi + ylo ~ x1 | x2 * x3, data = X, 
                  type = "l", allow.mult = T, xlab = xlab, col = c(1, 
                    2, 2), lty = lty, ylab = ylab, ylim = ylim, 
                  ...))
            }
        }
        else if (sum(f.factor) == 3) {
            barchart(x1 ~ y | x2 * x3, data = X, xlab = xlab, 
                ylab = ylab, origin = 0, ...)
        }
    }
	if (center) invisible(cvalue)
}

