\name{relative.influence}
\alias{relative.influence}
\alias{permutation.test.abt}
\alias{abt.loss}
\title{ Methods for estimating relative influence }
\description{
Helper functions for computing the relative influence of each variable in the abt object.
}
\usage{
relative.influence(object, n.trees, scale., sort.)
permutation.test.abt(object, n.trees)
abt.loss(y,f,w,offset,dist,baseline)
}
\arguments{
\item{object}{a \code{abt} object created from an initial call to \code{\link{abt}}.}
\item{n.trees}{ the number of trees to use for computations. If not provided, the
    the function will guess: if a test set was used in fitting, the number of
    trees resulting in lowest test set error will be used; otherwise, if
    cross-validation was performed, the number of trees resulting in lowest
    cross-validation error will be used; otherwise, all trees will be used.}
\item{scale.}{ whether or not the result should be scaled. Defaults to \code{FALSE}.}
\item{sort.}{ whether or not the results should be (reverse) sorted.
         Defaults to \code{FALSE}.}
\item{y,f,w,offset,dist,baseline}{For \code{abt.loss}: These components are the
outcome, predicted value, observation weight, offset, distribution, and comparison
loss function, respectively.}
}
\details{
This is not intended for end-user use. These functions offer the different
methods for computing the relative influence in \code{\link{summary.abt}}.
\code{abt.loss} is a helper function for \code{permutation.test.abt}.
}
\value{
By default, returns an unprocessed vector of estimated relative influences.
If the \code{scale.} and \code{sort.} arguments are used, returns a processed
version of the same.
}
\references{
J.H. Friedman (2001). "Greedy Function Approximation: A Gradient Boosting
Machine," Annals of Statistics 29(5):1189-1232.

L. Breiman (2001). "Random Forests," Available at \url{ftp://ftp.stat.berkeley.edu/pub/users/breiman/randomforest2001.pdf}.

}
\author{Greg Ridgeway \email{gregr@rand.org}}

\seealso{ \code{\link{summary.abt}} }

\keyword{ hplot }
