"ganova.list" <-
function (object, ..., scale = 0, test = c("F","Chi","Cp","IC","ICdev"))
{
    objects <- list(object, ...)
    responses <- as.character(lapply(objects, function(x) deparse(x$terms[[2]])))
    sameresp <- responses == responses[1]
    if (!all(sameresp)) {
        objects <- objects[sameresp]
        warning("Models with response ", deparse(responses[!sameresp]),
            " removed because response differs from ", "model 1")
    }
    ns <- sapply(objects, function(x) length(x$residuals))
    if (any(ns != ns[1]))
        stop("models were not all fitted to the same size of dataset")
    nmodels <- length(objects)
    test <- test[1]
    if (nmodels == 1)
        return(ganova.lm(object))
    resdf <- as.numeric(lapply(objects, df.residual))
    resdev <- as.numeric(lapply(objects, deviance))
    table <- data.frame(resdf, resdev, c(NA, -diff(resdf)), c(NA,
        -diff(resdev)))
    variables <- lapply(objects, function(x) paste(deparse(formula(x)),
        collapse = "\n"))
    dimnames(table) <- list(1:nmodels, c("Dev Df", "Deviance", "Diff Df",
        "Diff Dev"))
    title <- "Analysis of Variance Table"
    topnote <- paste("Model ", format(1:nmodels), ": ", variables,
        sep = "", collapse = "\n")
    attr(table,"n") <- length(objects[[1]]$residuals)
    if (!is.null(test)) {
        bigmodel <- order(resdf)[1]
        scale <- if (scale > 0)
            scale
        else resdev[bigmodel]/resdf[bigmodel]
        table <- stat.ganova(table = table, test = test, scale = scale,
            df.scale = resdf[bigmodel], n = length(objects[[1]]$residuals))
    }
    structure(table, heading = c(title, topnote), class = c("anova",
        "data.frame"))
}

