"gplot.numeric" <-
function(x, y, se.y = NULL, xlab, ylab, residuals = NULL, rugplot = F, scale = 0, se = F, xlim = NULL, ylim = NULL, fit = T,
    nse = 2, ...)
{
    if(length(x) != length(y))
        stop("x and y do not have the same length; possibly a consequence of an na.action")
    ux <- unique(sort(x))
    o <- match(ux, x)
    uy <- y[o]
    xlim <- range(xlim, ux)
    ylim <- range(ylim, uy)
    if(rugplot) {
        jx <- jitter(x[!is.na(x)])
        xlim <- range(c(xlim, jx))
    }
    if(se && !is.null(se.y)) {
        se.upper <- uy + nse * se.y[o]
        se.lower <- uy - nse * se.y[o]
        ylim <- range(c(ylim, se.upper, se.lower))
    }
    if(!is.null(residuals)) {
        if(length(residuals) == length(y)) {
            residuals <- y + residuals
            ylim <- range(c(ylim, residuals))
        }
        else {
            residuals <- NULL
            warning(paste("Residuals do not match x in \"", ylab, "\" preplot object", sep = ""))
        }
    }
    ylim <- ylim.scale(ylim, scale)
    if(!is.null(residuals)) {
        plot(x, residuals, type = "n", xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab, ...)
        if(se) {
            polygon(c(ux, rev(ux)), c(se.upper, rev(se.lower)), dens = -1, col = 8)
        }
        points(x, residuals, ...)
        if(fit)
            lines(ux, uy, ...)
    }
    else {
        plot(ux, uy, type = "n", xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab, ...)
        if(se) {
            polygon(c(ux, rev(ux)), c(se.upper, rev(se.lower)), dens = -1, col = 8)
        }
        if(fit)
            lines(ux, uy, ...)
    }
    if(rugplot)
        rug(jx)
    invisible(diff(ylim))
}

