"model.cv" <-
function (formula = formula(data), data = list(), FUN = lm,
    seed = 0, ngrp = 5, grp.col = 0, silent = TRUE, type = c("mse",
        "missclass"), ...)
{
    cv.var <- function(x, cv = 10) {
        x <- number(x)
        maxx <- max(x)
        if (maxx >= cv) {
            x <- match(x, sample(maxx))
            grps <- ceiling((cv * cumsum(table(x)))/length(x))
            x <- number(grps[x])
        }
        x
    }
    lst <- list()
    nrows <- nrow(data)
    if (seed > 0)
        set.seed(seed)
    if (grp.col == 0)
        nsamps <- sample(rep(1:ngrp, length = nrows))
    else {
        nsamps <- cv.var(data[, grp.col], ngrp)
        if (!silent)
            print(table(nsamps, data[, grp.col]))
    }
    preds <- rep(NA, length = nrows)
    for (i in 1:ngrp) {
        nTrain <- length(trainees <- (1:nrows)[nsamps != i])
        non.trainees <- (1:nrows)[is.na(match(1:nrows, trainees))]
        if (grp.col == 0) {
            temp.data <- data[trainees, ]
            lst[[i]] <- FUN(formula, data = temp.data, ...)
        }
        else {
            temp.data <- data[trainees, -grp.col]
            lst[[i]] <- FUN(formula, data = temp.data, ...)
        }
        preds[non.trainees] <- predict(lst[[i]], newdata = data[non.trainees,
            ])
    }
    lst[[1]]$nsamps <- nsamps
    lst[[1]]$preds <- preds
    resp <- eval(formula[[2]],data)
    if (type[1] == "mse")
         lst[[1]]$pmse <- mean((resp - lst[[1]]$preds)^2,
                na.rm = T)
    else {
            tab <- table(resp, lst[[1]]$preds > 0)
            lst[[1]]$pmse <- 1 - sum(diag(tab))/sum(tab)
    }
    lst
}

