"model.cv.ss" <-
function(formula = formula(data), data = list(), FUN = lm, seed = 0,
    ngrp = 5, grp.col = 0, ret.preds = FALSE, silent = TRUE, prn=FALSE, ...)
    {
    cv.var <- function(x, cv = 10)
    {
        x <- number(x)
        maxx <- max(x)
        if(maxx >= cv) {
            x <- match(x, sample(maxx))
            grps <- ceiling((cv * cumsum(table(x)))/length(x))
            x <- number(grps[x])
        }
        x
    }
    fit <- FUN(formula, data, y = TRUE)
    if (prn) print(anova(fit))
    y <- fit$y
    nrows <- nrow(data)
    if(seed > 0)
        set.seed(seed)
    if(grp.col == 0)
        nsamps <- sample(rep(1:ngrp, length = nrows))
    else {
        nsamps <- cv.var(data[, grp.col], ngrp)
        if(!silent)
            print(table(nsamps, data[, grp.col]))
    }
    preds <- rep(NA, length = nrows)
    for(i in 1:ngrp) {
        nTrain <- length(trainees <- (1:nrows)[nsamps != i])
        non.trainees <- (1:nrows)[is.na(match(1:nrows, trainees))]
        if(grp.col == 0) {
            temp.data <- data[trainees,  ]
            lst <- FUN(formula, data = temp.data, singular=TRUE)
        }
        else {
            temp.data <- data[trainees,  - grp.col]
            lst <- FUN(formula, data = temp.data, singular=TRUE)
        }
        preds[non.trainees] <- predict(lst, newdata = data[non.trainees,  ])
    }
    perr<-mean((y-preds)^2)
    if (prn) cat("Mean square predicted error = ",signif(perr,4),"\n")
    if (ret.preds) data.frame(y,preds) else perr
}

