mvdata.sim <- 
function (n = 1000, nvar = 4, mu = 10, Sig = runif(length(nvar)^2), 
    degree = 4, coefs = seq(1, 0.1, length = degree), scale = T, 
    err = 0.1, noise.vars = 0, b.scale = 1, binary = FALSE) 
{
    if (length(mu) == 1) 
        mu <- rep(mu, nvar)
    Sig <- matrix(Sig, ncol = nvar, nrow = nvar)
    diag(Sig) <- rep(1, nvar)
    x <- mvrunif(n, mu, Sig)
    x <- poly(x, degree = degree)
    deg <- attr(x, "degree")
    if (scale) 
        x <- scale(x)
    y <- x %*% (runif(length(deg), -1, 1) * coefs[deg])
    x <- x[, deg == 1, drop = F]
    if (noise.vars) 
        x <- cbind(x, matrix(rnorm(noise.vars * n), nrow = n))
    colnames(x) <- paste("x", 1:(nvar + noise.vars), sep = "")
    if (binary) {
        y <- y/sd(y) * b.scale
        ye <- rbinom(length(y), 1, 1/(1 + exp(-y)))
        data.frame(ye, y, x)
    }
    else {
        if (err) {
            err <- 1/(1/err - 1)
            y <- y/(sqrt(err) * sqrt(var(drop(y))))
            ye <- y + rnorm(n)
            data.frame(ye, y, x)
        }
        else data.frame(y, x)
    }
}
