`pintol` <-
function (formula, data, outer = TRUE, pts = T, xlab, ylab, log = "", 
    amany = 4, bmany = 4, reset.par = TRUE, bolap = 0.25, rc, nint = 50,
    omar = c(5, 5, 1, 1), mgap = 3, texts, col = rep(2:16,10), 
	cext = 1, cexl = 1, cexm = rep(1, length(y)), cexbub = 1, 
    bg = rep(2:16,10), delta = 0.2, reg = F, regz = F, 
	regtype = c("glm", "loess", "gam")[1], cvreg = T, se = 0, ci.cov = 0.95, 
    df = 1, minpts = 4, lo.span = 0.75, lo.degree = 1, lin = F, 
    linz = F, means = F, test.type = c("F", "Chi", "Cp")[1], 
    xaxis.all = F, yaxis.all = F, meansz = F, grid = F, ngrid = 4, 
    hline, vline, line = F, leg = F, leg.auto = F, leg.loc = c(0,1), 
	pch = 1, main = "", bty = "o", xlim, ylim, xlim.free = FALSE, 
    ylim.free = FALSE, ylim.adj = c(0, 0), xlim.adj = c(0, 0), 
    ylim.wide = 1, xlim.wide = 1, plot.gap = c(0, 0), lwd = 1, 
    lwdf = 1, jit = F, jit.factor = 1, ucol = T, ulinecol = 11, 
    err = F, hi, lo, use.spread = F, spread = 0, gap = 1, bar.ends = 1, 
    fit.linear = F, fit.smooth = F, ret.fit = F, diag.fit = F, head, 
    ...) 
{

	require(splines)
	require(mgcv)

	sem <- function (y, na.rm = TRUE) {
		if (na.rm) 
		    y <- y[!is.na(y)]
		if (length(y)) 
		    sqrt(var(y)/length(y))
	    else NA
	}

    mean.na <- function(x) mean(x, na.rm=T)

    xpand <- function(x, delta = 0.05) {
        xrnge <- range(as.numeric(x))
        xdiff <- delta * diff(xrnge)
        xrnge + xdiff * c(-1, 1)
    }

    deparen <- function(expr) {
        while (is.language(expr) && !is.name(expr) && deparse(expr[[1]]) == 
            "(") expr <- expr[[2]]
        expr
    }

	equal.count <- function(x, ...) {
		attr(x, "levels") <- make.list.from.intervals(co.intervals(x,  ...))
		class(attr(x, "levels")) <- "shingleLevel"
		class(x) <- "shingle"
		x
	}
	make.list.from.intervals <- function (x) {
		if (ncol(x) != 2) 
		    stop("x must be matrix with 2 columns")
		ans <- vector(mode = "list", length = nrow(x))
		for (i in seq_len(nrow(x))) ans[[i]] <- x[i, ]
    ans
	}

    bad.formula <- function() stop("invalid conditioning formula")
    bad.lengths <- function() stop("incompatible variable lengths")

    old.par <- par(no.readonly = TRUE)
    if (reset.par) 
        on.exit(par(old.par))

    if (is.null(dev.list())) {
		if (.Platform$OS.type == "unix") x11()
		else windows()
	}
    formula <- deparen(formula)
    if (!inherits(formula, "formula")) 
        bad.formula()
    y <- deparen(formula[[2]])
    rhs <- deparen(formula[[3]])
    if (length(rhs) == 1) {
        x <- deparen(rhs)
        have.a <- have.b <- FALSE
    }
    else {
        x <- deparen(rhs[[2]])
        rhs <- deparen(rhs[[3]])
        if (is.language(rhs) && !is.name(rhs) && (deparse(rhs[[1]]) == 
            "*" || deparse(rhs[[1]]) == "+")) {
            have.b <- TRUE
            have.a <- TRUE
            a <- deparen(rhs[[2]])
            b <- deparen(rhs[[3]])
        }
        else if (outer) {
            have.a <- FALSE
            have.b <- TRUE
            b <- rhs
        }
        else {
            have.b <- FALSE
            have.a <- TRUE
            a <- rhs
        }
    }
    if (missing(data)) 
        data <- parent.frame()
    x.name <- deparse(x)
    x <- eval(x, data, parent.frame())
    nobs <- length(x)
    y.name <- deparse(y)
    y <- eval(y, data, parent.frame())
    if (length(y) != nobs) 
        bad.lengths()
    if (have.a) {
        a.name <- deparse(a)
        a <- eval(a, data, parent.frame())
        if (length(a) != nobs) 
            bad.lengths()
        if (is.character(a)) 
            a <- as.factor(a)
    }
    if (have.b) {
        b.name <- deparse(b)
        b <- eval(b, data, parent.frame())
        if (length(b) != nobs) 
            bad.lengths()
        if (is.character(b)) 
            b <- as.factor(b)
        if (have.a) 
            missingrows <- which(is.na(x) | is.na(y) | is.na(a) | 
                is.na(b))
        else {
            missingrows <- which(is.na(x) | is.na(y) | is.na(b))
            a <- NULL
            a.name <- ""
        }
    }
    else if (have.a) {
        missingrows <- which(is.na(x) | is.na(y) | is.na(a))
        b <- NULL
        b.name <- ""
    }
    else {
        missingrows <- which(is.na(x) | is.na(y))
        a <- NULL
        a.name <- ""
        b <- NULL
        b.name <- ""
    }
    if (is.character(y)) 
        y <- as.factor(y)
    if (is.character(x)) 
        x <- as.factor(x)
    if (delta <= 0) {
        delta <- 0.1
        cat("\n delta must be > 0 : delta set to 0.1\n\n")
    }
    if (delta > 1) {
        delta <- 0.5
        cat("\n delta must be <= 1 : delta set to 0.5\n\n")
    }
    rtype <- c("gam", "glm", "loess")
    regtype <- rtype[pmatch(regtype, rtype)]
    fitted <- F
    blen <- 1
    bshing <- F
    if (have.b) {
        if (!is.factor(b) & is.numeric(b)) {
            ms <- equal.count(b, bmany, bolap)
            bshing <- T
            mainlabs <- rep("", bmany)
            for (i in 1:bmany) mainlabs[i] <- paste(b.name, " : ", 
                signif(attributes(ms)$levels[[i]][1], 3), " - ", 
                signif(attributes(ms)$levels[[i]][2], 3), sep = "")
            blen <- bmany
        }
        else {
            mainlabs <- buni <- levels(b)
            blen <- length(buni)
        }
    }
    if (length(cexm) == length(y)) 
        cexm <- sc01(cexm, cexbub, delta)
    if (length(cexm) == 1) 
        cexm <- rep(cexm, length(y))
    cexm <- cexm * par()$cex * 1.5
    cext <- cext * par()$cex
    cexl <- cexl * par()$cex
    if (have.a && (!is.factor(a)) && (is.numeric(a)) && length(unique(a)) >= 
        amany) 
        a <- cut(a, quantile(a, probs = seq(0, 1, 1/amany)), 
            include.lowest = T)
    if (outer & have.a & have.b) {
        na <- length(levels(a))
        nb <- length(levels(b))
        if (missing(rc)) 
            rc <- c(nb, na)
        b <- interaction(a, b)
        blen <- na * nb
        have.a <- FALSE
        a <- NULL
        a.name <- ""
        mainlabs <- buni <- levels(b)
    }
    if (have.b) {
        if (missing(rc)) 
            rc <- n.plt(blen)
        if (length(plot.gap) == 1) 
            plot.gap <- rep(plot.gap, 2)
        par(mfrow = rc, oma = omar, mar = c(0, 0, plot.gap[1], 
            plot.gap[2]))
        nrows <- rc[1]
        ncols <- rc[2]
    }
    else {
        nrows <- ncols <- 1
    }
    if (!is.null(a)) {
        if (missing(spread) & err) 
            spread <- 1
        lz <- length(auni <- levels(a))
        offset <- (spread * (0.5 * ((1:lz) - (lz + 1)/2)))/lz
        if (length(pch) == 1) 
            pch <- rep(pch, lz)
        if (length(col) == 1) 
            col <- rep(col, lz)
        if (length(bg) == 1) 
            bg <- rep(bg, lz)
        if (length(cexm) == 1) 
            cexm <- rep(cexm, lz)
    }
    if (missing(xlim)) 
        xlim <- xpand(x)
    if (missing(ylim)) {
        if (err & !missing(hi) & !missing(lo)) 
            ylim <- xpand(c(hi, lo))
        else if (err) 
            ylim <- xpand(c(y + err, y - err))
        else ylim <- xpand(y)
    }
    if (is.factor(x)) 
        xlim <- xlim + xlim.wide * c(-0.5, 0.5)
    if (is.factor(y)) 
        ylim <- ylim + ylim.wide * c(-0.5, 0.5)
    for (i in 1:blen) {
        if (nrows == 1 | xaxis.all) 
            row.axis <- T
        else row.axis <- ifelse(i > (nrows - 1) * ncols, T, F)
        if (ncols == 1 | yaxis.all) 
            col.axis <- T
        else col.axis <- ifelse((i%%ncols) == 1, T, F)
        if (blen != 1) {
            if (bshing) {
                limits <- attributes(ms)$levels[[i]]
                subset2 <- (b >= limits[1]) & (b <= limits[2])
                main <- mainlabs[i]
            }
            else {
                subset2 <- (b == buni[i])
                main <- mainlabs[i]
            }
        }
        else subset2 <- rep(T, length(y))
        yy <- y[subset2]
        if (length(yy) != 0) {
            if (is.factor(y)) 
                yy <- as.numeric(yy)
            xx <- x[subset2]
            if (is.factor(x)) 
                xx <- as.numeric(xx)
            cexmsub <- cexm[subset2]
            if (!is.null(a)) 
                zz <- a[subset2]
            ox <- order(xx)
            xx <- xx[ox]
            yy <- yy[ox]
            if (!missing(hi) & !missing(lo)) {
                hhi <- hi[subset2]
                llo <- lo[subset2]
                hhi <- hhi[ox]
                llo <- llo[ox]
            }
            cexmsub <- cexmsub[ox]
            if (!is.null(a)) 
                zz <- zz[ox]
            if (!is.null(a)) 
                ww <- zz
            if (xlim.free) 
                xlim <- range(as.numeric(xx)) + xlim.adj
            if (ylim.free) 
                ylim <- range(as.numeric(yy)) + ylim.adj
            plot(xx, yy, xlab = "", ylab = "", axes = F, type = "n", 
                bty = "none", cex.axis = cext, ylim = ylim, xlim = xlim, 
                log = log)
        }
        else {
            plot(0, 0, xlab = "", ylab = "", axes = F, type = "n", 
                bty = "none", cex.axis = cext, ylim = ylim, xlim = xlim, 
                log = log)
            cat("\n No data for this group \n\n")
        }
        box(bty = bty)
        if (grid) {
            if (missing(hline)) 
                hline <- pretty(ylim, ngrid)
            if (missing(vline)) 
                vline <- pretty(xlim, ngrid)
        }
        if (!missing(hline) || grid) 
            abline(h = hline, lty = 2, col = "grey")
        if (!missing(vline) || grid) 
            abline(v = vline, lty = 2, col = "grey")
        if (col.axis) {
            if (is.factor(y)) 
                axis(2, at = unique(sort(as.numeric(y))), labels = paste(levels(y), 
                  paste(rep(" ", max(nchar(levels(y)))), collapse = "")), 
                  cex.axis = cext)
            else axis(2, cex.axis = cext)
        }
        if (row.axis) {
            if (is.factor(x)) 
                axis(1, at = unique(sort(as.numeric(x))), labels = levels(x), 
                  cex.axis = cext)
            else axis(1, cex.axis = cext)
        }
        mtext(main, side = 3, -1.25, cex = cext)
        if (length(yy) != 0) {
            if (!is.null(a)) {
                for (i in 1:length(auni)) {
                  ins <- (zz == auni[i])
                  if (any(ins)) {
                    xxx <- xx[ins] + offset[i]
                    if (!err) 
                      if (!line & pts) 
                        if (!missing(texts)) 
                          if (jit) 
                            text(jitter(xxx, jit.factor), yy[ins], 
                              labels = pch[i], col = ifelse(ucol, 
                                col[i], ulinecol), cex = cexmsub[ins])
                          else text(xxx, yy[ins], labels = texts, 
                            col = ifelse(ucol, col[i], ulinecol), 
                            cex = cexmsub[ins])
                        else if (jit) 
                          points(jitter(xxx, jit.factor), yy[ins], 
                            col = ifelse(ucol, col[i], ulinecol), 
                            pch = pch[i], cex = cexmsub[ins], 
                            bg = bg)
                        else points(xxx, yy[ins], col = ifelse(ucol, 
                          col[i], ulinecol), pch = pch[i], cex = cexmsub[ins], 
                          bg = bg)
                      else if (line) 
                        lines(xxx, yy[ins], col = ifelse(ucol, 
                          col[i], ulinecol))
                    if (regz & (length(xxx) >= minpts)) {
					  if (regtype == "loess") 
                        pintol.loess(xxx, yy[ins], nse = se, nint = nint,
                          col = col[i], span = lo.span, degree = lo.degree, 
                          ...)
                      else if (regtype == "glm") 
                        pintol.glm(xxx, yy[ins], df = df, nse = se, nint = nint,
                          col = col[i], ...)
                      else if (regtype == "gam") 
                        pintol.gam(xxx, yy[ins], df = df, nse = se, nint = nint,
                          col = col[i], ...)
                    }
                    if (meansz & (length(xxx) >= minpts)) {
                      ym <- sapply(split(yy[ins], xxx), mean.na)
                      lines(unique(xxx), ym, col = col[i], lwd = lwdf)
                    }
                    if (err) {
                      if (missing(hi) & missing(lo)) {
                        errbar(unique(xxx), sapply(split(yy[ins], 
                          xxx), mean.na), sapply(split(yy[ins], 
                          xxx), sem), n = length(xxx), nse = se, 
                          ci.cover = ci.cov, pcheb = pch[i], bg = bg[i],
                          col = col[i], cexeb = cexm, gap = gap, 
                          bar.ends = bar.ends, texts = FALSE)
                      }
                      else {
                        errbar(xxx, yy[ins], hi = hhi[ins], lo = llo[ins], 
                          pcheb = pch[1], bg = bg[i], col = col[i], ci.cover = ci.cov, 
                          nse = se, cexeb = cexm, gap = gap, 
                          bar.ends = (10 * bar.ends)/length(unique(x)), 
                          texts = FALSE)
                      }
                    }
                  }
                }
            }
            else if (is.null(a)) {
                if (!err) 
                  if (!line & pts) 
                    if (!missing(texts)) {
                      if (jit) 
                        text(jitter(xx, jit.factor), yy, labels = texts, 
                          col = col[1], cex = cexm)
                      else text(xx, yy, labels = pch[1], col = col[1], 
                        cex = cexm)
                    }
                    else {
                      if (jit) 
                        points(jitter(xx, jit.factor), yy, pch = pch[1], 
                          col = col[1], cex = cexm, bg = bg)
                      else points(xx, yy, pch = pch[1], col = col[1], 
                        cex = cexm, bg = bg)
                    }
                  else if (line) {
                    lines(xx, yy, col = ifelse(ucol, col[i], 
                      ulinecol), lwd = lwd)
                  }
            }
            if (means & (length(xx) >= minpts)) {
                ym <- sapply(split(yy, xx), mean.na)
                lines(unique(xx), ym, lwd = lwdf)
            }
            if (err & is.null(a)) {
                if (missing(hi) & missing(lo)) {
                  errbar(unique(xx), sapply(split(yy, xx), mean.na), 
                    sapply(split(yy, xx), sem), pcheb = pch[1], bg = bg, 
                    n = length(xx), ci.cover = ci.cov, nse = se, 
                    cexeb = cexm, gap = gap, bar.ends = (10 * 
                      bar.ends)/length(unique(x)), texts = F)
                }
                else {
                  errbar(xx, yy, hi = hhi, lo = llo, pcheb = pch[1], 
                    bg = bg, ci.cover = ci.cov, nse = se, cexeb = cexm, 
                    gap = gap, bar.ends = (10 * bar.ends)/length(unique(x)), 
                    texts = FALSE)
                }
            }
            if (reg & (length(xx) >= minpts)) {
				if (regtype == "loess") 
                  pintol.loess(xx, yy, nse = se, span = lo.span, 
					 degree = lo.degree, nint = nint,  ...)
                else if (regtype == "glm") 
                  pintol.glm(xx, yy, df = df, nse = se, nint = nint, ...)
                else if (regtype == "gam") 
                  pintol.gam(xx, yy, cv = cvreg, df = df, nse = se, nint = nint, 
                    ...)
            }
            if (leg & leg.auto & !is.null(a)) 
                legend(xlim[1] * (1 - leg.loc[1]) + xlim[2] * 
                  leg.loc[1], ylim[1] * (1 - leg.loc[2]) + ylim[2] * 
                  leg.loc[2], legend = levels(zz), bty = "n", 
                  pch = pch, col = col, cex = cexl)
        }
    }
    if (missing(xlab)) 
        xlab <- x.name
    if (missing(ylab)) 
        ylab <- deparse(formula)
    if (blen > 1) {
        mtext(ylab, 2, outer = T, line = mgap, las = 0, cex = cext)
        mtext(xlab, 1, outer = T, line = mgap, las = 0, cex = cext)
		if (!missing(head)) mtext(head, 3, outer = T, line = mgap/2, las = 0, cex = 1.2*cext)    
	}
    else {
        mtext(ylab, 2, line = mgap, las = 0, cex = cext)
        mtext(xlab, 1, line = mgap, las = 0, cex = cext)
		if (!missing(head)) mtext(head, 3, line = mgap/2, las = 0, cex = 1.2*cext)    
    }
    if (fit.linear | fit.smooth) 
        pintol.fit(x = x, y = y, a = a, b = b,
            test = test.type, df = df, fit.linear = fit.linear, 
            fit.smooth = fit.smooth, diag.fit = diag.fit)
    if (leg & !leg.auto & !is.null(a)) 
        legend(locator(1), legend = levels(zz), bty = "n", pch = pch, 
            col = col, cex = cexl)
    invisible()
    if (ret.fit & fitted) 
        lmfit
}
