`geo3` <-
function (data = Soft_Corals_Reef, ycol = 101, xy = c(5:4), fname = deparse(substitute(data)),
	weights, hotspot = c("hi","lo")[1], value = "Value", unit = TRUE, lims, 
	image.prec = c(0,0.6,0.9,1), image.hot.hi = c(0,0.6,0.9,1), image.hot.lo = c(0,0.1,0.4,1),
	k30 = 30, cexp = 1, google.run = TRUE, extra = 0, mar.no.rot = rep(0, 4), fit = NULL,
	lab.nsig = 4, max.leg.col = 41, ascii = FALSE, leg.minw = 16, leg.col="orange", leg.font=2, 
    leg.eric = FALSE, leg.col.eric = "black", gdriver = "Xlib", link=FALSE,
    offset = 1, theta = 2*pi/9, logo = TRUE, logo.file = "logo.png",
    sidebar = TRUE, sidebar.file="sidebar.png", sidebar.hsv=c(0,0,0.15,1), 
    about = TRUE, merge = TRUE, rmKMLs = TRUE, zip = TRUE, se = TRUE, se.trim = 1.5, size = 4,
    size.png = 8, pixel = 19, font.scale = 1, icol.vals = col.g4yob4, icol.se = col.yorr4,
    ncol = 9, ...) 
{

	dir.work()
	if (is.numeric(ycol)) yname <- names(data)[ycol]
	else yname <- ycol
	cat("\nData set: ",fname,"\n")
	cat("Response: ",yname,"\n")
	if (unit && !is.null(attributes(data)$unit)) {
		if (!is.character(ycol)) value <- attributes(data)$unit[ycol]
		else value <- (1:ncol(data))[match(ycol,attributes(data)$unit)]
	}
	cat("Units: ",value,"\n")
	
	leg.w <- 0.8*(max(nchar(fname),nchar(yname),nchar(value))+2)
	leg.w <- max(c(leg.w, leg.minw))
	
	paster <- function(lab, qnt) {
		qnt <- round(100*qnt,2)
		qnt <- paste(qnt[-length(qnt)],qnt[-1],sep="-")
		paste(lab," ",qnt,"%",sep="")
	}

	data[, xy]	<- round(data[, xy],3)
	extra.cols <- c(ycol,xy)
	if (extra>0) extra.cols <- c(extra,extra.cols)
    opt_exask <- options(example.ask = FALSE)
    qk <- SpatialPointsDataFrame(data[, xy], data[,extra.cols])
    summary(Sobj_SpatialGrid(qk)$SG)
    proj4string(qk) <- CRS("+proj=longlat")
    SGqk <- GE_SpatialGrid(qk)

## generate Data points KML ##

	if (is.numeric(ycol)) ycol <- names(data)[ycol]
	writeOGR(qk, paste(fname,"-Data.kml", sep = ""), ycol, driver = "KML")

    if (is.null(fit)) {
        form <- formula(paste(yname,"~ s(across, along, k=",k30,",fx=T)",sep=" "))
        if (missing(weights)) 
        fit <- surfer.gam.gi(form, lat = names(data)[xy[2]], long = names(data)[xy[1]],
        ptype="response", data = data, se = se, se.trim = se.trim, ...)
        else 
        fit <- surfer.gam.gi(form, lat = names(data)[xy[2]], long = names(data)[xy[1]],
        weights = weights, ptype="response", data = data, se = se, se.trim = se.trim, ...)
	}
	else {
		if (is.null(fit$preds)) {
			cat("Predicting grid\n")
			fit <- predict2grid(fit)
		}
		cat("Using provided fit\n")
	}
	
	typetext <- c("Data","Value","Std_Error","Value-Legend","Std_Error-Legend")
	
## generate Value, Std_Error & HotSpots KMLs ##

    for (type in typetext[2:3]) {
		cat("Type = ",type,"\n")
		fname.png <- paste(fname,"-",type,".png", sep = "")
		if (.Platform$GUI == "X11") png(file = fname.png, width=size.png * SGqk$width,
				 height=size.png*SGqk$height, bg="transparent",type=gdriver)
		else png(file = fname.png, width=size.png * SGqk$width,
				 height=size.png*SGqk$height, bg="transparent")
		par(mar = c(0, 0, 0, 0), xaxs = "i", yaxs = "i")
		line.height <- 0.0012*pixel
			
		if (type=="Value") {
			llims <- min(max.leg.col,ncol)
			if (link)  {
				lnk <- fit$family$linkfun
				lnkinv <- fit$family$linkinv
				if (missing(lims)) lims <- lnkinv(seq(lnk(min(fit$preds$z,na.rm=T)),
					lnk(max(fit$preds$z,na.rm=T)),length=llims+1))
				else lims <- seq(lnk(lims[1]),lnk(lims[2]),length=llims+1)
			}
			else if (missing(lims)) lims <- seq(min(fit$preds$z,na.rm=T),max(fit$preds$z,na.rm=T),
			length=llims+1)
			else lims <- seq(lims[1],lims[2],length=llims+1)
			cat("lims : ",round(lims,2),"\n")
			labs <- formatter(lims,lab.nsig)
			labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
			fname.leg.png <- paste(fname,"-",type,"-Legend.png",sep="")				
			head <- gsub("_"," ",c(fname,yname,value))
			make.legend(labs=labs, head=head,nh=3,w=leg.w, cols=icol.vals,
			 name.png=fname.leg.png,npt=size*9,leg.col=leg.col,leg.font=leg.font)	
			png.size <- pixsize(fname.leg.png)
			xsze <- 0.55*font.scale*pixel*leg.w
			ysze <- xsze*png.size[2]/png.size[1]
			make.legend.kml(xpos=0.0,ypos=0.98,
			xsize=xsze, ysize=ysze, file=paste(fname,"-",type,"-Legend",sep=""))
			if (leg.eric) {
				make.legend.eric(labs=labs, w=leg.minw, cols=icol.vals,
				name.png=paste(getwd(),"/work/",fname,"-",yname,"-",value,"-Legend.png",
				sep=""),npt=size*4.53,leg.col=leg.col.eric)	
			}
			geoPlot4k(qk, preds = fit$preds, xlim = SGqk$xlim, ylim = SGqk$ylim, 
		    image.breaks = lims, image.col = icol.vals, image.ncol = ncol, ...) 
			dev.off()
			kmlOverlay2(SGqk,paste(fname,"-",type,".kml",sep = ""),fname.png,name=type,
			adj=0, order=4)
		}
		else if (type=="Std_Error") {
			llims <- min(max.leg.col,ncol)
			if (link)  {
				lnk <- fit$family$linkfun
				lnkinv <- fit$family$linkinv
				lims2 <- lnkinv(seq(lnk(min(fit$preds$z.se,na.rm=T)),
					lnk(max(fit$preds$z.se,na.rm=T)),length=llims+1))
			}
			else lims2 <- seq(min(fit$preds$z.se,na.rm=T),max(fit$preds$z.se,na.rm=T),
			length=llims+1)
			labs <- formatter(lims2,lab.nsig)
			labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
			fname.leg.png <- paste(fname,"-",type,"-Legend.png",sep="")				
			head <- gsub("_"," ",c(fname,yname,type))
			make.legend(labs=labs, head=head,nh=3,w=leg.w,cols=icol.se,
			name.png=fname.leg.png,npt=size*9,leg.col=leg.col,leg.font=leg.font)	
			png.size <- pixsize(fname.leg.png)
			xsze <- 0.55*font.scale*pixel*leg.w
			ysze <- xsze*png.size[2]/png.size[1]
			make.legend.kml(xpos=0.01,ypos=0.98-(4+llims)*line.height,
			xsize=xsze, ysize=ysze,	file=paste(fname,"-",type,"-Legend",sep=""))
			if (leg.eric) {
				make.legend.eric(labs=labs, w=leg.minw, cols=icol.se,
				name.png=paste(getwd(),"/work/",fname,"-",yname,"-",type,"-Legend.png",sep=""),
				npt=size*4.53,leg.col=leg.col.eric)	
			}
			geoPlot4k(qk, preds = fit$preds, se=T, xlim = SGqk$xlim, ylim = SGqk$ylim, 
			image.breaks = lims2, image.col = icol.se, image.ncol = ncol, ...)
			dev.off()
			kmlOverlay2(SGqk, paste(fname,"-",type,".kml", sep = ""), fname.png, name=type,
			adj=3, order=4)
		}
		if (ascii) 
			write.fit.asc(fit, paste(fname,"-",ycol,"-",type,".asc", sep = ""))
    }
	if (leg.eric) {
		efile <- paste(getwd(),"/work/",fname,"-",yname,".layers.csv",sep="")
		cat(paste("full.name,inputLayer,outputLayer,#cutpoints,cutpoints,cutpointColours\n"),file=efile)
		
		cat(paste(yname,"Value",value,ncol,
		paste(signif(lims,4),sep="",collapse=" "),
		paste(icol.vals(ncol),sep="",collapse=" "),sep=","),file=efile,append=TRUE)
		cat("\n",file=efile,append=TRUE)

		cat(paste(yname,"Std_Error","Std_Error",ncol,
		paste(signif(lims2,4),sep="",collapse=" "),	
		paste(icol.se(ncol),sep="",collapse=" "),sep=","),file=efile,append=TRUE)
		cat("\n",file=efile,append=TRUE)
	
		system(paste("zip -j ",getwd(),"/work/",fname,"-",yname,".zip ",
		getwd(),"/work/",fname,"*.png ",getwd(),"/work/",fname,"*.csv ", 
		getwd(),"/",fname,"_About.txt ",getwd(),"/",fname,".codes.csv ",  sep=""))
		system(paste("rm ",	getwd(),"/work/",fname,"*.png"," ",
		getwd(),"/work/",fname,"*.csv ",sep=""))
	}
	if (ascii) {
		system(paste("zip -j ",getwd(),"/work/",fname,"-",yname,".zip ",
		getwd(),"/",fname,"*.asc",sep=""))
		system(paste("rm ",	getwd(),"/",fname,"*.asc",sep=""))
	}
	if (sidebar) sidebar.do.png(sidebar.hsv,file=sidebar.file)
	if (merge) {
	    gname <-  paste(getwd(),"/",fname,"-",typetext,".kml",sep="")
		merge3(gname, about=about, fname= fname,logo=logo,sidebar=sidebar,
		sidebar.width=xsze)
		zfname <- paste(getwd(),"/work/",fname,"-",yname,"-3.kmz",sep="")
		if (zip) {
			if (logo) system(paste("zip -j ",zfname," ",fname,"-All.kml ",fname,
			"*.png ", logo.file, " ", sidebar.file, sep=""))
			else  system(paste("zip -j ",zfname," ",fname,"-All.kml ",fname,"*.png ",
			sidebar.file, sep=""))
		}
		if (rmKMLs) system(paste("rm ",fname,"*.kml ",fname,"*.png sidebar.png",sep=""))
		}
    if (google.run & merge)  
			system(paste("googleearth ",zfname," &",sep=""))
    invisible(fit)
}

