`geoFit2KML` <- function(fit, value = "VALUE",
	uppercase = F, locale, ascii = FALSE,
	google.run = FALSE, lab.nsig = 4, max.leg.col = 41, 
	leg.minw = 16, leg.col="orange", leg.font=2, leg.eric = FALSE,
	leg.col.eric = "black", gdriver = "cairo", link=FALSE, offset,
	theta, logo = TRUE, logo.file = "logo.png", sidebar = TRUE,
	sidebar.file="sidebar.png", sidebar.hsv=c(0,0,0.15,1), 
    about = FALSE, merge = TRUE, rmKMLs = TRUE, zip = TRUE, se = TRUE,
    se.trim = 0, size = 4, size.png = 8, pixel = 19, font.scale = 1,
    lims, icol.vals = col.g4yob4, icol.se = col.yorr4, 
    icol.grey = col.greys, grey.se = FALSE, ncol = 9, 
    xplot= FALSE, geo.png = TRUE, geo.eps = TRUE, wid=6, res=576, ...) 
{

	work.ok <- function() {
		work.dir <- paste(wdir,"/work",sep="")
		if (!file.exists(work.dir)) {
			dir.create(work.dir)
			cat(paste("Creating work directory",work.dir,"\n"))
		}
	}
	paster <- function(lab, qnt) {
		qnt <- round(100*qnt,2)
		qnt <- paste(qnt[-length(qnt)],qnt[-1],sep="-")
		paste(lab," ",qnt,"%",sep="")
	}
	wdir <- getwd()
	work.ok()
	
	data <- attr(fit,"data")
	fname <- attr(fit,"data.name")
	if (uppercase) fname <- toupper(fname)
	yname <- names(data)[1]
	if (uppercase) yname <- toupper(yname)
	cat("Data:",fname,";  Response:",yname,";  Value:",value,"\n")

	if (attr(fit,"model")=="abt") preds <- fit[[1]]$preds
    else preds <- fit$preds
	
	leg.w <- 0.8*(max(nchar(fname),nchar(yname),nchar(value))+2)
	leg.w <- max(c(leg.w, leg.minw))
	
	data2 <- round(data[,2:3],3)
    opt_exask <- options(example.ask = FALSE)
    qk <- SpatialPointsDataFrame(data2, data2)
    summary(Sobj_SpatialGrid(qk)$SG)
    proj4string(qk) <- CRS("+proj=longlat")
    SGqk <- GE_SpatialGrid(qk)

## generate Data points KML ##

	if (missing(locale)) locale <- attr(fit,"locale")
	cat("Locale =",locale,"\n")
	if (missing(offset)) offset <- c(3,-0.5)[match(locale,c("GBR","WA"))]
	if (missing(theta)) theta <- c(2*pi/9,0)[match(locale,c("GBR","WA"))]

	ycol <- names(data)[2:3]
	writeOGR(qk, paste(fname,"-Data.kml", sep = ""), ycol, driver = "KML")

	if (xplot) {
		cat("geoPlotting !\n")
		hratio <- 0.95*diff(range(data[,3]))/diff(range(data[,2]))
		if (geo.png) {
			png(file=paste("./work/",fname,"-",yname,"-2.png",sep=""),type="Xlib",
			hei=round(hratio*wid),wid=wid,unit="in",res=res)
			geoPlot(data,fit=fit,lat=names(data)[3],long=names(data)[2],
			pts.add=(nrow(data)<250),image.col=icol.vals,image.ncol=9)
			dev.off()
		}
		if (geo.eps) {
			X11(wid=wid,hei=round(hratio*wid))
			geoPlot(data,fit=fit,lat=names(data)[3],long=names(data)[2],
			pts.add=(nrow(data)<250),image.col=icol.vals,image.ncol=9)
			dev.copy2eps(file=paste("./work/",fname,"-",yname,"-2.eps",sep=""))
			dev.off()
		}
	} 

	typetext <- c("Data","Value","Std_Error","Value-Legend","Std_Error-Legend")
	
## generate Value, Std_Error KMLs ##


	if (!link & attr(fit,"model")=="gam")  {
		linkinv <- fit$family$linkinv
		preds$z.se <- (linkinv(preds$z+preds$z.se) -
			linkinv(preds$z-preds$z.se))/2
		preds$z <- linkinv(preds$z)
	}

    for (type in typetext[2:3]) {
		cat("Type = ",type,"\n")
		fname.png <- paste(fname,"-",type,".png", sep = "")
		if (.Platform$GUI == "X11") png(file = fname.png, width=size.png * SGqk$width,
				 height=size.png*SGqk$height, bg="transparent",type=gdriver)
		else png(file = fname.png, width=size.png * SGqk$width,
				 height=size.png*SGqk$height, unit="in", bg="transparent")
		par(mar = c(0, 0, 0, 0), xaxs = "i", yaxs = "i")
		line.height <- 0.0012*pixel
			
		if (type=="Value") {
			llims <- min(max.leg.col,ncol)
			if (missing(lims)) 	lims <- seq(min(preds$z,na.rm=T),
					max(preds$z,na.rm=T),length=llims+1)
			else lims <- seq(lims[1],lims[2],length=llims+1)
			labs <- formatter(lims,lab.nsig)
			labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
			fname.leg.png <- paste(fname,"-",type,"-Legend.png",sep="")				
			head <- gsub("_"," ",c(fname,yname,value))
			make.legend(labs=labs, head=head,nh=3,w=leg.w, cols=icol.vals,
			 name.png=fname.leg.png,npt=size*9,leg.col=leg.col,leg.font=leg.font)	
			png.size <- pixsize(fname.leg.png)
			xsze <- 0.55*font.scale*pixel*leg.w
			ysze <- xsze*png.size[2]/png.size[1]
			make.legend.kml(xpos=0.0,ypos=0.98,
			xsize=xsze, ysize=ysze, file=paste(fname,"-",type,"-Legend",sep=""))
			if (leg.eric) {
				make.legend.eric(labs=labs, w=leg.minw, cols=icol.vals,
				name.png=paste(wdir,"/work/",fname,"-",yname,"-",value,"-Legend.png",
				sep=""),npt=size*4.53,leg.col=leg.col.eric)	
			}
			geoPlot4k(qk, preds = preds, xlim = SGqk$xlim, ylim = SGqk$ylim, 
		    image.breaks = lims, image.col = icol.vals, image.ncol = ncol) 
			dev.off()
			kmlOverlay2(SGqk,paste(fname,"-",type,".kml",sep = ""),fname.png,name=type,
			adj=0, order=4)
			if (ascii) 
				write.fit.asc(fit, paste(fname,"-",yname,"-",type,".asc", sep = ""),se = F)
		}
		else if (type=="Std_Error") {
			llims <- min(max.leg.col,ncol)
		 	lims2 <- seq(min(preds$z.se,na.rm=T),max(preds$z.se,na.rm=T),length=llims+1)
			labs <- formatter(lims2,lab.nsig)
			labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
			fname.leg.png <- paste(fname,"-",type,"-Legend.png",sep="")				
			head <- gsub("_"," ",c(fname,yname,type))
			if(grey.se) gcols <- icol.grey else gcols <-icol.se
			make.legend(labs=labs, head=head,nh=3,w=leg.w,cols=gcols,
			name.png=fname.leg.png,npt=size*9,leg.col=leg.col,leg.font=leg.font)	
			png.size <- pixsize(fname.leg.png)
			xsze <- 0.55*font.scale*pixel*leg.w
			ysze <- xsze*png.size[2]/png.size[1]
			make.legend.kml(xpos=0.01,ypos=0.98-(4+llims)*line.height,
			xsize=xsze, ysize=ysze,	file=paste(fname,"-",type,"-Legend",sep=""))
			if (leg.eric) {
				make.legend.eric(labs=labs, w=leg.minw, cols=gcols,
				name.png=paste(wdir,"/work/",fname,"-",yname,"-",type,"-Legend.png",sep=""),
				npt=size*4.53,leg.col=leg.col.eric)	
			}
			geoPlot4k(qk, preds = preds, se=T, xlim = SGqk$xlim, ylim = SGqk$ylim, 
			image.breaks = lims2, image.col = gcols, image.ncol = ncol)
			dev.off()
			kmlOverlay2(SGqk, paste(fname,"-",type,".kml", sep = ""), fname.png, name=type,
			adj=ifelse(grey.se,0,offset), theta=theta, order=4)
			if (ascii) 
				write.fit.asc(fit, paste(fname,"-",yname,"-",type,".asc", sep = ""), se=T)
		}
    }
	if (leg.eric) {
		efile <- paste(wdir,"/work/",fname,"-",yname,".layers.csv",sep="")
		cat(paste("full.name,inputLayer,outputLayer,#cutpoints,cutpoints,cutpointColours\n"),file=efile)
		
		cat(paste(yname,"Value",value,ncol,
		paste(signif(lims,4),sep="",collapse=" "),
		paste(icol.vals(ncol),sep="",collapse=" "),sep=","),file=efile,append=TRUE)
		cat("\n",file=efile,append=TRUE)

		cat(paste(yname,"Std_Error","Std_Error",ncol,
		paste(signif(lims2,4),sep="",collapse=" "),	
		paste(icol.se(ncol),sep="",collapse=" "),sep=","),file=efile,append=TRUE)
		cat("\n",file=efile,append=TRUE)
	
		system(paste("zip -j ",wdir,"/work/",fname,"-",yname,".zip ",
		wdir,"/work/",fname,"*.png ",wdir,"/work/",fname,"*.csv ", 
		wdir,"/",fname,"_About.txt ",wdir,"/",fname,".codes.csv ",  sep=""))
		system(paste("rm ",	wdir,"/work/",fname,"*.png"," ",
		wdir,"/work/",fname,"*.csv ",sep=""))
	}
	if (ascii) {
		system(paste("zip -j ",wdir,"/work/",fname,"-",yname,".zip ",
		wdir,"/",fname,"*.asc",sep=""))
		system(paste("rm ",	wdir,"/",fname,"*.asc",sep=""))
	}
	if (sidebar) sidebar.do.png(sidebar.hsv,file=sidebar.file)
	if (merge) {
	    gname <-  paste(wdir,"/",fname,"-",typetext,".kml",sep="")
		merge3(gname, about=about, fname= fname,logo=logo,sidebar=sidebar,
		sidebar.width=xsze,locale=locale)
		zfname <- paste(wdir,"/work/",fname,"-",yname,"-2.kmz",sep="")
		if (zip) {
			if (logo) system(paste("zip -j ",zfname," ",fname,"-All.kml ",fname,
			"*.png ", logo.file, " ", sidebar.file, sep=""))
			else  system(paste("zip -j ",zfname," ",fname,"-All.kml ",fname,"*.png ",
			sidebar.file, sep=""))
		}
		if (rmKMLs) system(paste("rm ",fname,"*.kml ",fname,"*.png sidebar.png",sep=""))
		}
    if (google.run & merge)  
			system(paste("googleearth ",zfname," &",sep=""))
}

