geoFitx <- function (form, data, model=c("gam","abt","loess")[1], weights, 
	lat = "lat", long = "long",	locale, grid.predict = TRUE, link = TRUE, 
	aafun, land, grid = c("chull","locale")[2], xlim, ylim, xpnd = 0.02, 
	plt = TRUE, plt.size = 6, pts = TRUE, pch = 21, bg = "lightblue", nsigs = 4,
	se = TRUE, trim= FALSE, mark.se = c(10:0), se.omit = 3, 
	icol.vals = col.g4yob4, icol.se = col.yorr4, ncol=9,
	contour.add = FALSE, aauto = FALSE, aauto.vals = c(0.25,0.5,1,2,4,8,12),
    grid.size = 0.1, aak.min = c("fit","best")[1], aak = 4, ...) 
{
    require(mgcv)
    require(acrossAlong)
	if (model=="abt") require(abt)
	if (.Platform$OS.type == "unix") GFUN <- x11 else GFUN  <- windows

    g2i <- function(x, preds, preds.se) {
        ndiv <- attr(x, "ndiv")
        xr <- range(x$long)
        xi <- seq(xr[1], xr[2], by = 1/ndiv)
        nxi <- length(xi)
        yr <- range(x$lat)
        yi <- seq(yr[1], yr[2], by = 1/ndiv)
        nyi <- length(yi)
        zmat <- matrix(NA, ncol = nyi, nrow = nxi)
        xins <- match(round(x$long * ndiv), round(xi * ndiv))
        yins <- match(round(x$lat * ndiv), round(yi * ndiv))
        zmat[cbind(xins, yins)] <- preds
        lst <- list(x = xi, y = yi, z = zmat)
        if (!missing(preds.se)) {
            zmat[cbind(xins, yins)] <- preds.se
            lst$z.se <- zmat
        }
        lst
    }

	get.method <- function(form) {
		ch3 <- as.character(form)[[3]]
		if (ch3=="1") meth <- "cc"
 		else if ((length(grep("across",ch3))==1) | (length(grep("along",ch3))==1)) meth <- "aa"
		else meth <- "ll"
		meth
	}

	formatter <- function(x,ndig=5,eps=10*.Machine$double.eps) {
		n10 <- floor(log10(abs(x)+eps))*sign(x)
		xx <- rep("",length(x))
		for (i in unique(n10)) 
		xx[n10==i] <- formatC(x[n10==i],ndig-i-1,format="f")
		xx[nchar(xx)>ndig] <- substring(xx[nchar(xx)>ndig],1,ndig)
		nch <- nchar(xx)
		dots <- substring(xx,nch,nch)=="."
		xx[dots] <- substring(xx[dots],1,nch[dots]-1) 
		xx
	}

	make.legend <- function(x,cex=2.7,pch=15,cols) {
		rnge <- range(x,na.rm=T)
		rvals <- seq(max(rnge),min(rnge),length=length(cols))
		rvals <- formatter(rvals,nsigs)
		legend("topright",rvals,col=cols,bty="n",pch=pch,pt.cex=cex,cex=0.85*par()$cex)
	}

	if (missing(locale)) 
		locale <- ifelse(median(data[,long])>120, "GBR", "WA")
	cat("Locale =",locale,"\n")

	if (missing(aafun)) {
		if (locale=="GBR") aafun <- acrossAlong
		else aafun <- aaCoast
	}
	if (missing(land)) {
		if (locale=="GBR") land <- landonly
		else land <- ning.coast.smooth	
	}
	if (is.data.frame(grid)) 
		cat("User supplied grid:",deparse(substitute(grid)),"\n")	
	else if (grid=="locale") {
		if (locale=="GBR") {
			grid <- grid005.aa
			cat("Locale grid: grid005.aa\n")	
		}	
		else {
			grid <- grid001.nc	
			cat("Locale grid: grid001.nc\n")	
		}	
	}
	else if (grid=="chull") {
		grid <- makegrid.chull(data[,c(long,lat)])
	}

	data.name <- deparse(substitute(data))
#	y <- as.character(form[[2]])
#	data <- data[,c(y,long,lat)]

	method <- get.method(form)
	if (method=="aa") {
		cat("Calculating across-along for data\n")
		new.aa <- aafun(data[, c(long, lat)], land = land)
		data$across <- new.aa[, "across"]
		data$along <- new.aa[, "along"]
		if (length(na.omit(match(names(grid),c("across","along"))))!=2) {
			cat("Calculating across-along for grid\n")
			new.aa <- aafun(grid[, c(long, lat)], land = land)
			grid$across <- new.aa[, "across"]
			grid$along <- new.aa[, "along"]
		}
	}
    if (aauto) {
        nl <- length(aauto.vals)
        res <- matrix(NA, nrow = nl, ncol = 2)
        res[, 1] <- aauto.vals
        dimnames(res) <- list(as.character(aauto.vals), c("aak", 
            "gcv"))
        temp <- data
        for (i in 1:nl) {
            cat(i, " ")
            temp$along <- aauto.vals[i] * new.aa[, "along"]
			if (model=="gam") {
				if (missing(weights)) 
					fit <- gam(formula = form, data = temp, ...)
				else fit <- gam(formula = form, data = temp, weights = weights, ...)
			}
			else cat("aauto only available for gams \n")
		res[i, 2] <- fit$gcv.ubre
        }
        minr <- (1:nl)[res[, 2] == min(res[, 2])][1]
        if (aak.min == "best") 
            aak <- res[minr, 1]
        else if (minr == 1 | minr == nl) 
            aak <- res[minr, 1]
        else {
            res3 <- data.frame(res[(minr - 1):(minr + 1), ])
            fit <- lm(gcv ~ poly(aak, 2), res3)
            newdf <- data.frame(aak = seq(min(res3$aak), max(res3$aak), 
                length = 100))
            min.aak <- min(aak.pred <- predict(fit, new = newdf, 
                type = "response"))
            aak <- newdf[aak.pred == min.aak, ]
        }
        cat("\n")
        print(res)
        cat("Best aak =", aak, "\n")
    }
	if (method=="aa") data$along <- aak*data$along
    if (model=="gam") {
		if (missing(weights)) 
		    fit <- gam(formula = form, data = data, ...)
		else fit <- gam(formula = form, data = data, weights = weights, ...)
		print(summary(fit))
	}
	else if (model=="abt") {
		if (missing(weights)) 
			fit <- abt(formula = form, data = data, ...)
		else fit <- abt(formula = form, data = data, weights = weights, ...)
		print(summary(fit,plot=F))
	}            
	else if (model=="loess") {
		if (missing(weights)) 
		    fit <- loess(formula = form, data = data, normalize=FALSE, surface="direct", ...)
		else fit <- loess(formula = form, data = data, weights = weights, surface="direct",
			normalize=FALSE, ...)
		print(summary(fit))
	}
	
    if (plt) {
    	GFUN(xpos=0,ypos=0,wid=plt.size,hei=plt.size,title="Model Diagnostics")
    	par(mfrow=c(2,2),mar=c(3.5,3.5,1,1),mgp=c(2,0.75,0),cex=0.9)
		qqnorm(residuals(fit),main="")
		scatter.smooth(fitted(fit),abs(residuals(fit)),xlab="Fitted Values",
		ylab="Absolute Residuals")
		boxplot(residuals(fit),ylab="Residuals")
		va <- variogram(residuals(fit)~1, data=data, formula(paste("~",long,"+",lat)))
		scatter.smooth(va$dist,va$gamma,ylim=c(0,max(va$gamma)),span=0.5,
		xlab="Distance",ylab="Semi-Variance")
	}
 	if (method!="cc") {
		mg <- missing(grid)*35
	    if (missing(grid)) {
	    	GFUN(xpos=35+mg,ypos=35+mg,wid=plt.size,hei=plt.size,
	    	title="Density Plot and Grid Boundary")
	    	grid <- makegrid.density(data[,long], data[,lat], size=grid.size)
	    	fit$grid <- grid
		}
		if (grid.predict) {
		    if (missing(xlim) & missing(ylim)) {
		        xlim <- range(data[, long])
		        ylim <- range(data[, lat])
		    }
		    if (xpnd > 0) {
		        xlim <- xlim + xpnd * c(-1, 1) * diff(xlim)
		        ylim <- ylim + xpnd * c(-1, 1) * diff(ylim)
		    }
		    grid <- grid[grid$long > xlim[1] & grid$long < xlim[2] & 
		        grid$lat > ylim[1] & grid$lat < ylim[2], ]
			if (method=="aa")  grid$along <- aak * grid$along

		    if (!se) {
		        pd <- predict(fit, newdata = grid, type = ptype)
		        preds <- g2i(grid, pd)
		    }
		    else {
		        if (!mark.se[1]) {
		            pd <- predict(fit, newdata = grid, type = ptype, se = T)
		            preds <- g2i(grid, pd$fit, pd$se.fit)
		        }
		        else {
		            pd <- predict(fit, newdata = grid, type = "link", se = T)
		            preds <- g2i(grid, pd$fit, pd$se.fit)
		            preds.fit <- predict(fit, newdata = data,  type = "link", se = T)
		            bxp <- boxplot(preds.fit$se.fit,  plot = F)
					mark.n <- length(mark.se)
		            mark.yvals <- rep(0,mark.n)
		            if (mark.se[1]) { 
						se.bxplot <- 0.75*(bxp$stats[4]-bxp$stats[2])
						if (trim) {
					    	mark <- preds$z.se > bxp$stats[3] + se.omit*se.bxplot
							cat("Omitting", round(100 * sum(mark, 
						        na.rm = T)/sum(!is.na(preds$z.se)), 2), "% pixels on high SEs\n")
						    preds$z[mark] <- NA
						    preds$z.se[mark] <- NA
						}
				    for (i in mark.n:1) 
					    mark.yvals[i] <- bxp$stats[3] + mark.se[i]*se.bxplot					
		            }
		        }
		    }

		    if (plt & se) {
				GFUN(xpos=515,ypos=0,wid=0.7*plt.size,hei=plt.size,title="SE Distributions")
		        par(mfrow = c(1, 1))
				boxplot(predict(fit,newdata=data,se=T,wid=0.5)$se,NA,preds$z.se,
				names=c("Data","SE","Grid"))
				points(rep(2,mark.n),mark.yvals,col="blue",pch=3,cex=2)					
				text(rep(2.2,mark.n),mark.yvals,mark.se)
		    }

		    if (plt) {
		        if (!se) {
		            GFUN(xpos=35+mg,ypos=105+mg,wid=0.8*plt.size,hei=plt.size,
		            	title="Image of Predictions")
		            par(mar=c(3,3,1,1))
		            if (!link) preds$z <- fit$family$linkinv(preds$z)
		            image(x = preds$x, y = preds$y, z = preds$z, 
		             col = icol.vals(ncol), axes = T, asp = 1)
		            if (contour.add) contour(preds, add = T)
	 				if(pts) points(data[,long],data[,lat],pch=pch,bg=bg,...)
	 				make.legend(preds$z, cols=rev(icol.vals(ncol)))
		        }
		        else {
					GFUN(xpos=35+mg,ypos=35+mg,wid=1.6*plt.size,hei=plt.size,
					  title="Images of Predictions & SEs")
		            par(mfrow = c(1, 2),mar=c(3,3,1,1))
		            if (!link) {
		            	preds$z.se <- 0.5*(fit$family$linkinv(preds$z+preds$z.se) 
		            		- fit$family$linkinv(preds$z-preds$z.se))
		            	preds$z <- fit$family$linkinv(preds$z)
					}
		            image(x = preds$x, y = preds$y, z = preds$z,
		             col = icol.vals(ncol), axes = T, asp = 1)
		            if (contour.add) contour(preds, add = T)
	 				if(pts) points(data[,long],data[,lat],pch=pch,bg=bg,...)
	 				make.legend(preds$z, cols=rev(icol.vals(ncol)))
		            image(x = preds$x, y = preds$y, z = preds$z.se, 
		              col = icol.se(ncol), xlab = "", ylab = "", axes = T, asp = 1)
		            if (contour.add) contour(x = preds$x, y = preds$y, z = preds$z.se, add = T)
	 				if(pts) points(data[,long],data[,lat],pch=pch,bg=bg,...)
	 				make.legend(preds$z.se, cols=rev(icol.se(ncol)))
		        }
				if (model=="abt") {
					GFUN(xpos=70+mg,ypos=70+mg,wid=plt.size,hei=plt.size,
					  title="Pediction Error Plots")
					  plot.err(fit)
				}
			}
		    if (model!="abt") fit$preds <- preds
		    else fit[[1]]$preds <- preds
		}
    }

    attr(fit,"locale") <- locale
    attr(fit,"link") <- link
	if (se) {
	    attr(fit,"mark.median") <- bxp$stats[3]
	    attr(fit,"mark.se.bxplot") <- se.bxplot
	    attr(fit,"mark.yvals") <- mark.yvals
	    attr(fit,"mark.se") <- mark.se
	}
    attr(fit,"model") <- model
    attr(fit,"data.name") <- data.name    
    if (method == "aa") attr(fit,"data") <- data[,c(asch(fit$formula[[2]]),long,lat,"across","along")]
    else attr(fit,"data") <- data[,c(asch(fit$formula[[2]]),long,lat)]
    attr(fit, "aak") <- aak
    fit
}
