`geoG` <- function (data = Soft_Corals_Reef, ycol = 101, xy = c(5:4), grp = 6, 
	fname = deparse(substitute(data)), yname = names(data)[ycol], cut.breaks, cut.n = 4, 
	unit = TRUE, k30 = 20, cexp = 1, google.run = FALSE, offset = 3, 
    fit.m = NULL, lab.nsig = 4, max.leg.col = 41,
    leg.col="orange", legend = TRUE, leg.minw = 16, logo = TRUE, gdriver = "Xlib",
    zip = TRUE, ascii = FALSE, logo.file = "logo.png", 
    sidebar = TRUE, sidebar.file="sidebar.png", sidebar.hsv=c(0,0,0.15,1), 
    about = FALSE, se = TRUE, se.trim = 0, size = 8, pixel = 19, font.scale = 1,
    theta = 2*pi/9, merge = TRUE, rmKMLs = TRUE, icol = col.g4yob4, ncol = 9, breaks = NULL, ...) 
{

	if (is.numeric(ycol)) yname <- names(data)[ycol]
	else yname <- ycol
	cat("\nData set: ",fname,"\n")
	cat("Response: ",yname,"\n")
	if (unit && !is.null(attributes(data)$unit)) {
		if (!is.character(ycol)) values <- attributes(data)$unit[ycol]
		else values <- (1:ncol(data))[match(ycol,attributes(data)$unit)]
		cat("Units: ",values,"\n")
	}
	else values <- "Values"	
	
	groups <- data[,grp]
	gname <- names(data)[grp]
	
	if (!missing(cut.breaks)) groups <- cut(groups,breaks=cut.breaks,include=T,dig.lab=6)
	else groups <- cut(groups, 
	breaks=seq(min(groups),max(groups),length=cut.n,dig.lab=6), include=T)
	
	k30 <- round(0.5*min(k30,min(table(groups))))
	cat("Smoothing df: ",k30,"\n")
	
	uni <- as.character(sort(unique(groups)))
	luni <- length(uni)
	cat("Group: ",gname,"\n")
	cat("Group levels: ",uni,"\n")

	leg.w <- 0.8*(max(nchar(fname),nchar(yname),nchar(values),nchar(uni))+2)
	leg.w <- max(c(leg.w, leg.minw))
	
    opt_exask <- options(example.ask = FALSE)
    qk <- SpatialPointsDataFrame(data[, xy], data)
    summary(Sobj_SpatialGrid(qk)$SG)
    proj4string(qk) <- CRS("+proj=longlat")
    SGqk <- GE_SpatialGrid(qk)

## generate Data points KML ##

#	writeOGR(qk, paste(fname,"-Data.kml", sep = ""), ycol, driver = "KML")

	typetext <- c("Data","Values")
	if (legend) typetext <- c(typetext,"Values-Legend")

	nicol <- 1
	if (is.list(icol)) {
		nicol <- length(icol)
		icol.lst <- icol
	}


## generate Values for all variables  ##

	pred.lims <- NULL    
	if (is.null(fit.m)) {
		fit <- list()
		for (i in 1:luni) {
			unii <- uni[i]
			cat("\nGrp = ",unii,"\n")
				temp <- data[groups==unii, ]
		    fit[[i]] <- surfer.gam.gi(temp[,ycol] ~ s(across, along, k=k30), 
		    lat = names(data)[xy[2]], long = names(data)[xy[1]], data = temp, 
		    se = se, se.trim = se.trim, ...)
			pred.lims <- range(c(pred.lims,fit[[i]]$preds$z),na.rm=T)	
		}
	}
	else {
		fit <- fit.m
		for (i in 1:luni) {
			fit[[i]] <- fit.m[[i]]
			pred.lims <- range(c(pred.lims,fit[[i]]$preds$z),na.rm=T)
		}
		cat("Using provided fit\n")
	}

	key.breaks <- seq(min(pred.lims),max(pred.lims),length=ncol+1)
	
    for (i in 1:luni) {
		if (offset) adj <- (i-1)*offset else adj <- 0
 		fname.png <- paste(fname,"-",yname,"-",i,".png", sep = "")
		if (.Platform$GUI == "X11") png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent",type=gdriver)
		else png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent")
		par(mar = c(0, 0, 0, 0), xaxs = "i", yaxs = "i")
		line.height <- 0.0012*pixel
		if (nicol>1) col <- icol.lst[[i]]
		if (is.function(icol)) icol <- icol(ncol)
		geoPlot4k(qk, preds = fit[[i]]$preds, xlim = SGqk$xlim, ylim = SGqk$ylim, 
	    image.breaks = key.breaks, image.col = icol, image.ncol = ncol, ...)
		dev.off()
		kmlOverlay2(SGqk,paste(fname,"-",yname,"-",i,".kml",sep ="") ,fname.png,
		name=yname, adj = adj, order = 4)
		if (ascii) write.fit.asc(fit, paste(fname,"-",yname,"-",unii,".asc", sep = ""))
    }
	
	if (legend) {
		llims <- min(max.leg.col,ncol)
		lims <- seq(min(fit[[i]]$preds$z,na.rm=T),max(fit[[i]]$preds$z,na.rm=T),length=llims+1)
		labs <- formatter(lims,lab.nsig)
		labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
		fname.leg.png <- paste(fname,"-",yname,"-Legend.png",sep="")
		head <- gsub("_"," ",c(fname,yname,gname,uni))
		make.legend(labs=labs, head=head, nh=length(head), w=leg.w, cols = icol,
		name.png=fname.leg.png,npt=size*9,leg.col=leg.col) 
		png.size <- pixsize(fname.leg.png)
		xsze <- 0.55*font.scale*pixel*leg.w
		ysze <- xsze*png.size[2]/png.size[1]
		make.legend.kml(xpos=0.0,ypos=0.98,xsize=xsze, ysize=ysze, 
		file=paste(fname,"-",yname,"-Legend",sep=""))
	}
	if (sidebar) sidebar.do.png(sidebar.hsv,file=sidebar.file)
	if (merge) {
		gname <-  paste(getwd(),"/",fname,"-",yname,"-",1:luni,".kml",sep="")
	    if (legend)  gname <-  c(gname,paste(getwd(),"/",fname,"-",yname,"-Legend.kml",sep=""))
		outfile <- paste(fname,"-",yname,"-G.kml",sep="")
		mergeG(outfile, gname, 1:luni, about=about, fname=fname, logo=logo,
		sidebar=sidebar,sidebar.width=xsze)
		zfname <- paste(getwd(),"/work/",fname,"-",yname,"-G.kmz",sep="")
		if (zip) {
			if (logo) system(paste("zip -j ",zfname," ",fname,"-",yname,"-G.kml ",fname,
			"*.png ", logo.file, " ", sidebar.file, sep=""))
			else system(paste("zip -j ",zfname," ",fname,"-",yname,"-G.kml ",fname,"*.png ",
			sidebar.file, sep=""))
		}
		if (rmKMLs) system(paste("rm ",fname,"*.kml ",fname,"*.png sidebar.png",sep=""))
		}
    if (google.run & merge)  
			system(paste("googleearth",zfname,"&",sep=" "))
	invisible(fit)
}

