geoGAM <-
function (form, data, weights, grid.predict = TRUE, aafun, land, grid,
	xlim, ylim, xpnd = 0.02, lat = "lat", long = "long", ptype = "response", plt = TRUE,
	plt.size = 6, pts = TRUE, se = TRUE, se.mark = c(10:0), se.omit = 0,
	aauto = FALSE, aauto.vals = c(0.25,0.5,1,2,4,8,12),
    grid.size = 0.1, aak.min = c("fit","best")[1], aak = 4, ...) 
{
    require(mgcv)
    require(acrossAlong)

    g2i <- function(x, preds, preds.se) {
        ndiv <- attr(x, "ndiv")
        xr <- range(x$long)
        xi <- seq(xr[1], xr[2], by = 1/ndiv)
        nxi <- length(xi)
        yr <- range(x$lat)
        yi <- seq(yr[1], yr[2], by = 1/ndiv)
        nyi <- length(yi)
        zmat <- matrix(NA, ncol = nyi, nrow = nxi)
        xins <- match(round(x$long * ndiv), round(xi * ndiv))
        yins <- match(round(x$lat * ndiv), round(yi * ndiv))
        zmat[cbind(xins, yins)] <- preds
        lst <- list(x = xi, y = yi, z = zmat)
        if (!missing(preds.se)) {
            zmat[cbind(xins, yins)] <- preds.se
            lst$z.se <- zmat
        }
        lst
    }

	get.method <- function(form) {
		ch3 <- as.character(form)[[3]]
		if (ch3=="1") meth <- "cc"
 		else if ((length(grep("across",ch3))==1) | (length(grep("along",ch3))==1)) meth <- "aa"
		else meth <- "ll"
		meth
	}

	make.legend <- function(x,np=5,cex=2.7,pch=15) {
		rnge <- range(x,na.rm=T)
		rvals <- seq(min(rnge),max(rnge),length=2*np+1)[2*(1:np)]
		rvals <- as.character(signif(rvals,3))
		legend("topright",rvals,col=rev(heat.colors(length(rvals))),bty="n",pch=pch,pt.cex=cex)
	}

	if (missing(aafun)) {
		locale <- ifelse(median(data[,long])>120, "GBR", "WA")
		if (locale=="GBR") aafun <- acrossAlong
		else aafun <- aaCoast
	}
	if (missing(land)) {
		if (locale=="GBR") land <- landonly
		else land <- ning.coast.smooth	
	}
	if (missing(grid)) {
		if (locale=="GBR") grid <- grid005.aa
		else grid <- grid001.nc	
	}
	cat("Locale =",locale,"\n")

	data.name <- deparse(substitute(data))
	y <- as.character(form[[2]])
	data <- data[,c(y,long,lat)]

	method <- get.method(form)
	if (method=="aa") {
		cat("Calculating across-along\n")
		new.aa <- aafun(data[, c(long, lat)], land = land)
		data$across <- new.aa[, "across"]
		data$along <- new.aa[, "along"]
	}
    if (aauto) {
        nl <- length(aauto.vals)
        res <- matrix(NA, nrow = nl, ncol = 2)
        res[, 1] <- aauto.vals
        dimnames(res) <- list(as.character(aauto.vals), c("aak", 
            "gcv"))
        for (i in 1:nl) {
            cat(i, " ")
            temp <- data
            temp$along <- aauto.vals[i] * new.aa[, "along"]
            if (missing(weights)) 
                fit <- gam(formula = form, data = temp, ...)
            else fit <- gam(formula = form, data = temp, weights = weights, ...)
            res[i, 2] <- fit$gcv.ubre
        }
        minr <- (1:nl)[res[, 2] == min(res[, 2])][1]
        if (aak.min == "best") 
            aak <- res[minr, 1]
        else if (minr == 1 | minr == nl) 
            aak <- res[minr, 1]
        else {
            res3 <- data.frame(res[(minr - 1):(minr + 1), ])
            fit <- lm(gcv ~ poly(aak, 2), res3)
            newdf <- data.frame(aak = seq(min(res3$aak), max(res3$aak), 
                length = 100))
            min.aak <- min(aak.pred <- predict(fit, new = newdf, 
                type = ptype))
            aak <- newdf[aak.pred == min.aak, ]
        }
        cat("\n")
        print(res)
        cat("Best aak =", aak, "\n")
    }
	if (method=="aa") data$along <- aak*data$along
    if (missing(weights)) 
        fit <- gam(formula = form, data = data, ...)
    else fit <- gam(formula = form, data = data, weights = weights, ...)
	print(summary(fit))

#	fit$data.name <- data.name    
#    fit$data <- data

    if (plt) {
    	x11(xpos=0,ypos=0,wid=plt.size,hei=plt.size,title="Model Diagnostics")
    	par(mfrow=c(2,2),mar=c(3.5,3.5,1,1),mgp=c(2,0.75,0),cex=1)
		qqnorm(residuals(fit),main="")
		scatter.smooth(fitted(fit),abs(residuals(fit)),xlab="Fitted Values",
		ylab="Absolute Residuals")
		boxplot(residuals(fit),ylab="Residuals")
#    	x11(xpos=35,ypos=35,wid=plt.size,hei=plt.size,title="Variogram of Residuals")
		va <- variogram(residuals(fit)~1, data=data, formula(paste("~",long,"+",lat)))
		scatter.smooth(va$dist,va$gamma,ylim=c(0,max(va$gamma)),span=0.5,
		xlab="Distance",ylab="Covariance")
	}
 	if (method!="cc") {
		mg <- missing(grid)*35
	    if (missing(grid)) {
	    	x11(xpos=35+mg,ypos=35+mg,wid=plt.size,hei=plt.size,
	    	title="Density Plot and Grid Boundary")
	    	grid <- makegrid.density(data[,long], data[,lat], size=grid.size)
	    	fit$grid <- grid
		}
		if (grid.predict) {
		    if (missing(xlim) & missing(ylim)) {
		        xlim <- range(data[, long])
		        ylim <- range(data[, lat])
		    }
		    if (xpnd > 0) {
		        xlim <- xlim + xpnd * c(-1, 1) * diff(xlim)
		        ylim <- ylim + xpnd * c(-1, 1) * diff(ylim)
		    }
		    grid <- grid[grid$long > xlim[1] & grid$long < xlim[2] & 
		        grid$lat > ylim[1] & grid$lat < ylim[2], ]
			if (method=="aa")  grid$along <- aak * grid$along

		    if (!se) {
		        pd <- predict(fit, newdata = grid, type = ptype)
		        preds <- g2i(grid, pd)
		    }
		    else {
		        if (!se.mark[1]) {
		            pd <- predict(fit, newdata = grid, type = ptype, se = T)
		            preds <- g2i(grid, pd$fit, pd$se.fit)
		        }
		        else {
		            pd <- predict(fit, newdata = grid, type = "link", se = T)
		            preds <- g2i(grid, pd$fit, pd$se.fit)
		            preds.fit <- predict(fit, se = T, type = "link")
		            bxp <- boxplot(preds.fit$se.fit,  plot = F)
					mark.n <- length(se.mark)
		            mark.yvals <- rep(0,mark.n)
		            if (se.mark[1]) { 
						se.bxplot <- 0.75*(bxp$stats[4]-bxp$stats[2])
						if (se.omit) {
					    	mark <- preds$z.se > bxp$stats[3] + se.omit*se.bxplot
							cat("Omitting", round(100 * sum(mark, 
						        na.rm = T)/sum(!is.na(preds$z.se)), 2), "% pixels on high SEs\n")
						    preds$z[mark] <- NA
						    preds$z.se[mark] <- NA
						}
				    for (i in mark.n:1) 
					    mark.yvals[i] <- bxp$stats[3] + se.mark[i]*se.bxplot					
		            }
		        }
		    }

		    if (plt & se) {
				x11(xpos=515,ypos=0,wid=0.7*plt.size,hei=plt.size,title="SE Distributions")
		        par(mfrow = c(1, 1))
				boxplot(predict(fit,newdata=data,se=T,wid=0.5)$se,NA,preds$z.se,
				names=c("Data","SE","Grid"))
				points(rep(2,mark.n),mark.yvals,col="blue",pch=3,cex=2)					
				text(rep(2.2,mark.n),mark.yvals,se.mark)
		    }

		    if (plt) {
		        if (!se) {
		            x11(xpos=35+mg,ypos=105+mg,wid=plt.size,hei=plt.size,
		            	title="Image of Predictions")
		            par(mar=c(3,3,1,1))
		            image(preds, col = rev(heat.colors(200)), axes = T, asp = 1)
		            contour(preds, add = T)
	 				if(pts) points(data[,long],data[,lat],pch=21,bg="lightblue",cex=0.7)
	 				make.legend(preds$z)
		        }
		        else {
					x11(xpos=35+mg,ypos=35+mg,wid=1.8*plt.size,hei=0.9*plt.size,
					  title="Images of Predictions & SEs")
		            par(mfrow = c(1, 2),mar=c(3,3,1,1))
		            image(preds, col = rev(heat.colors(200)), axes = T, 
		              asp = 1)
		            contour(preds, add = T)
	 				if(pts) points(data[,long],data[,lat],pch=21,bg="lightblue",cex=0.7)
	 				make.legend(preds$z)
		            image(x = preds$x, y = preds$y, z = preds$z.se, 
		              col = rev(heat.colors(200)), xlab = "", ylab = "", 
		              axes = T, asp = 1)
		            contour(x = preds$x, y = preds$y, z = preds$z.se, 
		              add = T)
	 				if(pts) points(data[,long],data[,lat],pch=21,bg="lightblue",cex=0.7)
	 				make.legend(preds$z.se)
		        }

			}
		    fit$preds <- preds
		}
    }

    fit$locale <- locale
	if (!se.omit & se) {
		fit$mark.median <- bxp$stats[3]
		fit$mark.se <- se.bxplot
		fit$mark.yvals <- mark.yvals
	}
	fit$data.name <- data.name    
    fit$data <- data
    attr(fit, "aak") <- aak
    class(fit) <- c("surfer", class(fit))
    fit
}
