`geoM` <-
function (data = manta, ycol = 15, xy = c(12:11), tcol = 3, 
	fname = deparse(substitute(data)), zipname = yname[1], values = "Values", unit = TRUE, 
	k30 = 30, cexp = 1, convert = FALSE, google.run = TRUE, land.add = FALSE,
    pts.add = FALSE,  axes.add = FALSE, scale.bar.add = FALSE, mar.no.rot = rep(0, 4), 
    fit.m = NULL, key = FALSE, key.range = NULL, key.breaks = NULL, lab.nsig = 4, max.leg.col = 41,
    legend = TRUE, leg.minw = 16, leg.col="orange", leg.convert = TRUE, gdriver = "Xlib",
    offset = 1, zip = TRUE, logo = TRUE, logo.file = "/home/omni/reef-data/logo.png",
    sidebar = TRUE, sidebar.file="sidebar.png", sidebar.hsv=c(0,0,0.15,1), 
    about = TRUE,  se = TRUE, se.trim = 0, size = 8, pixel = 19, font.scale = 1,
    theta = 2*pi/9, merge = TRUE, rmKMLs = TRUE, icol = col.g4yob4, ncol = 9,  ...) 
{

	if (is.numeric(ycol)) yname <- names(data)[ycol]
	else yname <- ycol
	if (is.numeric(tcol)) tname <- names(data)[tcol]
	else tname <- tcol
	times <- data[,tcol]
	uni <- sort(unique(times))
	luni <- length(uni)

	cat("\nData set: ",fname,"\n")
	cat("Response: ",yname,"\n")
	cat("Time: ",tname,"\n")
	cat("Unique Times: ",uni,"\n")
	
	if (unit && !is.null(attributes(data)$unit)) {
		if (!is.character(ycol)) values <- attributes(data)$unit[ycol]
		else values <- (1:ncol(data))[match(ycol,attributes(data)$unit)]
	}
	else values <- rep(values,length(ycol)) 
	cat("Units: ",values,"\n")
	
	leg.w <- 0.8*(max(nchar(fname),nchar(yname),nchar(values))+2)
	leg.w <- max(c(leg.w, leg.minw))
	
    opt_exask <- options(example.ask = FALSE)
    qk <- SpatialPointsDataFrame(data[, xy], data)
    summary(Sobj_SpatialGrid(qk)$SG)
    proj4string(qk) <- CRS("+proj=longlat")
    SGqk <- GE_SpatialGrid(qk)

	typetext <- c("Data","Values")
	if (legend) typetext <- c(typetext,"Values-Legend")

	if (!is.null(key.breaks)) ncol <- length(key.breaks) - 1 

	nicol <- 1
	if (is.list(icol)) {
		nicol <- length(icol)
		icol.lst <- icol
	}
	 
## generate Values for all variables  ##

	pred.lims <- NULL
	if (is.null(fit.m)) {
		fit <- list()
		for (i in 1:luni) {
			cat("\nTimer = ",uni[i],"\n")
		    temp <- data[times==uni[i],]
		    fit[[i]] <- surfer.gam.gi(temp[,ycol] ~ s(across, along, k=k30), 
			lat = names(data)[xy[2]], long = names(data)[xy[1]], data = temp, 
			se = se, se.trim = se.trim, ...)
		}
	}
	else {
		fit <- fit.m
		cat("Using provided fit\n")
	}

	if (is.null(key.breaks)) {
		for (i in 1:luni) pred.lims <- range(c(pred.lims,fit[[i]]$preds$z),na.rm=T)
		key.breaks <- seq(min(pred.lims),max(pred.lims),length=ncol+1)
	}

    for (i in 1:luni) {
		fname.png <- paste(fname,"-",yname,"-",uni[i],".png", sep = "")
		if (.Platform$GUI == "X11") png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent",type=gdriverS)
		else png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent")
		par(mar = c(0, 0, 0, 0), xaxs = "i", yaxs = "i")
		line.height <- 0.0012*pixel
		if (nicol>1) col <- icol.lst[[i]]
		if (is.function(icol)) icol <- icol(ncol)
		geoPlot4k(qk, preds = fit[[i]]$preds, xlim = SGqk$xlim, ylim = SGqk$ylim, 
	    cexp = size * cexp, land.add = land.add, axes.add = axes.add, 
	    pts.add = pts.add, scale.bar.add = scale.bar.add, mar.no.rot = mar.no.rot, 
	    key = key, key.range = key.range, image.breaks = key.breaks, image.col = icol, 
	    ncol = ncol, ...)
		dev.off()
		kmlOverlayMovie(SGqk,paste(fname,"-",yname,"-movie.kml",sep=""),fname.png,name=yname,
		uni[i],first=(i==1),last=(i==luni),adj=0,order = 4)
		if (convert) {
		    cat("Converting ..... \n\n")
		    system(paste("convert -transparent white", fname.png,fname.png, sep = " "))
		}
    }
	
	invisible(fit)
}

