geoPies <- function(data, labels=1, longlat=c(2,3), vars=c(4:ncol(data)), txt,
	title=deparse(substitute(data)), eps=0.5, size=3, piecol, 
	minsize=0.1, google.run=FALSE, FUN) {

#
# "data" is the name of the data set
# "labels" is the column number of the labels on each pie
# "longlat" are the column numbers of the long & lat respectively
# "vars" are the column numbers of the varaibles in the pies charts
# "size" controls the size of pies multiplicatively
# "piecol" uses alternate color schemes : eg rainbow or terrain.colors
# "minsize" is the minimum size of pies
#

col2hex <- function (cname) 
{
    colMat <- col2rgb(cname)
    rgb(red = colMat[1, ]/255, green = colMat[2, ]/255, blue = colMat[3, 
        ]/255)
}

data.name <- deparse(substitute(data))
outfile <- paste(data.name,".kml",sep="")
nvars <- length(vars)
ncases <- nrow(data)

labs <- as.character(data[,labels])
ll <- data[,longlat]
xdata <- data[,vars]
xlabs <- names(xdata)

if (!missing(FUN)) xdata <- FUN(xdata)

if (missing(piecol)) 
	piecol <- col2hex(c("firebrick","gold","olivedrab","darkorange","skyblue","greenyellow", "darkslateblue","salmon","slategray","burlywood"))

coltxt <- paste(casefold(substring(piecol[1:nvars],2,7)),collapse=",")

mins <- apply(xdata,2,min)
if (any(mins<0)) {
	cat("Negative values in data: exiting\n")
	stop()
}

sums <- rowSums(xdata)
if (any(sums==0)) {
	cat("Zero row totals in data\n")
	s0 <- sums==0
#	sums[s0] <- eps
}
else s0 <- rep(F,nrow(xdata))

xdata[!s0,] <- 100*xdata[!s0,]/rep(sums[!s0],nrow(data))

sizes <- sqrt(size*(sums/max(sums) + minsize))
sizes[s0] <- eps

if (!missing(txt)) txt <- paste(txt,"<br><br> Total of all values: ",signif(sums,4),"<br>")
else txt <- paste(" Total of all values: ",signif(sums,4),"<br>")

# begin KML

cat(
'<?xml version="1.0" encoding="UTF-8"?>
<kml xmlns="http://www.opengis.net/kml/2.2" xmlns:gx="http://www.google.com/kml/ext/2.2" xmlns:kml="http://www.opengis.net/kml/2.2" xmlns:atom="http://www.w3.org/2005/Atom">
<Document>
 <name>\n',file=outfile) 
cat(data.name,file=outfile,append=TRUE)
cat('</name>\n\n',file=outfile,append=TRUE)

# begin loop for each pie

for (i in 1:ncases) {

cat('<Style id="',file=outfile,append=TRUE)
cat(labs[i],file=outfile,append=TRUE)
cat('_icon">
  <IconStyle>
   <scale>',file=outfile,append=TRUE)
cat(sizes[i],file=outfile,append=TRUE)
cat('</scale>
   <Icon>
    <href>http://chart.apis.google.com/chart?&amp;cht=p&amp;chd=t:',file=outfile,append=TRUE)
cat(paste(signif(xdata[i,]+eps,4),collapse=","),file=outfile,append=TRUE)

# original
# cat('&amp;chs=46x46&amp;chf=bg,s,00000000&amp;chco=',file=outfile,append=TRUE)
# variable res
cat('&amp;chs=',file=outfile,append=TRUE)
pie.size <- round(sizes[i]*30)
cat(paste(pie.size,"x",pie.size,sep=""),file=outfile,append=TRUE)
cat('&amp;chf=bg,s,00000000&amp;chco=',file=outfile,append=TRUE)

cat(coltxt,file=outfile,append=TRUE)
cat('</href>
   </Icon>
  </IconStyle>
  <ListStyle>
  </ListStyle>
 </Style>

 <Placemark>
  <name>',file=outfile,append=TRUE)
cat(labs[i],file=outfile,append=TRUE)
cat('</name>
  <description><![CDATA[',file=outfile,append=TRUE)
cat(txt[i],file=outfile,append=TRUE)
cat('<br/> <img src="http://chart.apis.google.com/chart?&cht=p&chd=t:',file=outfile,append=TRUE)
cat(paste(signif(xdata[i,],4),collapse=","),file=outfile,append=TRUE)
cat('&chs=470x200&chf=bg,s,00000000&chco=',file=outfile,append=TRUE)
cat(coltxt,file=outfile,append=TRUE)
cat('&chl=',file=outfile,append=TRUE)
cat(paste(paste(xlabs,"%20",signif(xdata[i,],4),sep=""),collapse="|"),file=outfile,append=TRUE)
cat('  " width="470" height="200" />]]></description>',file=outfile,append=TRUE)
# or for  header above pies
# cat('&chtt=Area proportional to values" width="470" height="200" />]]></description>',file=outfile,append=TRUE)
cat('<styleUrl>#',file=outfile,append=TRUE)
cat(labs[i],file=outfile,append=TRUE)
cat('_icon</styleUrl>
  <Point>
   <coordinates>',file=outfile,append=TRUE)
cat(ll[i,2],ll[i,1],sep=",",file=outfile,append=TRUE)
cat(',0</coordinates>
  </Point>
 </Placemark>\n\n',file=outfile,append=TRUE) 

}
 
# end loop for each pie

cat('</Document>
</kml>\n',file=outfile,append=T)

# end KML 

if (google.run & .Platform$OS.type=="unix")  {
	outfile <- paste(getwd(), "/", outfile, sep = "")
	system(paste("googleearth ", outfile, " &", sep = ""))
}
# invisible(data)

}
