`geoPlot` <-
function (data, fit, locale = c("GBR", "WA")[1], rotate = FALSE, alpha = 40,
	bound = boundary.aa,	cent = ifelse(locale=="GBR",c(147,-19),c(121.35,-14.05)), 
	cexp = par()$cex, interactive = FALSE, xlim, ylim, axes.add = T, 
	pts.add = TRUE, pch.d = 21, cex.d = cexp * 0.8, col.d = "black", 
	bg.d = "lightblue", auto.cex = TRUE, xlab = "Longitude", ylab = "Latitude",
	lat = "lat", long = "long", bgrnd.add = FALSE, bgrnd.col = "lightcyan", 
    snames = rownames(data), snames.add = FALSE, snames.cex = cexp * 0.8,
    land = qld.land, land.add = TRUE, land.col = "wheat", land.line.col = NA,
    island = qld.islands, island.add = TRUE, island.col = "wheat", island.line.col = NA,     
	reefs, reefs.add = FALSE, reefs.col = "grey90", reefs.line.col = NA,
    layer = zoning, layer.col, layer.add = FALSE, layer.do = rep(TRUE, length(layer)),
    layer.border = NA,
    stars.add = FALSE, stars = data, stars.vars, stars.key = c(0.86, 0.86),
    stars.len = 1, stars.full = TRUE, stars.col = rainbow(length(stars.vars)), 
    pick.pos = 0, towns = towns6, towns.add = FALSE, towns.pch = 16, 
    towns.col = "black", towns.cex = cexp, towns.pos = 2, towns.offset = 0.5,
    rivers = rivers9, rivers.add = FALSE, rivers.pch = 16, rivers.col = "darkblue",
    rivers.cex = cexp, rivers.pos = 2,
    image.add = TRUE, image.se = FALSE, image.col = colorRampPalette(c("green4",
    "yellow", "orange", "firebrick3"), space = "Lab"), breaks,
    image.ncol = 9, image.quant, image.expand = TRUE, image.filter,
    cont.add = FALSE, cont.col = "black", cont.cex = cexp * 0.8, cont.levs, nlevs = 10,
    scale.bar.add = T, scale.bar.x = c(0.05, 0.3), scale.bar.y = 0.025, 
	mar.rot = rep(1, 4), mar.no.rot = c(3.5, 4, 1, 1), 
    mgap = c(2, 0.5, 0), north = FALSE, arrow.x = 0.9, arrow.y = 0.95, 
    key = TRUE, key.fun, key.y = c(0.75, 0.95), key.x = c(0.86, 0.9), key.horiz = FALSE, 
    key.ntxt = ifelse(image.ncol<10,image.ncol+1,10),  
    se = FALSE, nbr.fit = 0, qprob.fit = (1:(nbr.fit - 1))/nbr.fit, nbr.se = 0,
    qprob.se = (1:(nbr.se - 1))/nbr.se, hrnge = c(0, 1), srnge = c(0, 1), 
    vrnge = c(0, 1), X11.type = c("Xlib","cairo")[1], ...) 
{
	if (!missing(breaks)) {
		key.breaks <- breaks
		image.ncol <- length(breaks)-1
	}
	
	if (.Platform$GUI == "X11") {
		old.X11.type <- X11.options()$type
		on.exit(X11.options(type=old.X11.type))
		X11.options(type=X11.type)
	}

    if (layer.add | reefs.add) 
        require(layers)

	bg.fun <- function(col="slategray1") {
		xpd <- par()$usr[1:2]
		ypd <- par()$usr[3:4]
		polygon(c(xpd,rev(xpd)),rep(ypd,each=2),col = col,border = NA)
	}
	expand <- function(x, eps = 0.01) {
        x + eps * c(-1, 1) * diff(x)
    }
    quant <- function(x, qq, qnt = c(0.6,0.9)) {
		if (missing(qq)) qq <- quantile(x[!is.na(x)], prob=qnt)
		cat("Quantiles = ", qq, "\n")
		xx <- x
		xx[!is.na(xx)] <- 1
		for (i in 1:length(qq)) xx[x>qq[i]] <- i + 1 
		xx
	}
    hms <- function(x, type = c("lat", "long")[1], eps = 10^(-6)) {
        if (type == "lat") {
            if (x[1] < 0) {
                nsew <- "S"
                x <- -x
            }
            else nsew <- "N"
        }
        else {
            x <- abs(x)
            nsew <- "E"
        }
        x <- x + eps
        degs <- floor(x)
        tmins <- x - degs
        mins <- floor(tmins * 60 + eps)
        secs <- floor((tmins * 60 - mins + eps) * 60)
        degs.ch <- as.character(degs)
        nch <- nchar(degs.ch)
        degs.ch[nch < max(nch)] <- paste("0", degs.ch[nch < max(nch)], 
            sep = "")
        mins.ch <- as.character(mins)
        nch <- nchar(mins.ch)
        mins.ch[nch == 1] <- paste("0", mins.ch[nch == 1], sep = "")
        secs.ch <- as.character(secs)
        nch <- nchar(secs.ch)
        secs.ch[nch == 1] <- paste("0", secs.ch[nch == 1], sep = "")
        if (all(mins == 0)) 
            paste(degs.ch, "°", nsew, sep = "")
        else if (diff(range(x)) > 0.25) 
            paste(degs.ch, "°", mins.ch, "'", nsew, sep = "")
        else paste(degs.ch, "°", mins.ch, "'", secs.ch, "''", 
            nsew, sep = "")
    }
    scale.bar <- function(scx = scale.bar.x, scy = scale.bar.y) {
        xi <- par()$usr[1:2]
        xii <- c(sum(xi * c(1 - scx[1], scx[1])), sum(xi * c(1 - 
            scx[2], scx[2])))
        dfx <- diff(range(xi))
        scz <- diff(range(scx))
        dfxii <- scz * dfx * 111
        opts <- c(outer(c(1, 2, 5), 10^(-3:3)))
        which <- sum(dfxii > opts)
        opts.lab <- paste(c(outer(c(1, 2, 5), 10^c(0:2, 0:3))), 
            c(rep(c("m", "km"), c(9, 12))), sep = "")
        yi <- par()$usr[3:4]
        yii <- yi[1] + scy * diff(range(yi))
        xii[2] <- xii[1] + opts[which]/111
        lines(xii, c(yii, yii), lwd = 1)
        lines(c(xii[1], xii[1]), c(yii, yii + 0.015 * diff(range(yi))), 
            lwd = 1)
        lines(c(xii[2], xii[2]), c(yii, yii + 0.015 * diff(range(yi))), 
            lwd = 1)
        text(mean(xii), yii, opts.lab[which], pos = 3, cex = cexp * 
            0.85)
    }
    key.pos <- function(kx = key.x, ky = key.y, predz = fit$preds$z,
    		key.col = key.image.c, breaks = key.breaks, key.horiz, key.fun)    {
        if (!missing(breaks)) {
#        	cat("Key breaks ",round(breaks,2),"\n")
#		   	cat("Keys ",length(breaks)," Colors ",length(key.col),"\n")
            minp <- min(breaks)
            maxp <- max(breaks)
        }
        else {
            minp <- min(predz, na.rm = T)
            maxp <- max(predz, na.rm = T)
        }

		if (key.horiz) { nr <- 50; nc <- 1 }
		else { nr <- 1; nc <- 50 }		
        xi <- par()$usr[1:2]
        xii <- xi[1] + kx * diff(range(xi))
        yi <- par()$usr[3:4]
        yii <- yi[1] + ky * diff(range(yi))
		
		if (missing(breaks))
	        image(z = matrix(seq(minp, maxp, len = 50), nrow = nr, ncol = nc),
            x = xii, y = seq(yii[1], yii[2], len = 50), col = key.col, 
            xlab = "", ylab = "", axes = T, add = T)
		else 
			image(z = matrix(seq(minp, maxp, len = 50), nrow = nr, ncol = nc),
			x = xii, y = seq(yii[1], yii[2], len = 50), col = key.col, 
			xlab = "", ylab = "", axes = T, add = T, breaks = breaks)

        if (!missing(key.fun)) 
        	txt <- signif(key.fun(seq(minp,maxp,length=key.ntxt)),3)
		else txt <- signif(seq(minp,maxp,length=key.ntxt),3)
        text(xii[2] + 1.5*par()$cxy[1], seq(yii[1],yii[2],length=key.ntxt), txt,
		cex = cexp * 0.8)
    }
    arrow.north <- function(x = arrow.x, y = arrow.y, rot = rotate, 
        alp = alpha, len = 1, angle = 30, lwd = 2, lty = 1, col = 1, 
        cex.arr = 1.25) {
        xi <- par()$usr[1:2]
        dfx <- diff(range(xi))
        xii <- xi[1] + x * dfx
        yi <- par()$usr[3:4]
        dfy <- diff(range(yi))
        yii <- yi[1] + y * dfy
        len.arr <- 0.025 * len * max(par()$pin)
        len <- abs(0.025 * len * max(dfy, dfx))
        if (!(rot)) 
            alp <- 0
        theta <- pi * alp/180
        st <- sin(theta)
        ct <- cos(theta)
        x0 <- xii - len * st
        y0 <- yii - len * ct
        x1 <- xii + len * st
        y1 <- yii + len * ct
        arrows(x0, y0, x1, y1, len = len.arr, angle = angle, 
            lwd = lwd, lty = lty, col = col)
        par(srt = rotate)
        text(x0, 0.7 * y0 + 0.3 * y1, "N", cex = cex.arr * cexp)
        par(srt = 0)
    }
    fit.se.cols <- function(preds, hr = c(0, 1), sr = c(0, 1), 
        vr = c(0, 1), image.add = TRUE, axes = FALSE) {
        nrow <- length(preds$x)
        ncol <- length(preds$y)
        n <- nrow * ncol
        fit <- c((preds$z - min(preds$z, na.rm = T))/diff(range(preds$z, 
            na.rm = T)))
        fit[is.na(fit)] <- 0
        fit <- diff(hr) * fit + hr[1]
        se <- c(preds$z.se)
        se <- rank(se, na = "keep")
        se <- 1 - (se - min(se, na.rm = T))/diff(range(se, na.rm = T))
        sr <- diff(sr) * se + sr[1]
        vr <- diff(vr) * se + vr[1]
        vr[is.na(se)] <- 1
        sr[is.na(se)] <- 0
        image(x = preds$x, y = preds$y, z = matrix(1:n, nrow = nrow), 
            col = hsv(h = fit, s = sr, v = vr), add = image.add)
    }

    if (locale=="GBR")  land <- qld.land
	if (class(data)[[1]]=="surfer") {
		fit <- data
		data <- fit$data
	}
 	else if (missing(data) & !missing(fit)) data <- fit$data
 
    if (reefs.add & missing(reefs)) 
    if (locale=="GBR") reefs <- gbr.reefs else reefs <- scott.reef

#    if (!missing(fit)) {
#        if (linkinv) {
#        		fit$preds$z <- fit$family$linkinv(fit$preds$z)
#        	if (fit$family$family=="quasibinomial" | fit$family$family=="binomial")
#        		fit$preds$z <- 100*(fit$preds$z)
#        	}
#        if (!is.null(attrib <- attr(fit, "alpha"))) 
#            alpha <- attrib
#        if (!is.null(attrib <- attr(fit, "cent"))) {
#            cent <- attrib
#            rotate <- TRUE
#        }
#    }

    if (rotate) 
        margins <- mar.rot
    else margins <- mar.no.rot
    	old.par <- par(xpd = F, mar = margins, cex.axis = 0.8 * cexp, 
        cex.lab = 0.9 * cexp, mgp = mgap)
	    on.exit(old.par)
    if (alpha != 0 & rotate) {
        cat("Rotating data sets\n")
        if (!missing(data)) {
            cent <- apply(data[, c(long, lat)], 2, mean)
            data[, c(long, lat)] <- ll2pm(data[, c(long, lat)], 
                alpha = alpha, cent = cent)
        }
		if (!missing(fit)) {
			fit$preds <- rotate.grid(fit$preds, alpha=alpha, cent=cent, bound = bound)
		}
        if (land.add)
        	land <- ll2pm.polylist(land, alpha = alpha, cent = cent)
        if (island.add)
        	island <- ll2pm.polylist(island, alpha = alpha, cent = cent)
		if (reefs.add) 
            reefs <- ll2pm.polylist(reefs, alpha = alpha, cent = cent)
        if (layer.add) 
            layer <- ll2pm.polylist(layer, alpha = alpha, cent = cent)
        if (towns.add) 
            towns[, c(long, lat)] <- ll2pm(towns[, c(long, lat)], 
                alpha = alpha, cent = cent)
        if (stars.add) 
            stars[, c(long, lat)] <- ll2pm(stars[, c(long, lat)], 
                alpha = alpha, cent = cent)
        if (rivers.add) 
            rivers[, c(long, lat)] <- ll2pm(rivers[, c(long, 
                lat)], alpha = alpha, cent = cent)
    }
    if (!missing(xlim) & !missing(ylim)) 
        xylims0 <- list(x = xlim, y = ylim)
    else if (!is.null(data) & (missing(xlim) | missing(ylim))) {
        xlim <- expand(range(data[, long]))
        ylim <- expand(range(data[, lat]))
        xylims0 <- list(x = xlim, y = ylim)
    }
    else if (!missing(data) & (missing(xlim) | missing(ylim))) {
        xlim <- expand(range(data[, long]))
        ylim <- expand(range(data[, lat]))
        xylims0 <- list(x = xlim, y = ylim)
    }
    else {
        if (locale == "GBR") {
        xlim <- c(142, 153); ylim <- c(-25, -12)
        }
        else {
        xlim <- c(111, 156); ylim <- c(-47, -7)
        }
        if (alpha != 0 & rotate) {
            temp <- ll2pm(cbind(xlim, ylim), alpha = alpha)
            xlim <- temp[, 1]
            ylim <- temp[, 2]
        }
        xylims0 <- list(xlim = xlim, ylim = ylim)
    }
    if (!missing(fit)) {
        if (rotate) {
            ret <- plot(range(fit$preds$x), range(fit$preds$y), 
                axes = F, asp = 1, xlab = "", ylab = "", type = "n", 
                xlim = xlim, ylim = ylim, 
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
        }
        else {
            ret <- plot(range(fit$preds$x), range(fit$preds$y), 
                asp = 1, xlab = ifelse(axes.add, xlab, ""),
                ylab = ifelse(axes.add, ylab, ""),
                type = "n", axes = F, xlim = xlim, ylim = ylim,
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
            if (axes.add) {
		        axis(1, at = axTicks(1), lab = hms(axTicks(1), type = "long"))
		        axis(2, at = axTicks(2), lab = hms(axTicks(2), type = "lat"))
            }
        }
        if (se) {
            if (is.null(fit$preds$z.se)) 
                stop("No SEs available\n")
            else fit$preds$z <- fit$preds$z.se
        }
        if (image.se & image.add) {
            cont.add <- FALSE
            if (nbr.fit > 0) 
                fit$preds$z <- quant(fit$preds$z, qnt = qprob.fit)
            if (nbr.se > 0) 
                fit$preds$z.se <- quant(fit$preds$z.se, qnt = qprob.se)
            fit.se.cols(fit$preds, hr = hrnge, sr = srnge, vr = vrnge, 
                image.add = T, axes = F)
            len <- ifelse(nbr.fit, nbr.fit, 50)
            image.c <- hsv(h = seq(hrnge[1], hrnge[2], length = len), 
                s = 1, v = 1)
        }
        else {
            if (!missing(image.filter)) 
            	fit$preds$z[fit$preds$z <= image.filter] <- NA
            if (!missing(breaks)) {
                if ((min(breaks) > min(fit$preds$z, na.rm = T)) | 
                  (max(breaks) < max(fit$preds$z, na.rm = T))) {
                  cat("Triming on range\n")
                  fit$preds$z[fit$preds$z < min(breaks)] <- NA
                  fit$preds$z[fit$preds$z > max(breaks)] <- NA
                }
                nmin <- (1:length(breaks))[breaks < min(fit$preds$z,na.rm = T)] 
                nmax <- (1:length(breaks))[breaks > max(fit$preds$z,na.rm = T)]
#                cat("nmin ",nmin, " nmax ",nmax,"\n")
                breaks <- breaks[max(nmin):min(nmax)]
                ncols <- max(nmin):(min(nmax)-1)
                if (class(image.col) == "function") { 
		              image.c <- image.col(image.ncol)[ncols]
		              key.image.c <- image.col(image.ncol)
                }
                else {
					image.c <- image.col[ncols]
		            key.image.c <- image.col
                }

#            	cat("imagencol ",image.ncol,"\n")
#            	cat("Breaks ",breaks,"\n")
#            	cat("Ncols ",ncols,"\n")            	
#            	cat("Lengthcols ",length(image.c),"\n")
#            	cat("KeyLengthcols ",length(key.image.c),"\n")
            }
            else {
				key.breaks <- breaks <- seq(min(fit$preds$z, na.rm = T),max(fit$preds$z, 
				na.rm = T),	length=image.ncol+1)
                if (class(image.col) == "function") 
                	image.c <- image.col(image.ncol) 
                else image.c <- image.col
            	key.image.c <- image.c
            }
            if (image.add) {
#				if (missing(image.quant))
#	                image(fit$preds, col = image.c, axes = F, add = T)
				if (!missing(breaks))
	                image(fit$preds, col = image.c, axes = F, add = T, breaks = breaks)
        		else {
        			breaks <- quantile(fit$preds$z,probs=image.quant,na.rm=T) 	
	                cat("Breaks: ",signif(breaks,3),"\n")
	                image(fit$preds, col = image.c, axes = F, add = T, breaks = breaks)
	            }
        	}        }
        if (cont.add) {
            if (missing(cont.levs)) 
                cont.levs <- pretty(range(fit$preds$z, na.rm = T), 
                  nlevs)
            cat("Levels ", cont.levs, "\n")
            contour(fit$preds, add = T, vfont = NULL, labcex = cont.cex, 
                levels = cont.levs, col = cont.col)
        }
    }

    if (missing(fit)) {
        if (rotate) {
            ret <- plot(0, 0, xlim = xlim, ylim = ylim, axes = F, 
                asp = 1, xlab = "", ylab = "", type = "n",
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
        }
        else {
            ret <- plot(0, 0, xlim = xlim, ylim = ylim, asp = 1,
                 xlab = ifelse(axes.add, xlab, 
                  ""), ylab = ifelse(axes.add, ylab, ""), 
                type = "n", axes = F, panel.first = if (bgrnd.add) 
                  bg.fun(col = bgrnd.col)
                else NULL)
            if (axes.add) {
                axis(1, at = axTicks(1), lab = hms(axTicks(1), 
                  type = "long"))
                axis(2, at = axTicks(2), lab = hms(axTicks(2), 
                  type = "lat"))
            }
        }
    }
    if (auto.cex | image.expand) {
        cex.d0 <- cex.d
        cex.rng <- diff(range(ylim))
    }
   	if (land.add)
		ret <- plot.polylist(land, xlim = xlim, ylim = ylim, xlab = "", 
        ylab = "", col = land.col, border = land.line.col, add = TRUE)
    if (layer.add) {
        if (missing(layer.col) & !is.null(attr(layer, "color"))) 
            layer.col <- attr(layer, "color")
        if (class(layer) == "polylist") {
            plot.polylist(layer, col = layer.col, border = layer.border, 
            add = TRUE, forcefill = FALSE, do = layer.do, xlim = xlim, 
            ylim = ylim, ...)
        }
        else {
            for (i in 1:length(layer)) plot.polylist(layer[[i]], 
                col = layer.col[i], add = TRUE, forcefill = FALSE,
                border = layer.border, do = layer.do, xlim = xlim, ylim = ylim, ...)
        }
    }
    if (reefs.add) 
        plot.polylist(reefs, xlim = xlim, ylim = ylim, xlab = "", 
        ylab = "", col = reefs.col, border = reefs.line.col, add = TRUE)
   	if (island.add)
		ret <- plot.polylist(island, xlim = xlim, ylim = ylim, xlab = "", 
        ylab = "", col = island.col, border = island.line.col, add = TRUE)


    if (!missing(fit) & key & image.add){
#    	if (missing(image.quant))  
		if (missing(breaks))  
			key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = key.image.c, 
			key.horiz = key.horiz, key.fun = key.fun)
		else  
			key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = key.image.c, 
			key.horiz = key.horiz, key.fun = key.fun, breaks=key.breaks)
	}
    if (axes.add) box()
    if (!missing(data)) {
        if (pts.add) {
            points(data[, long], data[, lat], pch = pch.d, cex = cex.d, 
                col = col.d, bg = bg.d)
        }
        if (stars.add) {
            stars.len <- stars.len * 0.015 * (diff(range(ylim)) + 
                diff(range(xlim)))
            stars.data <- apply(stars[, stars.vars], 2, scale01)
            stars.ord <- rev(order(apply(stars.data, 1, sum)))
            xi <- sum(par()$usr[1:2] * c(1 - stars.key[1], stars.key[1]))
            yi <- sum(par()$usr[3:4] * c(1 - stars.key[2], stars.key[2]))
            stars.key.pos <- c(xi, yi)
            stars(stars.data[stars.ord, ], location = stars[stars.ord, 
                c(long, lat)], add = TRUE, len = stars.len, labels = NULL, 
                draw.segments = TRUE, full = stars.full, col.segments = stars.col, 
                cex = cexp * 0.75, key.loc = stars.key.pos, key.xpd = TRUE)
        }
    }
    if (towns.add) {
        points(towns[, c(long, lat)], pch = towns.pch, col = towns.col, 
            cex = towns.cex)
        text(towns[, long], towns[, lat], towns[, 1], col = towns.col, 
            cex = 0.8 * towns.cex, pos = towns.pos, offset = towns.offset)
    }
    if (rivers.add) {
        points(rivers[, c(long, lat)], pch = rivers.pch, col = rivers.col, 
            cex = rivers.cex)
        text(rivers[, long], rivers[, lat], rivers[, 1], col = rivers.col, 
            cex = 0.8 * rivers.cex, pos = rivers.pos)
    }
    if (snames.add) 
        add.names(data, long, lat, snames, snames.cex, pos = pick.pos)
    if (north) 
        arrow.north(x = arrow.x, y = arrow.y)
    if (scale.bar.add) 
        scale.bar(scale.bar.x, scale.bar.y)
    if (interactive) {
        xy <- 1
        while (length(xy)) {
            xy <- loc.pts(data, xc = long, yc = lat, sn = snames, 
                cex = snames.cex)
            if (is.null(xy)) 
                return(invisible(list(xlim = xlim, ylim = ylim)))
            xlim <- xy$xlim
            ylim <- xy$ylim
            if (is.na(xlim[1])) 
                xlim <- xylims0$x
            if (is.na(ylim[1])) 
                ylim <- xylims0$y
            plot(xlim, ylim, xlim = xlim, ylim = ylim, axes = F, 
                asp = 1, xlab = ifelse(axes.add, xlab, ""),
                ylab = ifelse(axes.add, ylab, ""), type = "n",
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
            if (!rotate & axes.add) {
                axis(1, at = axTicks(1), lab = hms(axTicks(1), 
                  type = "long"))
                axis(2, at = axTicks(2), lab = hms(axTicks(2), 
                  type = "lat"))
            }
            if (!missing(fit)) {
                if (image.expand) {
                  nexp <- floor(log2(cex.rng/abs(diff(range(ylim))))) + 
                    1
                  if (nexp > 1) {
                    preds.new <- fit$preds
                    for (i in 1:(nexp - 1)) preds.new <- mat.exp(preds.new, 
                      par()$usr[1:2], par()$usr[3:4])
                    prng <- range(fit$preds$z, na.rm = T)
                    new.prng <- range(preds.new$z, na.rm = T)
                    min1 <- round((image.ncol - 1) * (new.prng[1] - 
                      prng[1])/diff(prng)) + 1
                    max1 <- round((image.ncol - 1) * (new.prng[2] - 
                      prng[1])/diff(prng)) + 1
                    if (image.add) 
                      image(preds.new, col = image.c[min1:max1], 
                        axes = F, add = T)
                    if (cont.add) 
                      contour(preds.new, add = T, vfont = NULL, 
                        labcex = cont.cex, levels = cont.levs)
                  }
                  else {
                    if (image.add) 
                      image(fit$preds, col = image.c, axes = F, 
                        add = T)
                    if (cont.add) 
                      contour(fit$preds, add = T, vfont = NULL, 
                        labcex = cont.cex, levels = cont.levs)
                  }
                }
                else {
                  if (image.add) 
                    image(fit$preds, col = image.c, axes = F, 
                      add = T)
                  if (cont.add) 
                    contour(fit$preds, add = T, vfont = NULL, 
                      labcex = cont.cex, levels = cont.levs, 
                      col = cont.col)
                }
            }
		   	if (land.add)
        	    plot.polylist(land, xlim = xlim, ylim = ylim, xlab = "", 
                ylab = "", col = land.col, border = land.line.col, add = T)
            if (layer.add) {
                if (class(layer) == "polylist") {
                  plot.polylist(layer, col = layer.col, add = TRUE, 
                  border = layer.border, forcefill = FALSE, 
                    do = layer.do, xlim = xlim, ylim = ylim, 
                    ...)
                }
                else {
                  for (i in length(layer)) plot(layer[[i]], col = layer.col[i], 
                    add = TRUE, border = layer.border, forcefill = FALSE, 
                    do = layer.do, xlim = xlim, ylim = ylim, ...)
                }
            }
            if (reefs.add) 
                plot.polylist(reefs, xlim = xlim, ylim = ylim, xlab = "", 
                  ylab = "", col = reefs.col, border = reefs.line.col, add = T)
		   	if (island.add)
				ret <- plot.polylist(island, xlim = xlim, ylim = ylim, xlab = "", 
				ylab = "", col = island.col, border = island.line.col, add = TRUE)

            if (!missing(data)) {
                if (auto.cex & (pts.add | stars.add)) {
                  auto.cex.exp0 <- (cex.rng/diff(range(ylim)))
                  auto.cex.exp1 <- auto.cex.exp0^(1/3)
                  auto.cex.exp2 <- auto.cex.exp0^(1/5)
                }
                if (pts.add) {
                  cex.d <- cex.d0 * ifelse(auto.cex, auto.cex.exp1, 
                    1)
                  points(data[, long], data[, lat], pch = pch.d, 
                    cex = cex.d, col = col.d, bg = bg.d)
                }
                if (stars.add) {
                  xi <- sum(par()$usr[1:2] * c(1 - stars.key[1], 
                    stars.key[1]))
                  yi <- sum(par()$usr[3:4] * c(1 - stars.key[2], 
                    stars.key[2]))
                  stars.key.pos <- c(xi, yi)
                  stars(stars.data[stars.ord, ], location = stars[stars.ord, 
                    c(long, lat)], add = TRUE, len = stars.len * 
                    ifelse(auto.cex, auto.cex.exp0^(-0.75), 1), 
                    labels = NULL, draw.segments = TRUE, full = stars.full, 
                    col.segments = stars.col, cex = cexp * 0.75, 
                    key.loc = stars.key.pos, key.xpd = TRUE)
                }
            }
			if (!missing(fit) & key & image.add){
				if (missing(image.quant))  key.pos(kx = key.x, ky = key.y, predz = fit$preds$z,
				 key.col = image.c, key.horiz = key.horiz,
					 key.fun = key.fun)
				else  key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = image.c,
				 key.horiz = key.horiz,
					 key.fun = key.fun, breaks)
			}
            if (towns.add) {
                points(towns[, c(long, lat)], pch = towns.pch, 
                  col = towns.col, cex = auto.cex.exp2 * towns.cex)
                text(towns[, long], towns[, lat], towns[, 1], 
                  col = towns.col, cex = auto.cex.exp2 * 0.8 * 
                    towns.cex, pos = towns.pos, offset = towns.offset)
            }
            if (rivers.add) {
                points(rivers[, c(long, lat)], pch = rivers.pch, 
                  col = rivers.col, cex = auto.cex.exp2 * rivers.cex)
                text(rivers[, long], rivers[, lat], rivers[, 
                  1], col = rivers.col, cex = auto.cex.exp2 * 
                  0.8 * rivers.cex, pos = rivers.pos)
            }
            if (snames.add) 
                add.names(data, long, lat, snames, auto.cex.exp2 * 
                  snames.cex, pos = pick.pos)
            if (axes.add) box()
            if (north) 
                arrow.north(x = arrow.x, y = arrow.y)
            if (scale.bar.add) 
                scale.bar(scale.bar.x, scale.bar.y)
        }
    }
    if (interactive) 
        xy$cent <- cent
    else xylims0$cent <- cent
    invisible(if (interactive) xy else xylims0)
}
