`geoR.gam` <- function (data = Soft_Corals_Reef, ycol = c(101,7), xy = c(5:4), data2, xy2, 
	fname = deparse(substitute(data)), fname2 = deparse(substitute(data2)),
	zipname = yname[1], values = "Values", unit = TRUE, 
	k30 = 30, google.run = FALSE, fit.m = NULL, lab.nsig = 4, max.leg.col = 41,
    ascii = FALSE, legend = TRUE, leg.minw = 16, leg.col="orange", gdriver = "Xlib",
    offset = 1, zip = TRUE, logo = FALSE, logo.file = "logo.png",
    sidebar = TRUE, sidebar.file="sidebar.png", sidebar.hsv=c(0,0,0.15,1), 
    about = FALSE,  se = FALSE, se.trim = 0, se2 = TRUE, size = 8, pixel = 19, font.scale = 1,
	icol = list(col.g4yob4,col.g4yob4,col.iyorfb4,col.iyorfb4), plot.effects = TRUE, 
    merge = TRUE, rmKMLs = TRUE, ncol = 9, stat = 1, ...) 
{

	if (!missing(data2)) {
		if (is.numeric(ycol[1])) yn1 <- names(data)[ycol[1]]
		else yn1 <- ycol[1]
		if (is.numeric(ycol[2])) yn2 <- names(data2)[ycol[2]]
		else yn2 <- ycol[2]
		yname <- c(yn1,yn2)
	}
	else {
		if (is.numeric(ycol)) yname <- names(data)[ycol]
		else yname <- ycol
	}
	cat("\nData set: ",fname,"\n")
	cat("Responses: ",yname,"\n")

	if (unit && !is.null(attributes(data)$unit)) {
		if (!is.character(ycol)) values <- attributes(data)$unit[ycol]
		else values <- (1:ncol(data))[match(ycol,attributes(data)$unit)]
		values <- c(values," "," ")
	}
	else values <- rep(values,length(ycol)+1) 
	cat("Units: ",values,"\n")
	
	leg.w <- 0.8*(max(nchar(fname),nchar(yname),nchar(values))+2)
	leg.w <- max(c(leg.w, leg.minw))
	
    opt_exask <- options(example.ask = FALSE)
    qk <- SpatialPointsDataFrame(data[, xy], data)
    summary(Sobj_SpatialGrid(qk)$SG)
    proj4string(qk) <- CRS("+proj=longlat")
    SGqk <- GE_SpatialGrid(qk)

## generate Data points KML ##

#	writeOGR(qk, paste(fname,"-Data.kml", sep = ""), ycol, driver = "KML")

	nys <- length(ycol)
	yvars <- c(yname,"Spatial_effects",paste(yname[2],"effects",sep="_"))
	typetext <- c("Data","Values","Values-Legend")

	nicol <- 1
	if (is.list(icol)) {
		nicol <- length(icol)
		icol.lst <- icol
	}
	 
## generate Values for all variables  ##

	if (is.null(fit.m)) {
		if (missing(data2)) {
			xlims <- range(data[,xy[1]])
			ylims <- range(data[,xy[2]])
		}
		else {
			xlims <- range(c(data[,xy[1]],data2[,xy2[1]]))
			ylims <- range(c(data[,xy[2]],data2[,xy2[2]]))
		}

	    fit <- list()
		for (i in 1:nys) {
			cat("\nVar = ",yvars[i],"\n")
		    if (i==2 && !missing(data2)) 
				fit[[i]] <- surfer.gam.gi(data2[, ycol[i]] ~ s(across, along, k=k30), 
				lat = names(data2)[xy2[2]], long = names(data2)[xy2[1]], data = data2, 
				se = se, se.trim = se.trim, xlim = xlims, ylim = ylims, ...)
			else 
				fit[[i]] <- surfer.gam.gi(data[, ycol[i]] ~ s(across, along, k=k30), 
				lat = names(data)[xy[2]], long = names(data)[xy[1]], data = data, 
				se = se, se.trim = se.trim, xlim = xlims, ylim = ylims, ...)
		}

			form <- formula(data[,ycol[1]]~s(across,along,k=k30)+s(x,k=k30/3))

		if (missing(data2)) {
			data$x <- data[,ycol[2]]
			cov.preds <- c(fit[[2]]$preds$z)[!is.na(c(fit[[2]]$preds$z))]
			fit[[3]] <- 
			surfer.gam.gix(form,  data=data, lat = names(data)[xy[2]], long = names(data)[xy[1]],
			se = se2, se.trim = se.trim, cov.preds = cov.preds, ptype = "terms", xlim = xlims,
			ylim = ylims, ...)
		}
		else  {
			data$x <- predict(fit[[2]], newdata=data.frame(acrossAlong(data[,xy])))
			cov.preds <- c(fit[[2]]$preds$z)[!is.na(c(fit[[2]]$preds$z))]
			fit[[3]] <- 
			surfer.gam.gix(form, data=data, lat = names(data)[xy[2]], long = names(data)[xy[1]],
			se = se2, se.trim = se.trim, fit2 = fit[[2]], ptype = "terms", xlim = xlims, 
			ylim = ylims, ...)
		}
# adjust the mean
		fit[[3]]$preds$z <- fit[[3]]$preds$z + coef(fit[[3]])[1]
	}
	else {
		fit <- fit.m
		cat("Using provided fit\n")
	}

    for (i in 1:4) {
		if (i == 4) {se <- TRUE; index <- i - 1}
		else index <- i
		ynm <- yvars[i]
		fname.png <- paste(fname,"-",ynm,".png", sep = "")
		fname.leg.png <- paste(fname,"-",ynm,"-Legend.png", sep = "")
		if (.Platform$GUI == "X11") png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent",type=gdriver)
		else png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent")
		par(mar = c(0, 0, 0, 0), xaxs = "i", yaxs = "i")
		line.height <- 0.0012*pixel
		if (nicol>1) {
		icol <- icol.lst[[i]]
		}
		if (is.function(icol)) icol <- icol(ncol)
		llims <- min(max.leg.col,ncol)
		if (i<4) 
			lims <- seq(min(fit[[i]]$preds$z,na.rm=T),max(fit[[i]]$preds$z,na.rm=T),
			length=llims+1)
		else { 
			lims <- seq(min(fit[[3]]$preds$z.se,na.rm=T),max(fit[[3]]$preds$z.se,na.rm=T),
			length=llims+1)
		}

		labs <- formatter(lims,lab.nsig)
		labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
		fname.leg.png <- paste(fname,"-",ynm,"-Legend.png",sep="")
		if (i < 3)  {
			head <- gsub("_"," ",c(fname,ynm,values[i]))
			nh <- 3
		}
		else {
			head <- gsub("_"," ",ynm)
			nh <- 1
		}
		make.legend(labs=labs, head=head,nh=nh,w=leg.w,
		cols=icol, name.png=fname.leg.png, npt=size*9, leg.col=leg.col)	
		png.size <- pixsize(fname.leg.png)
		xsze <- 0.55*font.scale*pixel*leg.w
		ysze <- xsze*png.size[2]/png.size[1]
		make.legend.kml(xpos=0.0,ypos=0.98-(i-1)*(1+llims-0.3*(i==4))*line.height,
		xsize=xsze, ysize=ysze,	file=paste(fname,"-",ynm,"-Legend",sep=""))
		geoPlot4k(qk, preds = fit[[index]]$preds, xlim = SGqk$xlim, ylim = SGqk$ylim, 
		image.breaks = lims, image.col = icol, image.ncol = ncol, se = se, ...)
		dev.off()
		kmlOverlay2(SGqk,paste(fname,"-",ynm,".kml",sep ="") ,fname.png,name=ynm,adj=(i-1)*3,
		order = 4)
		if (ascii) write.fit.asc(fit[[i]], paste(fname,"-",ynm,"-R.asc", sep = ""))
    }
	if (plot.effects) {
		png(file = "effects.png", pointsize=24, bg="ivory3")
		par(mar=c(2.5,2.5,0.5,0.5),cex=1,col="blue4",col.main="blue4",col.axis="blue4",
		col.lab="blue4",lwd=2,font=2,mgp=c(1.5,0.5,0))
		plot(fit[[3]],se=T,select=2,xlab=yvars[2],ylab="Effect",lwd=2,rug=FALSE)
		dev.off()
	}
	if (sidebar) sidebar.do.png(sidebar.hsv,file=sidebar.file)
	if (merge) {
		gname <-  paste(getwd(),"/",fname,"-",yvars,".kml",sep="")
	    if (legend)  gname <-  c(gname,paste(getwd(),"/",fname,"-",yvars,"-Legend.kml",sep=""))
		outfile <- paste(fname,"-R.kml",sep="")
		mergeR(outfile, gname, yvars, about=about, fname=fname, logo=logo, sidebar=sidebar,
		sidebar.width=xsze, effects = plot.effects)
		zfname <- paste(getwd(),"/work/",fname,"-",yvars[1],"-",yvars[2],"-Rgam.kmz",sep="")
		if (zip) {
			if (logo) system(paste("zip -j ",zfname," ",fname,"-R.kml ",fname,
			"*.png ", logo.file, " ", sidebar.file, sep=""))
			else system(paste("zip -j ",zfname," ",fname,"-R.kml ",fname,"*.png ",
			sidebar.file, sep=""))
			if (plot.effects) system(paste("zip -j ",zfname," effects.png",sep=""))
		}
		if (rmKMLs) 
		system(paste("rm ",fname,"*.kml ",fname,"*.png sidebar.png effects.png",sep=""))
		}
		
    if (google.run & merge)  
			system(paste("googleearth",zfname,"&",sep=" "))
	invisible(fit)
}

