`geoR.rank` <- function (data = Soft_Corals_Reef, ycol =  c(101,7), xy = c(5:4), xy2, data2, 
	fname = deparse(substitute(data)), fname2 = deparse(substitute(data2)),
	zipname = yname[1], values = "Values", unit = TRUE, k30 = 30, google.run = FALSE,
    fit.m = NULL, lab.nsig = 4, max.leg.col = 41, link=FALSE, 
    ascii = FALSE, legend = TRUE, leg.minw = 16, leg.col="orange", gdriver = "Xlib",
    offset = 1, zip = TRUE, logo = TRUE, logo.file = "logo.png",
    sidebar = TRUE, sidebar.file="sidebar.png", sidebar.hsv=c(0,0,0.15,1), 
    about = FALSE,  se = TRUE, se.trim = 0, size = 8, pixel = 19, font.scale = 1,
    theta = 2*pi/9, merge = TRUE, rmKMLs = TRUE, icol = list(col.g4yob4,col.g4yob4,col.bwr),
    ncol = 9, stat = 1,  ...) 
{

	if (!missing(data2)) {
		if (is.numeric(ycol[1])) yn1 <- names(data)[ycol[1]]
		else yn1 <- ycol[1]
		if (is.numeric(ycol[2])) yn2 <- names(data2)[ycol[2]]
		else yn2 <- ycol[2]
		yname <- c(yn1,yn2)
	}
	else {
		if (is.numeric(ycol)) yname <- names(data)[ycol]
		else yname <- ycol
	}
	cat("\nData set(s): ",fname,"\n")
	cat("Responses: ",yname,"\n")
	yname <- c(yname,"Relate")

	if (unit && !is.null(attributes(data)$unit)) {
		if (!is.character(ycol)) values <- attributes(data)$unit[ycol]
		else values <- (1:ncol(data))[match(ycol,attributes(data)$unit)]
		values <- c(values,"Correlation")
	}
	else values <- c(rep(values,length(ycol)),"Correlation") 
	cat("Units: ",values,"\n")
	
	leg.w <- 0.8*(max(nchar(fname),nchar(yname),nchar(values))+2)
	leg.w <- max(c(leg.w, leg.minw))
	
    opt_exask <- options(example.ask = FALSE)
    qk <- SpatialPointsDataFrame(data[, xy], data)
    summary(Sobj_SpatialGrid(qk)$SG)
    proj4string(qk) <- CRS("+proj=longlat")
    SGqk <- GE_SpatialGrid(qk)

## generate Data points KML ##

#	writeOGR(qk, paste(fname,"-Data.kml", sep = ""), ycol, driver = "KML")

	nys <- length(ycol)
	yvars <- yname
	typetext <- c("Data","Values","Values-Legend")

	nicol <- 1
	if (is.list(icol)) {
		nicol <- length(icol)
		icol.lst <- icol
	}
	 
## generate Values for all variables  ##

	if (is.null(fit.m)) {
	    fit <- list()
		for (i in 1:nys) {
			cat("\nVar = ",yvars[i],"\n")
		    if (i==2 && !missing(data2)) 
		    fit[[i]] <- surfer.gam.gi(data2[, ycol[i]] ~ s(across, along, k=k30), 
			lat = names(data2)[xy2[2]], long = names(data2)[xy2[1]], data = data2, 
			se = se, se.trim = se.trim, ...)
			else 
		    fit[[i]] <- surfer.gam.gi(data[, ycol[i]] ~ s(across, along, k=k30), 
			lat = names(data)[xy[2]], long = names(data)[xy[1]], data = data, 
			se = se, se.trim = se.trim, ...)
		}
	}
	else {
		fit <- fit.m
		cat("Using provided fit\n")
	}

## do the "rank correlations" ##

## trim preds to same size & same grid

# cat(length(fit[[1]]$preds$x),"\n",length(fit[[1]]$preds$y),"\n",dim(fit[[1]]$preds$z),"\n")
# cat(length(fit[[2]]$preds$x),"\n",length(fit[[2]]$preds$y),"\n",dim(fit[[2]]$preds$z),"\n")

	x1k <- fit[[1]]$preds$x %in% fit[[2]]$preds$x 
	y1k <- fit[[1]]$preds$y %in% fit[[2]]$preds$y 
	fit[[1]]$preds$x <- fit[[1]]$preds$x[x1k]
	fit[[1]]$preds$y <- fit[[1]]$preds$y[y1k]
	fit[[1]]$preds$z <- fit[[1]]$preds$z[x1k,y1k]

	x2k <- fit[[2]]$preds$x %in% fit[[1]]$preds$x 
	y2k <- fit[[2]]$preds$y %in% fit[[1]]$preds$y 
	fit[[2]]$preds$x <- fit[[2]]$preds$x[x2k]
	fit[[2]]$preds$y <- fit[[2]]$preds$y[y2k]
	fit[[2]]$preds$z <- fit[[2]]$preds$z[x2k,y2k]

#	cat("Dims:", dim(fit[[1]]$preds$z)," ",dim(fit[[2]]$preds$z),"\n")

	fit[[3]] <- fit[[1]]		 	
	fit[[3]]$preds$z <- matrix(NA,nrow=nrow(fit[[3]]$preds$z),ncol=ncol(fit[[3]]$preds$z)) 
	nas <- is.na(fit[[1]]$preds$z) | is.na(fit[[2]]$preds$z)
	x1 <- rank(fit[[1]]$preds$z[!nas])
	x2 <- rank(fit[[2]]$preds$z[!nas])
	x1 <- (x1-min(x1))/(max(x1)-min(x1))
	x2 <- (x2-min(x2))/(max(x2)-min(x2))
	x12 <- x1-x2
#	cat("x12 ",summary(x12),"\n")
	corr <- cor(x1,x2)
#	cat("corr ",corr <- signif(corr,3),"\n")
	fit[[3]]$preds$z[!nas] <- x12	 
#	cat(summary(c(fit[[3]]$preds$z)),"\n")
	fit[[3]]$family$link <- "identity"	

    for (i in 1:3) {
		ynm <- yvars[i]
		fname.png <- paste(fname,"-",ynm,".png", sep = "")
		fname.leg.png <- paste(fname,"-",ynm,"-Legend.png", sep = "")
		if (.Platform$GUI == "X11") png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent",type=gdriver)
		else png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent")
		par(mar = c(0, 0, 0, 0), xaxs = "i", yaxs = "i")
		line.height <- 0.0012*pixel
		if (nicol>1) {
		icol <- icol.lst[[i]]
		}
		if (is.function(icol)) icol <- icol(ncol)
		llims <- min(max.leg.col,ncol)
		if (link)  {
			lnk <- fit[[i]]$family$linkfun
			lnkinv <- fit[[i]]$family$linkinv
			lims <- lnkinv(seq(lnk(min(fit[[i]]$preds$z,na.rm=T)),
				lnk(max(fit[[i]]$preds$z,na.rm=T)),length=llims+1))
		}
		else lims <- seq(min(fit[[i]]$preds$z,na.rm=T),max(fit[[i]]$preds$z,na.rm=T),
			length=llims+1)
		labs <- formatter(lims,lab.nsig)
		labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
		fname.leg.png <- paste(fname,"-",ynm,"-Legend.png",sep="")
		if (i==3) head <- gsub("_"," ",c(fname,ynm,paste(values[i],signif(corr,3))))
		else head <- gsub("_"," ",c(fname,ynm,values[i]))
		make.legend(labs=labs, head=head,nh=3,w=leg.w,
		cols=icol, name.png=fname.leg.png,
		npt=size*9,leg.col=leg.col)	
		png.size <- pixsize(fname.leg.png)
		xsze <- 0.55*font.scale*pixel*leg.w
		ysze <- xsze*png.size[2]/png.size[1]
		make.legend.kml(xpos=0.0,ypos=0.98-(i-1)*(4+llims)*line.height,
		xsize=xsze, ysize=ysze,	file=paste(fname,"-",ynm,"-Legend",sep=""))
		geoPlot4k(qk, preds = fit[[i]]$preds, xlim = SGqk$xlim, ylim = SGqk$ylim, 
		image.breaks = lims, image.col = icol, image.ncol = ncol, ...)
		dev.off()
		kmlOverlay2(SGqk,paste(fname,"-",ynm,".kml",sep ="") ,fname.png,name=ynm,adj=(i-1)*3,
		order = 4)
		if (ascii) write.fit.asc(fit[[i]], paste(fname,"-",ynm,"-R.asc", sep = ""))
    }
	
	if (sidebar) sidebar.do.png(sidebar.hsv,file=sidebar.file)
	if (merge) {
		gname <-  paste(getwd(),"/",fname,"-",yvars,".kml",sep="")
	    if (legend)  gname <-  c(gname,paste(getwd(),"/",fname,"-",yvars,"-Legend.kml",sep=""))
		outfile <- paste(fname,"-R.kml",sep="")
		mergeR(outfile, gname, yvars, about=about, fname=fname, logo=logo, sidebar=sidebar,
		sidebar.width=xsze, effects=FALSE)
		zfname <- paste(getwd(),"/work/",fname,"-",yvars[1],"-",yvars[2],"-Rrank.kmz",sep="")
		if (zip) {
			if (logo) system(paste("zip -j ",zfname," ",fname,"-R.kml ",fname,
			"*.png ", logo.file, " ", sidebar.file, sep=""))
			else system(paste("zip -j ",zfname," ",fname,"-R.kml ",fname,"*.png " ,
			sidebar.file, sep=""))
		}
		if (rmKMLs) system(paste("rm ",fname,"*.kml ",fname,"*.png sidebar.png",sep=""))
		}
		
    if (google.run & merge)  
			system(paste("googleearth",zfname,"&",sep=" "))
	invisible(fit)
}

