geoTrim <-
function (fit, plt=TRUE, plt.size=6, pts=TRUE, pch = 21, bg = "lightblue", se.omit=3, 
	mark.se=10:0, icol.vals = col.g4yob4, icol.se = col.yorr4, 
	ncol=9,	contour.add = FALSE, nsigs=4, link = TRUE, ...) 
{
	if (.Platform$OS.type == "unix") GFUN <- x11 else GFUN  <- windows

	formatter <- function(x,ndig=5,eps=10*.Machine$double.eps) {
		n10 <- floor(log10(abs(x)+eps))*sign(x)
		xx <- rep("",length(x))
		for (i in unique(n10)) 
		xx[n10==i] <- formatC(x[n10==i],ndig-i-1,format="f")
		xx[nchar(xx)>ndig] <- substring(xx[nchar(xx)>ndig],1,ndig)
		nch <- nchar(xx)
		dots <- substring(xx,nch,nch)=="."
		xx[dots] <- substring(xx[dots],1,nch[dots]-1) 
		xx
	}

	make.legend <- function(x,cex=2.7,pch=15,cols) {
		rnge <- range(x,na.rm=T)
		rvals <- seq(max(rnge),min(rnge),length=length(cols))
		rvals <- formatter(rvals,nsigs)
		legend("topright",rvals,col=cols,bty="n",pch=pch,pt.cex=cex,cex=0.85*par()$cex)
	}

	data <- attr(fit,"data")
	if (class(fit)[1]=="abt" & class(fit)[2]=="multi") preds <- fit[[1]]$preds
	else preds <- fit$preds
	mark.yvals <- attr(fit,"mark.yvals")
	mark.median <- attr(fit,"mark.median")
	mark.se.bxplot <- attr(fit,"mark.se.bxplot")
	mark.se <- attr(fit,"mark.se")
		
	if (is.null(preds$z.se)) stop("Fit object does not have SEs !!")
	
	mark.nas <- preds$z.se > mark.median +  se.omit*mark.se.bxplot 
	cat("Omitting", round(100 * sum(mark.nas, 
        na.rm = T)/sum(!is.na(preds$z.se)), 2), "% pixels on high SEs\n")
    preds$z[mark.nas] <- NA
    preds$z.se[mark.nas] <- NA
	mark.n <- length(mark.yvals)
	
	if (plt) {
		GFUN(xpos=0,ypos=0,wid=0.7*plt.size,hei=plt.size,title="SE Distributions")
	    par(mfrow = c(1, 1))
		boxplot(predict(fit,newdata=data,se=T,wid=0.5)$se,NA,preds$z.se,
		names=c("Data","SE","Grid"))
		points(rep(2,mark.n),mark.yvals,col="blue",pch=3,cex=2)					
		text(rep(2.2,mark.n),mark.yvals,mark.se)
	}

	if (plt) {
		GFUN(xpos=35,ypos=35,wid=1.6*plt.size,hei=plt.size,
		  title="Images of Predictions & SEs")
        par(mfrow = c(1, 2),mar=c(3,3,1,1))
        image(preds, col = icol.vals(ncol), axes = T, asp = 1)
        if(contour.add) contour(preds, add = T)
		if(pts) points(data$long,data$lat,pch=pch,bg=bg,...)
		make.legend(preds$z, cols=rev(icol.vals(ncol)))
        image(x = preds$x, y = preds$y, z = preds$z.se, 
          col = icol.se(ncol), xlab = "", ylab = "", 
          axes = T, asp = 1)
        if(contour.add) contour(x = preds$x, y = preds$y, z = preds$z.se, 
          add = T)
		if(pts) points(data$long,data$lat,pch=pch,bg=bg,...)
		make.legend(preds$z.se, cols=rev(icol.se(ncol)))
    }
    fit$preds <- preds
    fit
}
