`geoV` <- function (data = Soft_Corals_Reef, ycol = 101:99, xy = c(5:4), 
	fname = deparse(substitute(data)),
	zipname = yname[1], values = "Values", unit = TRUE, npos=3, weights, mylims, 
	k30 = 30, cexp = 1, google.run = FALSE, fit.m = NULL,lab.nsig = 4, max.leg.col = 41,
    ascii = FALSE, leg.minw = 16, leg.col="orange", gdriver = "Xlib", link=FALSE,
    offset = 1, zip = TRUE, logo = TRUE, logo.file = "logo.png",
    sidebar = TRUE, sidebar.file="sidebar.png", sidebar.hsv=c(0,0,0.15,1), 
    about = FALSE,  se = TRUE, se.trim = 0, size = 8, pixel = 19, font.scale = 1,
    theta = 2*pi/9, merge = TRUE, rmKMLs = TRUE, ncol = 9, icol = col.g4yob4,
    family = c("gaussian","quasipoisson","quasibinomial")[1],  ...) 
{

	if (is.numeric(ycol)) yname <- names(data)[ycol]
	else yname <- ycol
	cat("\nData set: ",fname,"\n")
	cat("Response: ",yname,"\n")

	if (unit && !is.null(attributes(data)$unit)) {
		if (!is.character(ycol)) values <- attributes(data)$unit[ycol]
		else values <- (1:ncol(data))[match(ycol,attributes(data)$unit)]
	}
	else values <- rep(values,length(ycol)) 
	cat("Units: ",values,"\n")
	
	leg.w <- 0.8*(max(nchar(fname),nchar(yname),nchar(values))+2)
	leg.w <- max(c(leg.w, leg.minw))
	
    opt_exask <- options(example.ask = FALSE)
    qk <- SpatialPointsDataFrame(data[, xy], data)
    summary(Sobj_SpatialGrid(qk)$SG)
    proj4string(qk) <- CRS("+proj=longlat")
    SGqk <- GE_SpatialGrid(qk)

## generate Data points KML ##

#	writeOGR(qk, paste(fname,"-Data.kml", sep = ""), ycol, driver = "KML")

	nvars <- length(ycol)
	yvars <- names(data)[ycol]
	typetext <- c("Data","Values","Values-Legend")

#	families <- c("gaussian","quasipoisson","quasibinomial")

	if (length(family)==1) family <- rep(family,nvars)
	if (length(link)==1) link <- rep(link,nvars)
	
#   colours

	nicol <- 1
	if (is.list(icol)) {
		nicol <- length(icol)
		icol.lst <- icol
	}
	 
## generate Values for all variables  ##

	if (is.null(fit.m)) {
	    fit <- list()
		for (i in 1:nvars) {
			cat("\nVar = ",yvars[i]," Family = ",family[i],"\n")
		    form <- formula(paste(yvars[i],"~ s(across, along, k=",k30,")",sep=" "))
		    if (missing(weights)) 
		    fit[[i]] <- surfer.gam.gi(form, lat = names(data)[xy[2]], long = names(data)[xy[1]],
		    ptype="response", data = data, se = se, se.trim = se.trim, family=family[i],...)
		    else 
		    fit[[i]] <- surfer.gam.gi(form, lat = names(data)[xy[2]], long = names(data)[xy[1]],
		    weights = weights, ptype="response", data = data, se = se, se.trim = se.trim,
		    family=family[i], ...)
		}
	}
	else {
		fit <- fit.m
		cat("Using provided fit\n")
	}

    for (i in 1:nvars) {
		yname <- yvars[i]
		fname.png <- paste(fname,"-",yname,".png", sep = "")
		fname.leg.png <- paste(fname,"-",yname,"-Legend.png", sep = "")
		if (.Platform$GUI == "X11") png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent",type="Xlib")
		else png(file = fname.png, width=size * SGqk$width,
				 height=size*SGqk$height, bg="transparent")
		par(mar = c(0, 0, 0, 0), xaxs = "i", yaxs = "i")
		line.height <- 0.0012*pixel

		if (nicol>1) {
			icol <- icol.lst[[i]]
		}
		if (is.function(icol)) icol <- icol(ncol)

		llims <- min(max.leg.col,ncol)
		if (link[i])  {
			lnk <- fit[[i]]$family$linkfun
			lnkinv <- fit[[i]]$family$linkinv
			if (missing(mylims)) lims <- lnkinv(seq(lnk(min(fit[[i]]$preds$z,na.rm=T)),
				lnk(max(fit[[i]]$preds$z,na.rm=T)),length=llims+1))
			else lims <- seq(lnk(mylims[1]),lnk(mylims[2]),length=llims+1)
		}
		else if (missing(mylims)) lims <- seq(min(fit[[i]]$preds$z,na.rm=T),
			max(fit[[i]]$preds$z,na.rm=T), length=llims+1)
		else lims <- seq(mylims[1],mylims[2],length=llims+1)

		labs <- formatter(lims,lab.nsig)
		labs <- paste(labs[1:llims],labs[2:(llims+1)],sep="-")
		fname.leg.png <- paste(fname,"-",yname,"-Legend.png",sep="")
		head <- gsub("_"," ",c(fname,yname,values[i]))
		make.legend(labs=labs, head=head,nh=3,w=leg.w,
		cols=icol, name.png=fname.leg.png,npt=size*9,leg.col=leg.col)	
		png.size <- pixsize(fname.leg.png)
		xsze <- 0.55*font.scale*pixel*leg.w
		ysze <- xsze*png.size[2]/png.size[1]
		for (j in 1:npos) make.legend.kmlV(xpos=0.0,ypos=0.98-(j-1)*(4+llims)*line.height,
		xsize=xsze, ysize=ysze,	filename=paste(fname,"-",yname,"-",j,"-Legend",sep=""),
		filename.leg.png=fname.leg.png)
		geoPlot4k(qk, preds = fit[[i]]$preds, xlim = SGqk$xlim, ylim = SGqk$ylim, 
	    image.breaks = lims, image.col = icol, ...)
		dev.off()
		for (j in 1:npos)
			kmlOverlay2(SGqk,paste(fname,"-",yname,"-",j,".kml",sep ="") ,fname.png,name=yname,
			adj=(j-1)*3, order = 4, visibility = as.numeric(i==j))
		if (ascii) write.fit.asc(fit[[i]], paste(fname,"-V.asc", sep = ""))
    }
	if (sidebar) sidebar.do.png(sidebar.hsv,file=sidebar.file)
	if (merge) {
		gname <-  paste(getwd(),"/",fname,"-",outer(yvars,1:npos,paste,sep="-"),".kml",sep="")
		lgname <-  paste(getwd(),"/",fname,"-",outer(yvars,1:npos,paste,sep="-"),
		"-Legend.kml",sep="")
		gname <-  c(gname,lgname)
		outfile <- paste(fname,"-V.kml",sep="")
#	    cat(gname,"\n")
		mergeV(outfile, gname, yvars, nvars=nvars, npos=npos, about=about, fname=fname, 
		logo=logo, sidebar=sidebar, sidebar.width=xsze)
		zfname <- paste(getwd(),"/work/",fname,"-",zipname,"-V.kmz",sep="")
		if (zip) {
			if (logo) system(paste("zip -j ",zfname," ",fname,"-V.kml ",fname,
			"*.png ", logo.file, " ", sidebar.file, sep=""))
			else system(paste("zip -j ",zfname," ",fname,"-V.kml ",fname,"*.png ",
			sidebar.file, sep=""))
		}
		if (rmKMLs) system(paste("rm ",fname,"*.kml ",fname,"*.png sidebar.png",sep=""))
		}
		
    if (google.run & merge)  
			system(paste("googleearth",zfname,"&",sep=" "))
	invisible(fit)
}

