`grid.predict` <-
function (fit, grid = grid001.aa, xlim, ylim, xpnd = 0.01, 
	ptype = "response", plt = FALSE, se = TRUE, se.trim = 0, ...) 
{
 	g2i <- function(x, preds, preds.se) {
		ndiv <- attr(x,"ndiv")
		xr <- range(x$long)
		xi <- seq(xr[1],xr[2],by=1/ndiv)
		nxi <- length(xi)
		yr <- range(x$lat)
		yi <- seq(yr[1],yr[2],by=1/ndiv)
		nyi <- length(yi)
		zmat <- matrix(NA,ncol=nyi,nrow=nxi)
		xins <- match(round(x$long*ndiv),round(xi*ndiv))
		yins <- match(round(x$lat*ndiv),round(yi*ndiv))
		zmat[cbind(xins,yins)] <- preds
		lst <- list(x=xi,y=yi,z=zmat)
		if (!missing(preds.se)) {
			zmat[cbind(xins,yins)] <- preds.se
			lst$z.se <- zmat
		}
		lst
	}

    lat <- attr(fit, "latlong")[1]
    long <- attr(fit, "latlong")[2]
	aak <- attr(fit, "aak")
	aafn <- attr(fit, "aafn")
	
    if (missing(xlim) & missing(ylim)) {
        xlim <- range(fit$data[, long])
        ylim <- range(fit$data[, lat])
    }
    if (xpnd > 0) {
        xlim <- xlim + xpnd * c(-1, 1) * diff(xlim)
        ylim <- ylim + xpnd * c(-1, 1) * diff(ylim)
    }
    grid <- grid[grid$long>xlim[1] & grid$long<xlim[2] & grid$lat>ylim[1] & grid$lat<ylim[2],]
    grid$along <- aak * aafn(grid$along)

    if (!se) {
        pd <- predict(fit, newdata = grid, type = ptype)
        preds <- g2i(grid, pd)
    }
    else {
        if (!se.trim) {
	        pd <- predict(fit, newdata = grid, type = ptype, se = T)
	        preds <- g2i(grid, pd$fit, pd$se.fit)
        }
        else {
            pd <- predict(fit, newdata = grid, type = "link", se = T)
	        preds <- g2i(grid, pd$fit, pd$se.fit)
            se.max <- max(predict(fit, se = T, type = "link")$se.fit)
			outs <- preds$z.se > se.trim * se.max
            preds$z[outs] <- NA                
            preds$z.se[outs] <- NA
            preds$se.max <- se.max
        	if (ptype == "link" || fit$family$family != "identity") {
	        	preds$z.se <- (fit$family$linkinv(preds$z+preds$z.se)
				- fit$family$linkinv(preds$z-preds$z.se))/2
	        	preds$z <- fit$family$linkinv(preds$z)
        	}
            cat("Trimming SE max = ",signif(se.max,3)," % outs = ",
            signif(mean(!is.na(outs)),3)," Max SE : ",signif(max(preds$z.se,na.rm=T),3),"\n")
        }
    }
    if (plt) {
        if (!se) {
            image(preds, col = heat.colors(200), axes = T, asp = 1)
            contour(preds, add = T)
        }
        else {
            par(mfrow = c(1, 2))
            image(preds, col = heat.colors(200), axes = T, 
              asp = 1)
            contour(preds, add = T)
            image(x = preds$x, y = preds$y, z = preds$z.se, 
              col = heat.colors(200), xlab = "", ylab = "", 
              axes = T, asp = 1)
            contour(x = preds$x, y = preds$y, z = preds$z.se, 
              add = T)
            par(mfrow = c(1, 1))
        }
    }
    fit$preds <- preds

    attr(fit, "aak") <- aak
    class(fit) <- c("surfer", class(fit))
    fit
}

