`kmlOverlayMovie` <- function (obj, kmlfile = NULL, imagefile = NULL, name = "R image",
	i, first = TRUE, last = TRUE, adj = 0, theta = 2*pi/9, order = 3) 
{
    if (class(obj) != "GE_SG") 
        stop("obj must be of class GE_SG from function GE_SpatialGrid")
    if (is.na(proj4string(obj$SG)) || is.projected(obj$SG)) 
        stop("Spatial* object must be in geographical coordinates")
    if (is.null(imagefile)) {
        imagefile <- "<fill_in_later>"
        warning("image file name missing, edit in manually")
    }
    bbox <- bbox(obj$SG)
    W <- bbox[1, 1]
    E <- bbox[1, 2]
    S <- bbox[2, 1]
    N <- bbox[2, 2]
    if (adj) {
		W <- W + adj*cos(theta)    
		E <- E + adj*cos(theta)    
		S <- S + adj*sin(theta)    
		N <- N + adj*sin(theta)    		
    }
    kmlheader <- c("<?xml version='1.0' encoding='UTF-8'?>", 
        "<kml xmlns='http://earth.google.com/kml/2.0'>\n<Folder>")
    if (order) draw.ord <- paste("<drawOrder>",as.character(order),"</drawOrder>",sep="")
    kmname <- paste("<GroundOverlay>\n<name>", name, "</name>", sep = "")
	ts <- paste("<TimeSpan>\n<begin>",i,"</begin>\n<end>",i+1,"</end>\n</TimeSpan>",sep="\n")
    icon <- paste("<Icon><href>", imagefile, "</href><viewBoundScale>0.75</viewBoundScale></Icon>", 
        sep = "")
    latlonbox <- paste("<LatLonBox><north>", N, "</north><south>", 
        S, "</south><east>", E, "</east><west>", W, "</west></LatLonBox>", 
        sep = "")
    footer <- "</GroundOverlay>\n"
    kmlend <- "</Folder>\n</kml>\n"
    if (first) 	x <- kmlheader
	else x <- scan(kmlfile, what="character")
	x <- append(x, kmname)
    if (order) x <- append(x, draw.ord)
    x <- append(x, ts)
    x <- append(x, icon)
    x <- append(x, latlonbox)
	x <- append(x, footer)
    if (last) x <- append(x, kmlend)
    cat(paste(x, sep = "", collapse = "\n"), file = kmlfile, sep = "\n", append = FALSE) 
}
