map.gen <- function(data=sc07.rf, y="richall", name="soft-coral-richness", ret.frame=FALSE, grid=grid010, plot=TRUE, copy=FALSE, missing = -1, ...) {
	require(geoTools)
	eps <- attr(grid,"eps")
	cat("eps = ",eps,"\n")
	z0 <- surfer.gam.aa(data[,y]~s(across,along),data=data,...)
	if (plot) {
		geoPlot(data,fit=z0)
		mtext(y,line=-2,cex=1.5)
	}
	if (copy) cpng(file=paste(name,".png",sep=""))

	lar <- range(data[,"lat"],na.rm=T)
	lor <- range(data[,"long"],na.rm=T)
	grid.tmp <- grid[grid$lat>lar[1] & grid$lat<lar[2] & grid$long>lor[1] & grid$long<lor[2],]
	pred <- predict(z0,newdata=data.frame(acrossAlong(grid.tmp)),type="response",se=T)
	pgrid <- data.frame(grid.tmp,fit=pred$fit,se.fit=pred$se.fit)
	llong <- seq(min(pgrid$long),max(pgrid$long),by=eps)
	llat <- seq(min(pgrid$lat),max(pgrid$lat),by=eps)
	newgrid <- expand.grid(long=llong,lat=llat)
	newgrid <- merge(newgrid,pgrid,by.x=1:2,by.y=1:2,all.x=TRUE)
	nr <- length(llong)
	nc <- length(llat)
	print(summary(newgrid))
	if (!is.na(missing)) {
		newgrid[is.na(newgrid[,3]),3] <- missing
		newgrid[is.na(newgrid[,4]),4] <- missing
	}	
	fit <- matrix(newgrid[,3],nrow=nr,byrow=T)
	se.fit <- matrix(newgrid[,4],nrow=nr,byrow=T)
	image(fit)
	list(long=llong,lat=llat,fit=fit,se.fit=se.fit,nrows=nr,ncols=nc,eps=eps)
}

