`plot.polylist` <-
function (x, col, border = par("fg"), add = FALSE, xlim = NULL, 
    ylim = NULL, xlab = "", ylab = "", xpd = NULL, density = NULL, 
    angle = 45, pbg = NULL, forcefill = TRUE, do = rep(TRUE, 
        length(x)), ...) 
{
    if (!inherits(x, "polylist")) 
        stop("Not a polygon list")
    usrpoly <- function(x) {
        p <- matrix(c(x[1], x[2], x[2], x[1], x[3], x[3], x[4], 
            x[4]), ncol = 2)
        p
    }
    if (!add) {
        maplim <- attr(x, "maplim")
        if (is.null(maplim)) 
            if (is.null(xlim) || is.null(ylim)) 
                stop("map limits missing")
        if (is.null(xlim)) 
            xlim <- maplim$x
        if (is.null(ylim)) 
            ylim <- maplim$y
        plot(x = xlim, y = ylim, xlim = xlim, ylim = ylim, type = "n", 
            asp = 1, xlab = xlab, ylab = ylab)
        if (!forcefill && !is.null(pbg)) {
            polygon(usrpoly(par("usr")), col = pbg, border = NA)
            box()
        }
    }
    pO <- attr(x, "plotOrder")
    if (length(x) < 1) 
        stop("zero length polylist")
    if (is.null(pO) || length(x) != length(pO)) 
        pO <- 1:length(x)
    pO <- as.integer(pO)
    if (length(pO) != length(unique(pO))) 
        stop("malformed plot order")
    if (!identical(sort(pO), sort(1:length(x)))) 
        stop("malformed plot order")
    if (missing(col) || is.null(col)) {
        if (length(density) != length(x)) {
            density <- rep(density, length(x), length(x))
        }
        if (length(angle) != length(x)) {
            angle <- rep(angle, length(x), length(x))
        }
        for (j in pO[do]) polygonholes(x[[j]], border = border, 
            xpd = xpd, density = density[j], angle = angle[j], 
            pbg = pbg, forcefill = forcefill, ...)
    }
    else {
        if (length(col) != length(x)) {
            col <- rep(col, length(x), length(x))
        }
        for (j in pO[do]) polygonholes(x[[j]], col = col[j], 
            border = border, xpd = xpd, pbg = pbg, forcefill = forcefill, ...)
    }
}

