rotate.grid <- function(grid, alpha = 45, cent, bound, eps, block=1000) {

	require(akima)
	require(makegrid)
	
	ll2pm <- function (x, alpha, cent, lat = 2, long = 1) {
		if (missing(cent)) cent <- apply(x[, c(long, lat)],2,mean)
		sth <- sin(pi * alpha/180)
		cth <- cos(pi * alpha/180)
		x <- scale(x, center = cent, scale = F)
		x[, c(long, lat)] %*% matrix(c(cth, sth, -sth, cth), nrow = 2, 
		    byrow = F)
	}

	if (missing(cent)) cent <- round(c(mean(grid$x),mean(grid$y)),2)
	if (missing(eps)) eps <- min(c(grid$x[2]-grid$x[1],grid$y[2]-grid$y[1]))
	xlim <- c(min(grid$x),max(grid$x))
	ylim <- c(min(grid$y),max(grid$y))

	# convert grid to xyz matrix

	mat <- cbind(x=rep(grid$x,length(grid$y)),y=rep(grid$y,each=length(grid$x)))
	mat <- cbind(mat[!is.na(grid$z),],z=c(grid$z[!is.na(grid$z)]))
#	plot(mat[,1],mat[,2],pch=".")
	
	# rotate the grid values

	mat[,1:2] <- ll2pm(mat[,1:2],alpha,cent,lat=2,long=1)
#	plot(mat[,1],mat[,2],pch=".")
	if (!missing(bound)) 
		bound[,1:2] <- ll2pm(bound[,1:2],alpha,cent,lat=2,long=1)

	# make new grid 

	xr <- range(mat[,1])
	xr1 <- floor(xr[1]/eps)*eps
	xr2 <- ceiling(xr[2]/eps)*eps
	xr <- c(xr1,xr2)	
	yr <- range(mat[,2])
	yr1 <- floor(yr[1]/eps)*eps
	yr2 <- ceiling(yr[2]/eps)*eps
	yr <- c(yr1,yr2)	

	newgrid <- interp(x=mat[,1], y=mat[,2], z=mat[,3], xo=seq(xr1,xr2,by=eps),
	 yo=seq(yr1,yr2,by=eps), extrap=FALSE)
	grd <- expand.grid(newgrid$x,newgrid$y)
	
	if (!missing(bound)) {
		nr <- nrow(grd)
		if (nr > block) {
			pins <- rep(NA,nr)
			n <- ceiling(nr/block)
			for (i in 1:n) {
				inds <- (1 + (i-1)*block):(i*block)
				inds <- inds[inds<=nr]
				pins[inds] <- poly.in(grd[inds,], bound)
				cat(i,"")
			}
			cat("\n")
		}
		else {
			pins <- poly.in(grd,bound)
		}
		newgrid$z[!pins]<-NA
	}
	newgrid
}
