`surfer.abt.gig` <-
function (form, data, weights, grid.predict = TRUE, type=c("aa","ll")[1], cv = 0, single=T, 
	box.trim = TRUE, se = FALSE, se.trim = 1.5, sumry = FALSE, xlim, ylim, 
	bnd = bound, xpnd = 0.01, xdeg.west = 0.15, alpha = 40, rotate = FALSE, cent, 
    plt = TRUE, lat = "lat", long = "long", adj.mean = T, extrap = F, ptype = "response", 
    grid = grid005.aa, multi = FALSE, deg = 10, FUN=mean, lev=1, ...) 
{
  require(abt)
  require(acrossAlong)
  require(geoTools) 
  require(gridaa)
    
	g2i <- function(x, preds, preds.se) {
		ndiv <- attr(x,"ndiv")
		xr <- range(x$long)
		xi <- seq(xr[1],xr[2],by=1/ndiv)
		nxi <- length(xi)
		yr <- range(x$lat)
		yi <- seq(yr[1],yr[2],by=1/ndiv)
		nyi <- length(yi)
		zmat <- matrix(NA,ncol=nyi,nrow=nxi)
		xins <- match(round(x$long*ndiv),round(xi*ndiv))
		yins <- match(round(x$lat*ndiv),round(yi*ndiv))
		zmat[cbind(xins,yins)] <- preds
		lst <- list(x=xi,y=yi,z=zmat)
		if (!missing(preds.se)) {
			zmat[cbind(xins,yins)] <- preds.se
			lst$z.se <- zmat
		}
		lst
	}

   getval <- function(x, FUN=mean, lev=1) {
  	 if (is.factor(x)) levels(x)[lev]
	   else FUN(x)
	}
    if (!missing(cent) && !is.null(attr(cent, "cent"))) {
        lims <- attr(cent, "xylims")
        xlim <- lims$xlim
        ylim <- lims$ylim
        cent <- attr(cent, "cent")
        got <- T
    }
    else got <- F

#	if (type=="aa")   {
	  aa <- acrossAlong(data[, c(long, lat)])
	  data$across <- aa[,"across"]
	  data$along <- aa[,"along"]
#	}

	  if (missing(weights)) fit <- abt(form, data = data, cv=cv, single=single, ...)
	  else fit <- abt(form, data = data, cv=cv, single=single, weights = weights, ...)
	 
	  if (plt) plot.all(fit)
	  
	  if (sumry) print(all.summary(fit))

	if (is.null(names(fit))) mdata <- fit[[1]]$mdata
	else mdata <- fit$mdata 
	
	if (grid.predict) {
		if (alpha != 0 & rotate) {
			cat("Rotating data sets\n\n")
			if (!got & missing(cent)) 
				cent <- apply(data[, c(long, lat)], 2, mean)
			data[, c(long, lat)] <- ll2pm(data[, c(long, lat)], 
			alpha = alpha, cent = cent)
			grid[,1:2] <- ll2pm(grid[,c(long, lat)], alpha = alpha, cent = cent)
		}
        if (!got && (missing(xlim) & missing(ylim))) {
            xlim <- range(data[, long])
            ylim <- range(data[, lat])
        }
        if (!got & xpnd > 0) {
            xlim <- xlim + xpnd * c(-1, 1) * diff(xlim)
            ylim <- ylim + xpnd * c(-1, 1) * diff(ylim)
            if (rotate) 
                xlim[1] <- xlim[1] - xdeg.west
        }
        grid <- grid[grid$long>xlim[1] & grid$long<xlim[2] & grid$lat>ylim[1] & grid$lat<ylim[2],]
		if (box.trim) {
		    hpts <- chull(as.matrix(data[, c(long, lat)]))
		    xbox <- xpdbox(as.matrix(data[c(hpts, hpts[1]), c(long, lat)]))
		    ins <- poly.in(as.matrix(grid[, c(long, lat)]), xbox)
	  		grid <- grid[ins,]  
		}
  		if (alpha != 0 & rotate) {
			land <- ll2pm(landonly, alpha = alpha, cent = cent)
			reef <- ll2pm(outonly, alpha = alpha, cent = cent)
		    north <- ll2pm(nrthonly, alpha = alpha, cent = cent)
			south <- ll2pm(sthonly, alpha = alpha, cent = cent)
			out <- ll2pm(outonly, alpha = alpha, cent = cent)
			grid <- data.frame(acrossAlong(grid, land, reef, north, south, out, check = T))
		}

		if ((nxc <- ncol(mdata)) > 3) {
			xval.df <- data.frame(matrix(NA,ncol=nxc-3,nrow=nx*ny))
			names(xval.df) <- names(mdata)[4:nxc]
			for (i in 4:nxc) {
				xval.df[[i-3]] <- rep(getval(mdata[,i],FUN=FUN, lev=lev),nx*ny)
	  		}
		}
        ndiv <- attr(grid,"ndiv")
		if (nxc>3) grid <- data.frame(grid,xval.df)
        attr(grid,"ndiv") <- ndiv	 

        if (!se) {
            pd <- predict(fit, newdata = grid, type = ptype)
            preds <- g2i(grid, pd)
        }
        else {
            if (!se.trim) {
		        pd <- predict(fit, newdata = grid, type = ptype, se = F)
		        preds <- g2i(grid, pd$preds)
            }
            else {
#
# Note SES are problematic for trimming with abts
#
                pd <- predict(fit, newdata = grid, type = "link", se = T)
		        preds <- g2i(grid, pd$preds, pd$preds.se)
	            preds.fit <- predict(fit, newdata= data, se = T, type = "link",se.smooth="none")
			    print(summary(preds.fit$preds.se))
			    bxp <- boxplot(preds.fit$preds.se,range=se.trim,plot=F)
				se.high <- (bxp$stats[4] + (bxp$stats[4] - bxp$stats[2]) * se.trim) 
			    omit.high <- preds$preds.se > se.high
			    if (any(omit.high)) {
					cat("Omitting",round(100 * sum(omit.high,na.rm=T)/
					sum(!is.na(preds$z.se)),2), "% pixels on high SEs\n")
					preds$z[omit.high] <- NA
					preds$z.se[omit.high] <- NA
					preds$preds.fit <- preds.fit
			    }
            	if (fit$distribution != "guassian") {
		        	preds$z.se <- (fit$family$linkinv(preds$z+preds$z.se)
					- fit$family$linkinv(preds$z-preds$z.se))/2
		        	preds$z <- fit$family$linkinv(preds$z)
            	}
            }
        }

		if (plt) {
		    if (!se) {
		        image(preds, col = heat.colors(200), axes = T, asp = 1)
		        contour(preds, add = T)
		    }
		    else {
		        par(mfrow = c(1, 2))
		        image(preds, col = heat.colors(200), axes = T, asp = 1)
		        contour(preds, add = T)
		        image(x = preds$x, y = preds$y, z = preds$z.se, col = heat.colors(200), 
		            xlab = "", ylab = "", axes = T, asp = 1)
		        contour(x = preds$x, y = preds$y, z = preds$z.se, 
		            add = T)
		        par(mfrow = c(1, 1))
		    }
		}
		fit$preds <- preds
		if (rotate) {
		    attr(fit, "alpha") <- alpha
		    attr(fit, "cent") <- cent
		    attr(fit, "xylims") <- list(xlim = xlim, ylim = ylim)
		}
	}
	class(fit) <- c("surfer",class(fit))
    fit
    }
