surfer.gam.gi2 <-
function (form, data, weights, grid.predict = TRUE, FUN = acrossAlong, land = landonly,
	xlim, ylim, xpnd = 0.02, lat = "lat", long = "long", ptype = "response", plt = TRUE,
	se = TRUE, mark.se = c(2,3), omit.se = FALSE, 
	aauto = FALSE, aauto.vals = c(0.25,0.5,1,2,4,8,12),
    grid = grid001.aa, aak.min = c("fit","best")[1], aak = 4, ...) 
{
    require(mgcv)
    require(acrossAlong)

    g2i <- function(x, preds, preds.se) {
        ndiv <- attr(x, "ndiv")
        xr <- range(x$long)
        xi <- seq(xr[1], xr[2], by = 1/ndiv)
        nxi <- length(xi)
        yr <- range(x$lat)
        yi <- seq(yr[1], yr[2], by = 1/ndiv)
        nyi <- length(yi)
        zmat <- matrix(NA, ncol = nyi, nrow = nxi)
        xins <- match(round(x$long * ndiv), round(xi * ndiv))
        yins <- match(round(x$lat * ndiv), round(yi * ndiv))
        zmat[cbind(xins, yins)] <- preds
        lst <- list(x = xi, y = yi, z = zmat)
        if (!missing(preds.se)) {
            zmat[cbind(xins, yins)] <- preds.se
            lst$z.se <- zmat
        }
        lst
    }

	get.method <- function(form) {
		ch3 <- as.character(form)[[3]]
		if ((length(grep("across",ch3))==1) & (length(grep("along",ch3))==1)) meth <- "aa"
		else if ((length(grep("long",ch3))==1) & (length(grep("lat",ch3))==1)) meth <- "ll"
		else {
			meth <- "none"
			stop("Model MUST include either 'across' AND 'along' OR 'long' AND 'lat' \n")
		}
		meth
	}

	make.legend <- function(x,np=5,cex=2.7,pch=15) {
		rnge <- range(x,na.rm=T)
		rvals <- seq(min(rnge),max(rnge),length=2*np+1)[2*(1:np)]
		rvals <- as.character(signif(rvals,3))
		legend("topright",rvals,col=heat.colors(length(rvals)),bty="n",pch=pch,pt.cex=cex)
	}

	method <- get.method(form)
	if (method=="aa") {
		cat("Calculating across-along\n")
		new.aa <- FUN(data[, c(long, lat)], land = land)
		data$across <- new.aa[, "across"]
		data$along <- new.aa[, "along"]
	}
    if (aauto) {
        nl <- length(aauto.vals)
        res <- matrix(NA, nrow = nl, ncol = 2)
        res[, 1] <- aauto.vals
        dimnames(res) <- list(as.character(aauto.vals), c("aak", 
            "gcv"))
        for (i in 1:nl) {
            cat(i, " ")
            temp <- data
            temp$along <- aauto.vals[i] * new.aa[, "along"]
            if (missing(weights)) 
                fit <- gam(formula = form, data = temp, ...)
            else fit <- gam(formula = form, data = temp, weights = weights, 
                ...)
            res[i, 2] <- fit$gcv.ubre
        }
        minr <- (1:nl)[res[, 2] == min(res[, 2])][1]
        if (aak.min == "best") 
            aak <- res[minr, 1]
        else if (minr == 1 | minr == nl) 
            aak <- res[minr, 1]
        else {
            res3 <- data.frame(res[(minr - 1):(minr + 1), ])
            fit <- lm(gcv ~ poly(aak, 2), res3)
            newdf <- data.frame(aak = seq(min(res3$aak), max(res3$aak), 
                length = 100))
            min.aak <- min(aak.pred <- predict(fit, new = newdf, 
                type = ptype))
            aak <- newdf[aak.pred == min.aak, ]
        }
        cat("\n")
        print(res)
        cat("Best aak =", aak, "\n")
    }
	if (method=="aa") data$along <- aak*data$along
    if (missing(weights)) 
        fit <- gam(formula = form, data = data, ...)
    else fit <- gam(formula = form, data = data, weights = weights, ...)
    print(summary(fit))
 
    if (grid.predict) {
        if (missing(xlim) & missing(ylim)) {
            xlim <- range(data[, long])
            ylim <- range(data[, lat])
        }
        if (xpnd > 0) {
            xlim <- xlim + xpnd * c(-1, 1) * diff(xlim)
            ylim <- ylim + xpnd * c(-1, 1) * diff(ylim)
        }
        grid <- grid[grid$long > xlim[1] & grid$long < xlim[2] & 
            grid$lat > ylim[1] & grid$lat < ylim[2], ]
		if (method=="aa")  grid$along <- aak * grid$along
        nxxs <- length(tlabs <- attributes(fit$terms)$term.labels) - 
            2
        if (!se) {
            pd <- predict(fit, newdata = grid, type = ptype)
            preds <- g2i(grid, pd)
        }
        else {
            if (!mark.se[1]) {
                pd <- predict(fit, newdata = grid, type = ptype, 
                  se = T)
                preds <- g2i(grid, pd$fit, pd$se.fit)
            }
            else {
                pd <- predict(fit, newdata = grid, type = "link", 
                  se = T)
                preds <- g2i(grid, pd$fit, pd$se.fit)
                preds.fit <- predict(fit, se = T, type = "link")
                bxp <- boxplot(preds.fit$se.fit,  plot = F)
				mark.se <- rev(sort(mark.se))
				mark.n <- length(mark.se)
                mark.yvals <- rep(0,mark.n)
                if (mark.se[1]) { 
		            preds$z.mark <- (preds$z>0)*1
					if (omit.se) {
			        	mark <- preds$z.se > bxp$stats[3] + 
			        	min(mark.se)*0.75*(bxp$stats[4]-bxp$stats[2])
						cat("\nOmitting", round(100 * sum(mark, 
				            na.rm = T)/sum(!is.na(preds$z.se)), 2), "% pixels on high SEs\n")
				        preds$z[mark] <- NA
				        preds$z.se[mark] <- NA
					}
		            else for (i in 1:mark.n) {
			        	cat("mark.se =",mark.se[i],"\n")
			        	mark.yvals[i] <- bxp$stats[3] + 
			        	mark.se[i]*0.75*(bxp$stats[4]-bxp$stats[2])
			        	mark <- preds$z.se > mark.yvals[i]
			        	preds$z.mark[mark] <- mark.se[i] 
			        }
                }
            }
        }

        if (plt) {
        	x11(xpos=0,ypos=0,wid=6,hei=6,title="Model Diagnostics")
        	par(mar=c(4,4,2,2))
        	gam.check(fit)

        	x11(xpos=35,ypos=35,wid=6,hei=6,title="Variogram of Residuals")
			va <- variogram(residuals(fit)~1, data=data, formula(paste("~",long,"+",lat)))
			scatter.smooth(va$dist,va$gamma,ylim=c(0,max(va$gamma)),span=0.5,
			xlab="Distance",ylab="Covariance")
			
            if (se) {
			    x11(xpos=70,ypos=70,wid=4,hei=6,title="SE Distributions")
                par(mfrow = c(1, 1))
			    boxplot(predict(fit,newdata=data,se=T)$se,preds$z.se,
			    names=c("Data","Grid"))
            	if (!omit.se) {
		        	points(rep(2,mark.n),mark.yvals,bg=rainbow(mark.n),pch=21,cex=2)
			        text(rep(2,mark.n)+0.15,mark.yvals,mark.se)
	            }
            }
        }

		if (fit$family$family != "identity" & ptype=="response") {
			  if (se) preds$z.se <- (fit$family$linkinv(preds$z+preds$z.se) -
						fit$family$linkinv(preds$z-preds$z.se))/2 
			  preds$z <- fit$family$linkinv(preds$z)
		}

        if (plt) {
            if (!se) {
                x11(xpos=105,ypos=105,wid=6,hei=6,title="Image of Predictions")
                par(mar=c(3,3,1,1))
                image(preds, col = heat.colors(200), axes = T, asp = 1)
                contour(preds, add = T)
 				make.legend(preds$z)
            }
            else {
			    x11(xpos=105,ypos=105,wid=10,hei=6,title="Images of Predictions & SEs")
                par(mfrow = c(1, 2),mar=c(3,3,1,1))
                image(preds, col = heat.colors(200), axes = T, 
                  asp = 1)
                contour(preds, add = T)
 				make.legend(preds$z)
                image(x = preds$x, y = preds$y, z = preds$z.se, 
                  col = heat.colors(200), xlab = "", ylab = "", 
                  axes = T, asp = 1)
                contour(x = preds$x, y = preds$y, z = preds$z.se, 
                  add = T)
 				make.legend(preds$z.se)
            }

		}
        fit$preds <- preds
    }
    attr(fit, "aak") <- aak
    class(fit) <- c("surfer", class(fit))
    fit
}
