surfer.loess.gi <- function (form, data, weights, grid.predict = TRUE, trim = TRUE, 
    FUN = aaCoast, land = ning.coast.smooth, xlim, ylim, lat = "lat", long = "long", 
	plt = FALSE,  se = FALSE, se.trim = 0, xvals, grid = grid001.nc, xpnd=0.02,
	low=low, high=high, ...) 
{
    require(mgcv)
    require(acrossAlong)
    g2i <- function(x, preds, preds.se) {
        ndiv <- attr(x, "ndiv")
        xr <- range(x$long)
        xi <- seq(xr[1], xr[2], by = 1/ndiv)
        nxi <- length(xi)
        yr <- range(x$lat)
        yi <- seq(yr[1], yr[2], by = 1/ndiv)
        nyi <- length(yi)
        zmat <- matrix(NA, ncol = nyi, nrow = nxi)
        xins <- match(round(x$long * ndiv), round(xi * ndiv))
        yins <- match(round(x$lat * ndiv), round(yi * ndiv))
        zmat[cbind(xins, yins)] <- preds
        lst <- list(x = xi, y = yi, z = zmat)
        if (!missing(preds.se)) {
            zmat[cbind(xins, yins)] <- preds.se
            lst$z.se <- zmat
        }
        lst
    }
    aa <- FUN(data[, c(long, lat)],land=land)
    data$across <- aa[, "across"]
    data$along <- aa[, "along"]

    if (missing(weights)) 
        fit <- loess(formula = form, data = data, normalize=FALSE, surface="direct", ...)
    else fit <- loess(formula = form, data = data, weights = weights, surface="direct",
    	normalize=FALSE, ...)
    print(summary(fit))
    if (grid.predict) {
        if (missing(xlim) & missing(ylim)) {
            xlim <- range(data[, long])
            ylim <- range(data[, lat])
        }
        if (xpnd > 0) {
            xlim <- xlim + xpnd * c(-1, 1) * diff(xlim)
            ylim <- ylim + xpnd * c(-1, 1) * diff(ylim)
        }
        grid <- grid[grid$long > xlim[1] & grid$long < xlim[2] & 
            grid$lat > ylim[1] & grid$lat < ylim[2], ]
        if (!se) {
            pd <- predict(fit, newdata = grid, type = ptype)
            preds <- g2i(grid, pd)
        }
        else {
            if (!se.trim) {
                pd <- predict(fit, newdata = grid, type = ptype, 
                  se = T)
                if (!is.null(low)) pd$fit[pd$fit<low] <- low
                if (!is.null(high)) pd$fit[pd$fit>high] <- high
                preds <- g2i(grid, pd$fit, pd$se.fit)
            }
            else {
                pd <- predict(fit, newdata = grid, type = "link", 
                  se = T)
                preds <- g2i(grid, pd$fit, pd$se.fit)
                preds.fit <- predict(fit, se = T, type = "link")
                bxp <- boxplot(preds.fit$se.fit, range = se.trim, 
                  plot = F)
                se.high <- (bxp$stats[4] + (bxp$stats[4] - bxp$stats[2]) * 
                  se.trim)
                omit.high <- preds$z.se > se.high
                if (any(omit.high)) {
                  cat("Omitting", round(100 * sum(omit.high, 
                    na.rm = T)/sum(!is.na(preds$z.se)), 2), "% pixels on high SEs\n")
                  preds$z[omit.high] <- NA
                  preds$z.se[omit.high] <- NA
                  preds$preds.fit <- preds.fit
                }
            }
        }
        fit$preds <- preds
    }
    fit
}
