`geoMap` <-	function(data = list(), shape = list(queensland), fit, control,
	xlim, ylim, fit.se = FALSE, rotate = FALSE, alpha = 40, bound = boundary.aa, 
	interactive = FALSE, axes.add = T, auto.cex = TRUE,  
	xlab = "Longitude", ylab = "Latitude",...) 
{
    require(layers)

### Check arguments for class ###

	if (!missing(data) & class(data)=="data.frame") stop("data MUST be a list of data.frames")
	if (!missing(shape) & any(class(shape)=="polylist")) stop("shape MUST be a list of poly")

### Some local image parameters that are for more exotic plots !!  ###

    nbr.fit <-  0; qprob.fit <-  (1:(nbr.fit - 1))/nbr.fit; nbr.se <-  0;
    qprob.se <-  (1:(nbr.se - 1))/nbr.se; hrnge <-  c(0, 1); srnge <-  c(0, 1); 
    vrnge <-  c(0, 1) 

### unpack controls ###

	extraArgs <- list(...)
    if (length(extraArgs)) {
        controlargs <- names(formals(geoMap.control))
        indx <- match(names(extraArgs), controlargs, nomatch = 0)
        if (any(indx == 0)) 
            stop("Argument ", names(extraArgs)[indx == 0], "not matched")
    }
    controls <- geoMap.control(...)
    if (!missing(control)) 
        controls[names(control)] <- control
	bgrnd.add <- controls$bgrnd.add
	bgrnd.col <- controls$bgrnd.col
	scale.bar.add <- controls$scale.bar.add
    scale.bar.x <- controls$scale.bar.x
	scale.bar.y <- controls$scale.bar.y
	north <- controls$north
	north.x <- controls$north.x
	north.y <- controls$north.y
	mar.rot <- controls$mar.rot
	mar.no.rot <- controls$mar.no.rot
	mgap <- controls$mgap
	key <- controls$key
	key.fun <- controls$key.fun
	key.y <- controls$key.y
	key.x <- controls$key.x
    key.horiz <- controls$key.horiz
	X11.type <- controls$X11.type

### X11 options  ###

	if (.Platform$GUI == "X11") {
		old.X11.type <- X11.options()$type
		on.exit(X11.options(type=old.X11.type))
		X11.options(type=X11.type)
	}
    
### FUNCTION definitions ###

	bg.fun <- function(col="slategray1") {
		xpd <- par()$usr[1:2]
		ypd <- par()$usr[3:4]
		polygon(c(xpd,rev(xpd)),rep(ypd,each=2),col = col,border = NA)
	}
    quant <- function(x, qq, qnt = c(0.6,0.9)) {
		if (missing(qq)) qq <- quantile(x[!is.na(x)], prob=qnt)
		cat("Quantiles = ", qq, "\n")
		xx <- x
		xx[!is.na(xx)] <- 1
		for (i in 1:length(qq)) xx[x>qq[i]] <- i + 1 
		xx
	}
	dms <- function(theta, type = c("latitude", "longitude",
                  "LATITUDE", "LONGITUDE",
                  "Latitude", "Longitude"), eps = 1.0e-6) {
	  type <- tolower(match.arg(type))
	  hemi <- sign(theta)+2
	  if (type == "latitude") nsew <- c("S"," ","N")[hemi]
	  else nsew <- c("W"," ","E")[hemi]
	  th <- abs(theta)+eps
	  deg <- floor(th)
	  min <- floor(60*(th - deg))
	  sec <- floor(60*60*(th - deg - min/60))
	  deg.ch <- as.character(deg)
	  nch <- nchar(deg.ch)
#	  deg.ch[nch < max(nch)] <- paste("0", deg.ch[nch < max(nch)], sep = "")
	  min.ch <- as.character(min)
	  nch <- nchar(min.ch)
	  min.ch[nch == 1] <- paste("0", min.ch[nch == 1], sep = "")
	  sec.ch <- as.character(sec)
	  nch <- nchar(sec.ch)
	  sec.ch[nch == 1] <- paste("0", sec.ch[nch == 1], sep = "")
	  if (all(min == 0)) 
		  paste(deg.ch, "°", nsew, sep = "")
	  else if (diff(range(theta)) > 0.25) 
		  paste(deg.ch, "°", min.ch, "'", nsew, sep = "")
	  else paste(deg.ch, "°", min.ch, "'", sec.ch, "''", 
		         nsew, sep = "")
	}
    scale.bar <- function(scx = scale.bar.x, scy = scale.bar.y) {
        xi <- par()$usr[1:2]
        xii <- c(sum(xi * c(1 - scx[1], scx[1])), sum(xi * c(1 - 
            scx[2], scx[2])))
        dfx <- diff(range(xi))
        scz <- diff(range(scx))
        dfxii <- scz * dfx * 111
        opts <- c(outer(c(1, 2, 5), 10^(-3:3)))
        which <- sum(dfxii > opts)
        opts.lab <- paste(c(outer(c(1, 2, 5), 10^c(0:2, 0:3))), 
            c(rep(c("m", "km"), c(9, 12))), sep = "")
        yi <- par()$usr[3:4]
        yii <- yi[1] + scy * diff(range(yi))
        xii[2] <- xii[1] + opts[which]/111
        lines(xii, c(yii, yii), lwd = 1)
        lines(c(xii[1], xii[1]), c(yii, yii + 0.015 * diff(range(yi))), 
            lwd = 1)
        lines(c(xii[2], xii[2]), c(yii, yii + 0.015 * diff(range(yi))), 
            lwd = 1)
        text(mean(xii), yii, opts.lab[which], pos = 3, cex = par()$cex * 
            0.85)
    }
    key.pos <- function(kx = key.x, ky = key.y, predz = fit$preds$z,
    		key.col = key.image.c, breaks = key.breaks, key.horiz, key.fun)    {
        if (!missing(breaks)) {
            minp <- min(breaks)
            maxp <- max(breaks)
        }
        else {
            minp <- min(predz, na.rm = T)
            maxp <- max(predz, na.rm = T)
        }

		if (key.horiz) { nr <- 50; nc <- 1 }
		else { nr <- 1; nc <- 50 }		
        xi <- par()$usr[1:2]
        xii <- xi[1] + kx * diff(range(xi))
        yi <- par()$usr[3:4]
        yii <- yi[1] + ky * diff(range(yi))
		
		if (missing(breaks))
	        image(z = matrix(seq(minp, maxp, len = 50), nrow = nr, ncol = nc),
            x = xii, y = seq(yii[1], yii[2], len = 50), col = key.col, 
            xlab = "", ylab = "", axes = T, add = T)
		else 
			image(z = matrix(seq(minp, maxp, len = 50), nrow = nr, ncol = nc),
			x = xii, y = seq(yii[1], yii[2], len = 50), col = key.col, 
			xlab = "", ylab = "", axes = T, add = T, breaks = breaks)

        if (!is.na(key.fun)) 
        	txt <- signif(key.fun(seq(minp,maxp,length=key.ntxt)),3)
		else txt <- signif(seq(minp,maxp,length=key.ntxt),3)
        text(xii[2] + 1.5*par()$cxy[1], seq(yii[1],yii[2],length=key.ntxt), txt,
		cex = par()$cex * 0.8)
    }
    arrow.north <- function(x = north.x, y = north.y, rot = rotate, 
        alp = alpha, len = 1, angle = 30, lwd = 2, lty = 1, col = 1, 
        cex.arr = 1.25) {
        xi <- par()$usr[1:2]
        dfx <- diff(range(xi))
        xii <- xi[1] + x * dfx
        yi <- par()$usr[3:4]
        dfy <- diff(range(yi))
        yii <- yi[1] + y * dfy
        len.arr <- 0.025 * len * max(par()$pin)
        len <- abs(0.025 * len * max(dfy, dfx))
        if (!(rot)) 
            alp <- 0
        theta <- pi * alp/180
        st <- sin(theta)
        ct <- cos(theta)
        x0 <- xii - len * st
        y0 <- yii - len * ct
        x1 <- xii + len * st
        y1 <- yii + len * ct
        arrows(x0, y0, x1, y1, len = len.arr, angle = angle, 
            lwd = lwd, lty = lty, col = col)
        par(srt = rotate)
        text(x0, 0.7 * y0 + 0.3 * y1, "N", cex = cex.arr * par()$cex)
        par(srt = 0)
    }
    fit.se.cols <- function(preds, hr = c(0, 1), sr = c(0, 1), 
        vr = c(0, 1), image.add = TRUE, axes = FALSE) {
        nrow <- length(preds$x)
        ncol <- length(preds$y)
        n <- nrow * ncol
        fit <- c((preds$z - min(preds$z, na.rm = T))/diff(range(preds$z, 
            na.rm = T)))
        fit[is.na(fit)] <- 0
        fit <- diff(hr) * fit + hr[1]
        se <- c(preds$z.se)
        se <- rank(se, na = "keep")
        se <- 1 - (se - min(se, na.rm = T))/diff(range(se, na.rm = T))
        sr <- diff(sr) * se + sr[1]
        vr <- diff(vr) * se + vr[1]
        vr[is.na(se)] <- 1
        sr[is.na(se)] <- 0
        image(x = preds$x, y = preds$y, z = matrix(1:n, nrow = nrow), 
            col = hsv(h = fit, s = sr, v = vr), add = image.add)
    }

### END of FUNCTION definitions ###

###

if (length(shape)) 
	for (i in 1:length(shape)) 
        if (is.null(attr(shape[[i]],"color"))) shape[[i]] <- attrSet(shape[[i]])

if (length(data)) 
	for (i in 1:length(data)) 
        if (is.null(attr(data[[i]],"xy"))) data[[i]] <- attrSet(data[[i]])

### START SETUP of PLOTS ###

# Limits of plots #

	if (!missing(xlim) & !missing(ylim)) {
        xylims0 <- list(x = xlim, y = ylim)
        }
    else if (!missing(data) & (missing(xlim) | missing(ylim))) {
		expand <- function(x, eps = 0.01) {
		    x + eps * c(-1, 1) * diff(x)
		}
		longlat <- attr(data[[1]],"xy")
		long <- longlat[1]
		lat <- longlat[2]
        xlim <- expand(range(data[[1]][, long]))
        ylim <- expand(range(data[[1]][, lat]))
    }
    else if (!missing(fit) & (missing(xlim) | missing(ylim))) {
        xlim <- range(fit$preds$x)
        ylim <- range(fit$preds$y)
    }
    else { xlim <- c(141, 154); ylim <- c(-24.5, -11) }
	cent <- c(mean(xlim),mean(ylim))
    xylims0 <- list(x = xlim, y = ylim)

#   Rotation set-up  #

    if (rotate | axes.add==FALSE)  margins <- mar.rot
    else margins <- mar.no.rot
   	old.par <- par(xpd = F, mar = margins, cex.axis = 0.8 * par()$cex, 
    cex.lab = 0.9 * par()$cex, mgp = mgap)
    on.exit(old.par)

    if (alpha != 0 & rotate) {
        cat("Rotating data sets\n")
        temp <- ll2pm(cbind(xlim, ylim), alpha = alpha)
        xlim <- temp[, 1]
        ylim <- temp[, 2]
	    xylims0 <- list(xlim = xlim, ylim = ylim)
        if (!missing(data)) {
 			for (i in 1:length(data)) 
	            data[[i]][, c(long, lat)] <- ll2pm(data[[i]][, c(long, lat)], 
                alpha = alpha, cent = cent)
        }
        if (length(txt)) {
 			for (i in 1:length(txt)) 
	            data[[i]][, c(long, lat)] <- ll2pm(data[[i]][, c(long, lat)], 
                alpha = alpha, cent = cent)
        }
		if (!missing(fit)) {
			fit$preds <- rotate.grid(fit$preds, alpha=alpha, cent=cent, bound = bound)
		}
        if (length(shape)) 
			for (i in 1:length(shape)) 
				shape[[i]] <- ll2pm.polylist(shape[[i]], alpha = alpha, cent = cent)
    }

### START of regular PLOT ###

    if (rotate) {
        	plot(0, 0, xlim = xlim, ylim = ylim, axes = F, 
            asp = 1, xlab = "", ylab = "", type = "n",
            panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
    }
    else {
        	plot(0, 0, xlim = xlim, ylim = ylim, asp = 1, xlab = ifelse(axes.add, xlab,""),
        	ylab = ifelse(axes.add, ylab, ""), type = "n", axes = F, 
        	panel.first = if (bgrnd.add) bg.fun(col = bgrnd.col) else NULL)
        if (axes.add) {
            axis(1, at = axTicks(1), lab = dms(axTicks(1), type = "long"))
            axis(2, at = axTicks(2), lab = dms(axTicks(2), type = "lat"))
        }
    }

### IMAGE & CONTOUR SETUP & PLOT ###

    if (!missing(fit)) {
        if (is.null(attr(fit,"image.breaks"))) fit <- attrSet(fit)
		image.add <- TRUE # attr(fit,"image.add")
		image.breaks <- attr(fit,"image.breaks")
		image.se <-	attr(fit,"image.se")
		image.col <- attr(fit,"image.col") 
		image.quant	<- attr(fit,"image.quant")
		contour.add <- attr(fit,"contour.add")
		contour.col <- attr(fit,"contour.col")
		contour.cex <- attr(fit,"contour.cex")
		contour.levs <- attr(fit,"contour.levs")
		contour.nlevs <- attr(fit,"contour.nlevs")
		
		image.expand=TRUE

        if (fit.se) {
            if (is.null(fit$preds$z.se)) 
                stop("No SEs available\n")
            else fit$preds$z <- fit$preds$z.se
        }
	    if (image.se & image.add) {
	        contour.add <- FALSE
	        if (nbr.fit > 0) 
	            fit$preds$z <- quant(fit$preds$z, qnt = qprob.fit)
	        if (nbr.se > 0) 
	            fit$preds$z.se <- quant(fit$preds$z.se, qnt = qprob.se)
	        fit.se.cols(fit$preds, hr = hrnge, sr = srnge, vr = vrnge, 
	            image.add = T, axes = F)
	        len <- ifelse(nbr.fit, nbr.fit, 50)
	        image.c <- hsv(h = seq(hrnge[1], hrnge[2], length = len), 
	            s = 1, v = 1)
	    }
	    else {
	        if (image.add) {
	         	if (length(image.breaks)>1) {
					cat(image.breaks,"\n")
					key.breaks <- image.breaks
					image.ncol <- length(image.breaks) - 1
			        if ((min(image.breaks) > min(fit$preds$z, na.rm = T)) | 
			          (max(image.breaks) < max(fit$preds$z, na.rm = T))) {
			          cat("Triming on range\n")
			          fit$preds$z[fit$preds$z < min(image.breaks)] <- NA
			          fit$preds$z[fit$preds$z > max(image.breaks)] <- NA
			        }
			        if (class(image.col) == "function") { 
					      image.c <- image.col(image.ncol)
					      key.image.c <- image.col(image.ncol)
			        }
			        else {
						image.c <- image.col[ncols]
					    key.image.c <- image.col
			        }
			    }
			    else {
					image.ncol <- image.breaks - 1
					key.breaks <- image.breaks <- seq(min(fit$preds$z, na.rm = T),max(fit$preds$z, 
					na.rm = T),	length=image.breaks)
			        if (class(image.col) == "function") 
			        	image.c <- image.col(length(image.breaks)-1) 
			        else image.c <- image.col
			    	key.image.c <- image.c
			    }
			    key.ntxt <- ifelse(image.ncol<15,image.ncol+1,15)  
	            image(fit$preds, col = image.c, axes = F, add = T, breaks = image.breaks)
	    	}        

			if (contour.add) {
			    if (length(contour.levs)==1) 
			        contour.levs <- pretty(range(fit$preds$z, na.rm = T), 
			          contour.levs)
			    cat("Levels ", contour.levs, "\n")
			    contour(fit$preds, add = T, vfont = NULL, labcex = contour.cex, 
			        levels = contour.levs, col = contour.col)
			}
    	}
	}
	else {
		image.add = FALSE
		contour.add = FALSE
	}

### END of IMAGE & CONTOUR SETUP & PLOT ###

### ADD layers etc to PLOT ###

    if (auto.cex) {
        cex.d0 <- par()$cex
        cex.rng <- diff(range(ylim))
    }
    if (length(shape)) {
	    for (i in 1:length(shape)) {
#	        if (is.null(attr(shape[[i]],"color"))) shape[[i]] <- attrSet(shape[[i]])
			if (!is.null(attr(shape[[i]], "color"))) shape.color <- attr(shape[[i]], "color") 
			else shape.color <- NA
			if (!is.null(attr(shape[[i]], "border"))) shape.border <- attr(shape[[i]], "border") 
			else shape.border <- NA
			if (!is.null(attr(shape[[i]], "density"))) shape.border <- attr(shape[[i]], "density") 
			else shape.density <- NA
			if (!is.null(attr(shape[[i]], "border"))) shape.angle <- attr(shape[[i]], "angle") 
			else shape.angle <- NA
			if (!is.null(attr(shape[[i]], "do"))) shape.do <- attr(shape[[i]], "do") 
			else shape.do <- rep(T,length(shape))
			plot(shape[[i]], col = shape.color, border = shape.border, density = shape.density,
			angle = shape.angle, add = TRUE, forcefill = FALSE, do = shape.do, xlim = xlim, 
			ylim = ylim)
		}
    }
    if (length(data)) {
	    for (i in 1:length(data)) {
#	        if (is.null(attr(data[[i]],"xy"))) data[[i]] <- attrSet(data[[i]])
			xy <- attr(data[[i]], "xy")
			pts.add <- attr(data[[i]], "pts.add")
			ins <- (data[[i]][,xy[1]] > xlim[1]) & (data[[i]][,xy[1]] < xlim[2]) & 
				(data[[i]][,xy[2]] > ylim[1]) & (data[[i]][,xy[2]] < ylim[2])
			if (pts.add) {
				data.pch <- attr(data[[i]], "pch") 
				data.cex <- attr(data[[i]], "cex") 
				data.col <- attr(data[[i]], "col") 
				data.bg <- attr(data[[i]], "bg")  
				points(data[[i]][ins, xy[1]], data[[i]][ins, xy[2]], pch = data.pch, 
					cex = data.cex, col = data.col, bg = data.bg) 
			}
			txt.add <- attr(data[[i]], "txt.add")
			if (txt.add) {
				txt.text <- attr(data[[i]], "txt.text")
				txt.cex <- attr(data[[i]], "txt.cex")
				txt.col <- attr(data[[i]], "txt.col")
				pos <- attr(data[[i]], "txt.pos")
		        if (pos==0) pos <- NULL 
		        else if (pos<0) pos <- thigmophobe(data[[i]][ins, xy[1]], data[[i]][ins, xy[2]])
		        text(data[[i]][ins, xy[1]], data[[i]][ins, xy[2]], labels = txt.text[ins], 
		        	cex = txt.cex, col = txt.col, pos = pos) 		
		    }    
  		}
    }
    if (!missing(fit)){
    	if (key & image.add){
			if (is.null(image.breaks))  
				key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = key.image.c, 
				key.horiz = key.horiz, key.fun = key.fun)
			else  
				key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = key.image.c, 
				key.horiz = key.horiz, key.fun = key.fun, breaks=key.breaks)
		}
	}
    if (axes.add) box()
    if (north) 
        arrow.north(x = north.x, y = north.y)
    if (scale.bar.add) 
        scale.bar(scale.bar.x, scale.bar.y)

### END of regular PLOT ###

### IF INTERACTIVE ###
   
    if (interactive) {
        xy <- 1
        while (length(xy)) {
            xy <- loc.pts(data, xc = long, yc = lat, sn = data.names, 
                cex = data.names.cex)
            if (is.null(xy)) 
                return(invisible(list(xlim = xlim, ylim = ylim)))
            xlim <- xy$xlim
            ylim <- xy$ylim
            if (is.na(xlim[1])) xlim <- xylims0$x
            if (is.na(ylim[1])) ylim <- xylims0$y
            plot(xlim, ylim, xlim = xlim, ylim = ylim, axes = F, 
                asp = 1, xlab = ifelse(axes.add, xlab, ""),
                ylab = ifelse(axes.add, ylab, ""), type = "n",
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
            if (!rotate & axes.add) {
                axis(1, at = axTicks(1), lab = dms(axTicks(1), type = "long"))
                axis(2, at = axTicks(2), lab = dms(axTicks(2), type = "lat"))
            }
            if (!missing(fit)) {
                if (image.expand) {
                  nexp <- floor(log2(cex.rng/abs(diff(range(ylim))))) +  1
                  if (nexp > 1) {
                    preds.new <- fit$preds
                    for (i in 1:(nexp - 1)) preds.new <- mat.exp(preds.new, 
                      par()$usr[1:2], par()$usr[3:4])
                    prng <- range(fit$preds$z, na.rm = T)
                    new.prng <- range(preds.new$z, na.rm = T)
                    min1 <- round((image.ncol - 1) * (new.prng[1] - 
                      prng[1])/diff(prng)) + 1
                    max1 <- round((image.ncol - 1) * (new.prng[2] - 
                      prng[1])/diff(prng)) + 1
                    if (image.add) 
                      image(preds.new, col = image.c[min1:max1], axes = F, add = T)
                    if (contour.add) 
                      contour(preds.new, add = T, vfont = NULL, 
                        labcex = contour.cex, levels = contour.levs)
                  }
                  else {
                    if (image.add) 
                      image(fit$preds, col = image.c, axes = F, add = T)
                    if (contour.add) 
                      contour(fit$preds, add = T, vfont = NULL, 
                        labcex = contour.cex, levels = contour.levs)
                  }
                }
                else {
                  if (image.add) 
                    image(fit$preds, col = image.c, axes = F, add = T)
                  if (contour.add) 
                    contour(fit$preds, add = T, vfont = NULL, 
                      labcex = contour.cex, levels = contour.levs, col = contour.col)
                }
            }
			if (length(shape)) {
				for (i in 1:length(shape)) {
					if (!is.null(attr(shape[[i]], "color"))) shape.color <- attr(shape[[i]], "color") 
					else shape.color <- NA
					if (!is.null(attr(shape[[i]], "border"))) shape.border <- attr(shape[[i]], "border") 
					else shape.border <- NA
					if (!is.null(attr(shape[[i]], "do"))) shape.do <- attr(shape[[i]], "do") 
					else shape.do <- rep(T,length(shape))
					plot(shape[[i]], col = shape.color, border = shape.border, 
					add = TRUE, forcefill = FALSE, do = shape.do, xlim = xlim, ylim = ylim)
				}
			}
            if (auto.cex) {
                  auto.cex.exp0 <- (cex.rng/diff(range(ylim)))
                  auto.cex.exp1 <- auto.cex.exp0^(1/4)
                  auto.cex.exp2 <- auto.cex.exp0^(1/6)
            }
			if (length(data)) {
				for (i in 1:length(data)) {
					xy <- attr(data[[i]], "xy")
					pts.add <- attr(data[[i]], "pts.add")
					ins <- (data[[i]][,xy[1]] > xlim[1]) & (data[[i]][,xy[1]] < xlim[2]) & 
						(data[[i]][,xy[2]] > ylim[1]) & (data[[i]][,xy[2]] < ylim[2])
					if (any(ins)) {
						if (pts.add) {
							data.pch <- attr(data[[i]], "pch") 
							data.cex <- attr(data[[i]], "cex") * auto.cex.exp2
							data.col <- attr(data[[i]], "col") 
							data.bg <- attr(data[[i]], "bg")  
							points(data[[i]][, xy[1]], data[[i]][, xy[2]], pch = data.pch, 
								cex = data.cex, col = data.col, bg = data.bg)
						}
						txt.add <- attr(data[[i]], "txt.add")
						if (txt.add) {
							txt.text <- attr(data[[i]], "txt.text")
							txt.cex <- attr(data[[i]], "txt.cex")
							txt.col <- attr(data[[i]], "txt.col")
							pos <- attr(data[[i]], "txt.pos")
							if (pos==0) pos <- NULL 
							else if (pos<0) pos <- thigmophobe(data[[i]][, xy[1]], data[[i]][, xy[2]])
							text(data[[i]][ins, xy[1]], data[[i]][ins, xy[2]], labels = txt.text[ins], 
								cex = txt.cex, col = txt.col, pos = pos) 		
						}
					}    
		  		}
			}
			if (!missing(fit) & key & image.add){
				if (missing(image.quant))  key.pos(kx = key.x, ky = key.y, predz = fit$preds$z,
				 key.col = image.c, key.horiz = key.horiz,
					 key.fun = key.fun)
				else  key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = image.c,
				 key.horiz = key.horiz,
					 key.fun = key.fun, image.breaks)
			}
	        if (axes.add) box()
            if (north) 
                arrow.north(x = north.x, y = north.y)
            if (scale.bar.add) 
                scale.bar(scale.bar.x, scale.bar.y)
        }
    }
    if (interactive) 
        xy$cent <- cent
    else xylims0$cent <- cent
    invisible(if (interactive) xy else xylims0)
}

