`geoMap3` <-
function (data, fit, fit.se=FALSE,  locale = c("GBR", "WA")[1], rotate = FALSE, alpha = 40,
	bound = boundary.aa, cent, 
	cexp = par()$cex, interactive = FALSE, xlim, ylim, axes.add = T, 
	pts.add = TRUE, pch.d = 21, cex.d = cexp, col.d = "black", 
	bg.d = "lightblue", auto.cex = TRUE, xlab = "Longitude", ylab = "Latitude",
	lat = "lat", long = "long", bgrnd.add = FALSE, bgrnd.col = grey(0.95), 
    data.names = rownames(data), data.names.add = FALSE, data.names.cex = cexp,
    layer = zoning, layer.add = FALSE, 
    stars.add = FALSE, stars = data, stars.vars, stars.key = c(0.86, 0.86),
    stars.len = 1, stars.full = TRUE, stars.col = rainbow(length(stars.vars)), 
    pick.pos = 0, towns = towns6, towns.add = FALSE, towns.pch = 16, 
    towns.col = "black", towns.cex = cexp, towns.pos = 2, towns.offset = 0.5,
    rivers = rivers9, rivers.add = FALSE, rivers.pch = 16, rivers.col = "darkblue",
    rivers.cex = cexp, rivers.pos = 2,
    image.add = TRUE, image.se = FALSE, image.col = colorRampPalette(c("green4",
    "yellow", "orange", "firebrick3"), space = "Lab"), image.breaks,
    image.ncol = 9, image.quant, image.expand = TRUE, 
    contour.add = FALSE, contour.col = "black", contour.cex = cexp, contour.levs, 
    contour.nlevs = 10, scale.bar.add = T, scale.bar.x = c(0.05, 0.3), scale.bar.y = 0.025, 
	north = FALSE, north.x = 0.9, north.y = 0.95, 
	mar.rot = rep(1, 4), mar.no.rot = c(3.5, 4, 1, 1), mgap = c(2, 0.5, 0), 
    key = TRUE, key.fun, key.y = c(0.65, 0.95), key.x = c(0.84, 0.9), key.horiz = FALSE, 
    key.ntxt = ifelse(image.ncol<10,image.ncol+1,10),  
    nbr.fit = 0, qprob.fit = (1:(nbr.fit - 1))/nbr.fit, nbr.se = 0,
    qprob.se = (1:(nbr.se - 1))/nbr.se, hrnge = c(0, 1), srnge = c(0, 1), 
    vrnge = c(0, 1), X11.type = c("Xlib","cairo")[1], ...) 
{
	if (!missing(image.breaks)) {
		key.breaks <- image.breaks
		image.ncol <- length(image.breaks)-1
	}
	
	if (.Platform$GUI == "X11") {
		old.X11.type <- X11.options()$type
		on.exit(X11.options(type=old.X11.type))
		X11.options(type=X11.type)
	}

    if (layer.add) {
        if (class(layer)!="polylist" & any(sapply(layer,class)!="polylist")) 
        	stop("layer must be a polylist OR a list of polylists")
        require(layers)
	}
	
	if (missing(data) & missing(cent)) cent <- c(147,-18)
	
	bg.fun <- function(col="slategray1") {
		xpd <- par()$usr[1:2]
		ypd <- par()$usr[3:4]
		polygon(c(xpd,rev(xpd)),rep(ypd,each=2),col = col,border = NA)
	}
	expand <- function(x, eps = 0.01) {
        x + eps * c(-1, 1) * diff(x)
    }
    quant <- function(x, qq, qnt = c(0.6,0.9)) {
		if (missing(qq)) qq <- quantile(x[!is.na(x)], prob=qnt)
		cat("Quantiles = ", qq, "\n")
		xx <- x
		xx[!is.na(xx)] <- 1
		for (i in 1:length(qq)) xx[x>qq[i]] <- i + 1 
		xx
	}
	dms <- function(theta, type = c("latitude", "longitude",
                  "LATITUDE", "LONGITUDE",
                  "Latitude", "Longitude"), eps = 1.0e-6) {
	  type <- tolower(match.arg(type))
	  hemi <- sign(theta)+2
	  if (type == "latitude") nsew <- c("S"," ","N")[hemi]
	  else nsew <- c("W"," ","E")[hemi]
	  th <- abs(theta)+eps
	  deg <- floor(th)
	  min <- floor(60*(th - deg))
	  sec <- floor(60*60*(th - deg - min/60))
	  deg.ch <- as.character(deg)
	  nch <- nchar(deg.ch)
#	  deg.ch[nch < max(nch)] <- paste("0", deg.ch[nch < max(nch)], sep = "")
	  min.ch <- as.character(min)
	  nch <- nchar(min.ch)
	  min.ch[nch == 1] <- paste("0", min.ch[nch == 1], sep = "")
	  sec.ch <- as.character(sec)
	  nch <- nchar(sec.ch)
	  sec.ch[nch == 1] <- paste("0", sec.ch[nch == 1], sep = "")
	  if (all(min == 0)) 
		  paste(deg.ch, "°", nsew, sep = "")
	  else if (diff(range(theta)) > 0.25) 
		  paste(deg.ch, "°", min.ch, "'", nsew, sep = "")
	  else paste(deg.ch, "°", min.ch, "'", sec.ch, "''", 
		         nsew, sep = "")
	}
    scale.bar <- function(scx = scale.bar.x, scy = scale.bar.y) {
        xi <- par()$usr[1:2]
        xii <- c(sum(xi * c(1 - scx[1], scx[1])), sum(xi * c(1 - 
            scx[2], scx[2])))
        dfx <- diff(range(xi))
        scz <- diff(range(scx))
        dfxii <- scz * dfx * 111
        opts <- c(outer(c(1, 2, 5), 10^(-3:3)))
        which <- sum(dfxii > opts)
        opts.lab <- paste(c(outer(c(1, 2, 5), 10^c(0:2, 0:3))), 
            c(rep(c("m", "km"), c(9, 12))), sep = "")
        yi <- par()$usr[3:4]
        yii <- yi[1] + scy * diff(range(yi))
        xii[2] <- xii[1] + opts[which]/111
        lines(xii, c(yii, yii), lwd = 1)
        lines(c(xii[1], xii[1]), c(yii, yii + 0.015 * diff(range(yi))), 
            lwd = 1)
        lines(c(xii[2], xii[2]), c(yii, yii + 0.015 * diff(range(yi))), 
            lwd = 1)
        text(mean(xii), yii, opts.lab[which], pos = 3, cex = cexp * 
            0.85)
    }
    key.pos <- function(kx = key.x, ky = key.y, predz = fit$preds$z,
    		key.col = key.image.c, breaks = key.breaks, key.horiz, key.fun)    {
        if (!missing(breaks)) {
            minp <- min(breaks)
            maxp <- max(breaks)
        }
        else {
            minp <- min(predz, na.rm = T)
            maxp <- max(predz, na.rm = T)
        }

		if (key.horiz) { nr <- 50; nc <- 1 }
		else { nr <- 1; nc <- 50 }		
        xi <- par()$usr[1:2]
        xii <- xi[1] + kx * diff(range(xi))
        yi <- par()$usr[3:4]
        yii <- yi[1] + ky * diff(range(yi))
		
		if (missing(breaks))
	        image(z = matrix(seq(minp, maxp, len = 50), nrow = nr, ncol = nc),
            x = xii, y = seq(yii[1], yii[2], len = 50), col = key.col, 
            xlab = "", ylab = "", axes = T, add = T)
		else 
			image(z = matrix(seq(minp, maxp, len = 50), nrow = nr, ncol = nc),
			x = xii, y = seq(yii[1], yii[2], len = 50), col = key.col, 
			xlab = "", ylab = "", axes = T, add = T, breaks = breaks)

        if (!missing(key.fun)) 
        	txt <- signif(key.fun(seq(minp,maxp,length=key.ntxt)),3)
		else txt <- signif(seq(minp,maxp,length=key.ntxt),3)
        text(xii[2] + 1.5*par()$cxy[1], seq(yii[1],yii[2],length=key.ntxt), txt,
		cex = cexp * 0.8)
    }
    arrow.north <- function(x = north.x, y = north.y, rot = rotate, 
        alp = alpha, len = 1, angle = 30, lwd = 2, lty = 1, col = 1, 
        cex.arr = 1.25) {
        xi <- par()$usr[1:2]
        dfx <- diff(range(xi))
        xii <- xi[1] + x * dfx
        yi <- par()$usr[3:4]
        dfy <- diff(range(yi))
        yii <- yi[1] + y * dfy
        len.arr <- 0.025 * len * max(par()$pin)
        len <- abs(0.025 * len * max(dfy, dfx))
        if (!(rot)) 
            alp <- 0
        theta <- pi * alp/180
        st <- sin(theta)
        ct <- cos(theta)
        x0 <- xii - len * st
        y0 <- yii - len * ct
        x1 <- xii + len * st
        y1 <- yii + len * ct
        arrows(x0, y0, x1, y1, len = len.arr, angle = angle, 
            lwd = lwd, lty = lty, col = col)
        par(srt = rotate)
        text(x0, 0.7 * y0 + 0.3 * y1, "N", cex = cex.arr * cexp)
        par(srt = 0)
    }
    fit.se.cols <- function(preds, hr = c(0, 1), sr = c(0, 1), 
        vr = c(0, 1), image.add = TRUE, axes = FALSE) {
        nrow <- length(preds$x)
        ncol <- length(preds$y)
        n <- nrow * ncol
        fit <- c((preds$z - min(preds$z, na.rm = T))/diff(range(preds$z, 
            na.rm = T)))
        fit[is.na(fit)] <- 0
        fit <- diff(hr) * fit + hr[1]
        se <- c(preds$z.se)
        se <- rank(se, na = "keep")
        se <- 1 - (se - min(se, na.rm = T))/diff(range(se, na.rm = T))
        sr <- diff(sr) * se + sr[1]
        vr <- diff(vr) * se + vr[1]
        vr[is.na(se)] <- 1
        sr[is.na(se)] <- 0
        image(x = preds$x, y = preds$y, z = matrix(1:n, nrow = nrow), 
            col = hsv(h = fit, s = sr, v = vr), add = image.add)
    }

    if (rotate) 
        margins <- mar.rot
    else margins <- mar.no.rot
    	old.par <- par(xpd = F, mar = margins, cex.axis = 0.8 * cexp, 
        cex.lab = 0.9 * cexp, mgp = mgap)
	    on.exit(old.par)
    if (alpha != 0 & rotate) {
        cat("Rotating data sets\n")
        if (!missing(data)) {
            cent <- apply(data[, c(long, lat)], 2, mean)
            data[, c(long, lat)] <- ll2pm(data[, c(long, lat)], 
                alpha = alpha, cent = cent)
        }
		if (!missing(fit)) {
			fit$preds <- rotate.grid(fit$preds, alpha=alpha, cent=cent, bound = bound)
		}
        if (layer.add) 
			for (i in 1:length(layer)) 
				layer[[i]] <- ll2pm.polylist(layer[[i]], alpha = alpha, cent = cent)
        if (towns.add) 
            towns[, c(long, lat)] <- ll2pm(towns[, c(long, lat)], 
                alpha = alpha, cent = cent)
        if (stars.add) 
            stars[, c(long, lat)] <- ll2pm(stars[, c(long, lat)], 
                alpha = alpha, cent = cent)
        if (rivers.add) 
            rivers[, c(long, lat)] <- ll2pm(rivers[, c(long, 
                lat)], alpha = alpha, cent = cent)
    }
    if (!missing(xlim) & !missing(ylim)) 
        xylims0 <- list(x = xlim, y = ylim)
    else if (!missing(data) & (missing(xlim) | missing(ylim))) {
        xlim <- expand(range(data[, long]))
        ylim <- expand(range(data[, lat]))
        xylims0 <- list(x = xlim, y = ylim)
    }
    else if (!missing(fit) & (missing(xlim) | missing(ylim))) {
        xlim <- range(fit$preds$x)
        ylim <- range(fit$preds$y)
        xylims0 <- list(x = xlim, y = ylim)
    }
    else {
        if (locale == "GBR") {
        xlim <- c(142, 153); ylim <- c(-24, -11)
        }
        else {
        xlim <- c(111, 156); ylim <- c(-47, -7)
        }
        if (alpha != 0 & rotate) {
            temp <- ll2pm(cbind(xlim, ylim), alpha = alpha)
            xlim <- temp[, 1]
            ylim <- temp[, 2]
        }
        xylims0 <- list(xlim = xlim, ylim = ylim)
    }

	missing.image.breaks <- missing(image.breaks)
	missing.contour.levs <- missing(contour.levs)
#	
#	plot.fit <- function() {
#

if (!missing(fit))  {
        if (rotate) {
            	plot(range(fit$preds$x), range(fit$preds$y), 
                axes = F, asp = 1, xlab = "", ylab = "", type = "n", 
                xlim = xlim, ylim = ylim, 
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
        }
        else {
            	plot(range(fit$preds$x), range(fit$preds$y), 
                asp = 1, xlab = ifelse(axes.add, xlab, ""),
                ylab = ifelse(axes.add, ylab, ""),
                type = "n", axes = F, xlim = xlim, ylim = ylim,
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
            if (axes.add) {
		        axis(1, at = axTicks(1), lab = dms(axTicks(1), type = "long"))
		        axis(2, at = axTicks(2), lab = dms(axTicks(2), type = "lat"))
            }
        }
        if (fit.se) {
            if (is.null(fit$preds$z.se)) 
                stop("No SEs available\n")
            else fit$preds$z <- fit$preds$z.se
        }
        if (image.se & image.add) {
            contour.add <- FALSE
            if (nbr.fit > 0) 
                fit$preds$z <- quant(fit$preds$z, qnt = qprob.fit)
            if (nbr.se > 0) 
                fit$preds$z.se <- quant(fit$preds$z.se, qnt = qprob.se)
            fit.se.cols(fit$preds, hr = hrnge, sr = srnge, vr = vrnge, 
                image.add = T, axes = F)
            len <- ifelse(nbr.fit, nbr.fit, 50)
            image.c <- hsv(h = seq(hrnge[1], hrnge[2], length = len), 
                s = 1, v = 1)
        }
        else {
            if (!missing.image.breaks) {
                if ((min(image.breaks) > min(fit$preds$z, na.rm = T)) | 
                  (max(image.breaks) < max(fit$preds$z, na.rm = T))) {
                  cat("Triming on range\n")
                  fit$preds$z[fit$preds$z < min(image.breaks)] <- NA
                  fit$preds$z[fit$preds$z > max(image.breaks)] <- NA
                }
                nmin <- (1:length(image.breaks))[image.breaks < min(fit$preds$z,na.rm = T)] 
                nmax <- (1:length(image.breaks))[image.breaks > max(fit$preds$z,na.rm = T)]
                image.breaks <- image.breaks[max(nmin):min(nmax)]
                ncols <- max(nmin):(min(nmax)-1)
                if (class(image.col) == "function") { 
		              image.c <- image.col(image.ncol)[ncols]
		              key.image.c <- image.col(image.ncol)
                }
                else {
					image.c <- image.col[ncols]
		            key.image.c <- image.col
                }

            }
            else {
				key.breaks <- image.breaks <- seq(min(fit$preds$z, na.rm = T),max(fit$preds$z, 
				na.rm = T),	length=image.ncol+1)
                if (class(image.col) == "function") 
                	image.c <- image.col(image.ncol) 
                else image.c <- image.col
            	key.image.c <- image.c
            }
            if (image.add) {
				if (!missing.image.breaks)
	                image(fit$preds, col = image.c, axes = F, add = T, breaks = image.breaks)
        		else {
        			image.breaks <- quantile(fit$preds$z,probs=image.quant,na.rm=T) 	
	                cat("Breaks: ",signif(image.breaks,3),"\n")
	                image(fit$preds, col = image.c, axes = F, add = T, breaks = image.breaks)
	            }
        	}        }
        if (contour.add) {
            if (missing.contour.levs) 
                contour.levs <- pretty(range(fit$preds$z, na.rm = T), 
                  contour.nlevs)
            cat("Levels ", contour.levs, "\n")
            contour(fit$preds, add = T, vfont = NULL, labcex = contour.cex, 
                levels = contour.levs, col = contour.col)
        }
    }

#   if (!missing(fit)) plot.fit()

    else if (missing(fit)) {
        if (rotate) {
            	plot(0, 0, xlim = xlim, ylim = ylim, axes = F, 
                asp = 1, xlab = "", ylab = "", type = "n",
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
        }
        else {
            	plot(0, 0, xlim = xlim, ylim = ylim, asp = 1,
                 xlab = ifelse(axes.add, xlab, 
                  ""), ylab = ifelse(axes.add, ylab, ""), 
                type = "n", axes = F, panel.first = if (bgrnd.add) 
                  bg.fun(col = bgrnd.col)
                else NULL)
            if (axes.add) {
                axis(1, at = axTicks(1), lab = dms(axTicks(1), 
                  type = "long"))
                axis(2, at = axTicks(2), lab = dms(axTicks(2), 
                  type = "lat"))
            }
        }
    }
    if (auto.cex | image.expand) {
        cex.d0 <- cex.d
        cex.rng <- diff(range(ylim))
    }
    if (layer.add) {
	    for (i in 1:length(layer)) {
			if (!is.null(attr(layer[[i]], "color"))) layer.color <- attr(layer[[i]], "color") 
			else layer.color <- NA
			if (!is.null(attr(layer[[i]], "border"))) layer.border <- attr(layer[[i]], "border") 
			else layer.border <- NA
			if (!is.null(attr(layer[[i]], "do"))) layer.do <- attr(layer[[i]], "do") 
			else layer.do <- rep(T,length(layer))
			plot(layer[[i]], col = layer.color, border = layer.border, 
			add = TRUE, forcefill = FALSE, do = layer.do, xlim = xlim, 
			ylim = ylim, ...)
		}
    }
    if (!missing(fit) & key & image.add){
		if (missing(image.breaks))  
			key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = key.image.c, 
			key.horiz = key.horiz, key.fun = key.fun)
		else  
			key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = key.image.c, 
			key.horiz = key.horiz, key.fun = key.fun, breaks=key.breaks)
	}
    if (axes.add) box()
    if (towns.add) {
        points(towns[, c(long, lat)], pch = towns.pch, col = towns.col, 
            cex = towns.cex)
        text(towns[, long], towns[, lat], towns[, 1], col = towns.col, 
            cex = 0.8 * towns.cex, pos = towns.pos, offset = towns.offset)
    }
    if (rivers.add) {
        points(rivers[, c(long, lat)], pch = rivers.pch, col = rivers.col, 
            cex = rivers.cex)
        text(rivers[, long], rivers[, lat], rivers[, 1], col = rivers.col, 
            cex = 0.8 * rivers.cex, pos = rivers.pos)
    }
    if (!missing(data)) {
        if (stars.add) {
            stars.len <- stars.len * 0.015 * (diff(range(ylim)) + 
                diff(range(xlim)))
            stars.data <- apply(stars[, stars.vars], 2, scale01)
            stars.ord <- rev(order(apply(stars.data, 1, sum)))
            xi <- sum(par()$usr[1:2] * c(1 - stars.key[1], stars.key[1]))
            yi <- sum(par()$usr[3:4] * c(1 - stars.key[2], stars.key[2]))
            stars.key.pos <- c(xi, yi)
            stars(stars.data[stars.ord, ], location = stars[stars.ord, 
                c(long, lat)], add = TRUE, len = stars.len, labels = NULL, 
                draw.segments = TRUE, full = stars.full, col.segments = stars.col, 
                cex = cexp * 0.75, key.loc = stars.key.pos, key.xpd = TRUE)
        }
        if (pts.add) {
            points(data[, long], data[, lat], pch = pch.d, cex = cex.d, 
                col = col.d, bg = bg.d)
        }
    }
    if (data.names.add) 
        add.names(data, long, lat, data.names, data.names.cex, pos = pick.pos)
    if (north) 
        arrow.north(x = north.x, y = north.y)
    if (scale.bar.add) 
        scale.bar(scale.bar.x, scale.bar.y)
    if (interactive) {
        xy <- 1
        while (length(xy)) {
            xy <- loc.pts(data, xc = long, yc = lat, sn = data.names, 
                cex = data.names.cex)
            if (is.null(xy)) 
                return(invisible(list(xlim = xlim, ylim = ylim)))
            xlim <- xy$xlim
            ylim <- xy$ylim
            if (is.na(xlim[1])) 
                xlim <- xylims0$x
            if (is.na(ylim[1])) 
                ylim <- xylims0$y
            plot(xlim, ylim, xlim = xlim, ylim = ylim, axes = F, 
                asp = 1, xlab = ifelse(axes.add, xlab, ""),
                ylab = ifelse(axes.add, ylab, ""), type = "n",
                panel.first = if (bgrnd.add) bg.fun(col=bgrnd.col) else NULL)
            if (!rotate & axes.add) {
                axis(1, at = axTicks(1), lab = dms(axTicks(1), 
                  type = "long"))
                axis(2, at = axTicks(2), lab = dms(axTicks(2), 
                  type = "lat"))
            }
            if (!missing(fit)) {
                if (image.expand) {
                  nexp <- floor(log2(cex.rng/abs(diff(range(ylim))))) + 
                    1
                  if (nexp > 1) {
                    preds.new <- fit$preds
                    for (i in 1:(nexp - 1)) preds.new <- mat.exp(preds.new, 
                      par()$usr[1:2], par()$usr[3:4])
                    prng <- range(fit$preds$z, na.rm = T)
                    new.prng <- range(preds.new$z, na.rm = T)
                    min1 <- round((image.ncol - 1) * (new.prng[1] - 
                      prng[1])/diff(prng)) + 1
                    max1 <- round((image.ncol - 1) * (new.prng[2] - 
                      prng[1])/diff(prng)) + 1
                    if (image.add) 
                      image(preds.new, col = image.c[min1:max1], 
                        axes = F, add = T)
                    if (contour.add) 
                      contour(preds.new, add = T, vfont = NULL, 
                        labcex = contour.cex, levels = contour.levs)
                  }
                  else {
                    if (image.add) 
                      image(fit$preds, col = image.c, axes = F, 
                        add = T)
                    if (contour.add) 
                      contour(fit$preds, add = T, vfont = NULL, 
                        labcex = contour.cex, levels = contour.levs)
                  }
                }
                else {
                  if (image.add) 
                    image(fit$preds, col = image.c, axes = F, 
                      add = T)
                  if (contour.add) 
                    contour(fit$preds, add = T, vfont = NULL, 
                      labcex = contour.cex, levels = contour.levs, 
                      col = contour.col)
                }
            }
			if (layer.add) {
				for (i in 1:length(layer)) {
					if (!is.null(attr(layer[[i]], "color"))) layer.color <- attr(layer[[i]], "color") 
					else layer.color <- NA
					if (!is.null(attr(layer[[i]], "border"))) layer.border <- attr(layer[[i]], "border") 
					else layer.border <- NA
					if (!is.null(attr(layer[[i]], "do"))) layer.do <- attr(layer[[i]], "do") 
					else layer.do <- rep(T,length(layer))
					plot(layer[[i]], col = layer.color, border = layer.border, 
					add = TRUE, forcefill = FALSE, do = layer.do, xlim = xlim, 
					ylim = ylim, ...)
				}
			}
            if (auto.cex & (pts.add | stars.add)) {
                  auto.cex.exp0 <- (cex.rng/diff(range(ylim)))
                  auto.cex.exp1 <- auto.cex.exp0^(1/4)
                  auto.cex.exp2 <- auto.cex.exp0^(1/6)
            }
            if (towns.add) {
                points(towns[, c(long, lat)], pch = towns.pch, 
                  col = towns.col, cex = auto.cex.exp2 * towns.cex)
                text(towns[, long], towns[, lat], towns[, 1], 
                  col = towns.col, cex = auto.cex.exp2 * 0.8 * 
                    towns.cex, pos = towns.pos, offset = towns.offset)
            }
            if (rivers.add) {
                points(rivers[, c(long, lat)], pch = rivers.pch, 
                  col = rivers.col, cex = auto.cex.exp2 * rivers.cex)
                text(rivers[, long], rivers[, lat], rivers[, 
                  1], col = rivers.col, cex = auto.cex.exp2 * 
                  0.8 * rivers.cex, pos = rivers.pos)
            }
            if (!missing(data)) {
                if (pts.add) {
                  cex.d <- cex.d0 * ifelse(auto.cex, auto.cex.exp2, 
                    1)
                  points(data[, long], data[, lat], pch = pch.d, 
                    cex = cex.d, col = col.d, bg = bg.d)
                }
                if (stars.add) {
                  xi <- sum(par()$usr[1:2] * c(1 - stars.key[1], 
                    stars.key[1]))
                  yi <- sum(par()$usr[3:4] * c(1 - stars.key[2], 
                    stars.key[2]))
                  stars.key.pos <- c(xi, yi)
                  stars(stars.data[stars.ord, ], location = stars[stars.ord, 
                    c(long, lat)], add = TRUE, len = stars.len * 
                    ifelse(auto.cex, auto.cex.exp0^(-0.75), 1), 
                    labels = NULL, draw.segments = TRUE, full = stars.full, 
                    col.segments = stars.col, cex = cexp * 0.75, 
                    key.loc = stars.key.pos, key.xpd = TRUE)
                }
            }
			if (!missing(fit) & key & image.add){
				if (missing(image.quant))  key.pos(kx = key.x, ky = key.y, predz = fit$preds$z,
				 key.col = image.c, key.horiz = key.horiz,
					 key.fun = key.fun)
				else  key.pos(kx = key.x, ky = key.y, predz = fit$preds$z, key.col = image.c,
				 key.horiz = key.horiz,
					 key.fun = key.fun, image.breaks)
			}
            if (data.names.add) 
                add.names(data, long, lat, data.names, auto.cex.exp2 * 0.8 *
                  data.names.cex, pos = pick.pos)
            if (axes.add) box()
            if (north) 
                arrow.north(x = north.x, y = north.y)
            if (scale.bar.add) 
                scale.bar(scale.bar.x, scale.bar.y)
        }
    }
    if (interactive) 
        xy$cent <- cent
    else xylims0$cent <- cent
    invisible(if (interactive) xy else xylims0)
}
