makegrid.chull <- function(data, long="long", lat="lat", size=0.05, buffer=0.1, block = 1000, plt=TRUE) {

##
## function for generating a grid from a data set using the convex hull
##
## "data" a data frame or matrix with long, lat -- if not missing 
##    add the data points to the plot
## "size": cell size of the grid; typically 0.01 for intial model selection, 
##    and 0.001 - 0.0005 for gridded outpus (ASCII & KMLs)
## "plt" if TRUE plot the grid
##

	xpdbox <- function(x, delta = 0.05) {
		x + delta * t(t(x) - apply(x, 2, mean))
	}
    boundary <- data[chull(data[,c(long,lat)]),c(long,lat)]
    boundary <- xpdbox(boundary, delta = buffer)

	rx <- range(boundary[,1])
	ry <- range(boundary[,2])
		
	rx2 <- ceiling(rx[2]/size)*size
	rx1 <- floor(rx[1]/size)*size
	ry2 <- ceiling(ry[2]/size)*size
	ry1 <- floor(ry[1]/size)*size

	grid <- expand.grid(long=seq(rx1,rx2,by=size),lat=seq(ry1,ry2,by=size))
	grid <- makegrid.clip(as.matrix(grid),as.matrix(boundary),block=1000)
	grid <- data.frame(na.omit(grid))
	if (plt) {
		plot(grid,pch=".",asp=1,xlab="Longitude",ylab="Latitude")
		lines(rbind(boundary,boundary[1,]), col="red")
		points(data[,c(long,lat)],pch=21,bg="lightblue")
	}
	attr(grid,"ndiv") <- round(1/size)
	grid
}
