makegrid.clip <- function(grid, bound, block = 1000) {

##
## function for clipping big grids
## 
## "grid": the input grid as a matrix of long & lat
##   can be generated by "makegrid"
## "boundary": the input boundary as a matrix of long & lat
##   can come from "makemask"
## "block" : the block size to prevent memory overflows 
## 


poly.in <- function(xy, poly)
{
    if(ncol(poly) == 2)
        poly <- poly.tst(poly)
    n <- nrow(xy)
    np <- nrow(poly)
    nnp <- rep(n, np)
    check1 <- xor(xy[, 1] >= rep(poly[, 1], nnp), xy[, 1] > rep(poly[, 3], nnp))
    check2 <- rep(poly[, 2], nnp) + rep(poly[, 4], nnp) * xy[, 1] > xy[, 2]
    as.vector(matrix(check1 & check2, n, np) %*% rep.int(1, np) %% 2 > 0)
}

poly.tst <- function(xy)
{
    if(is.list(xy))
        xy <- as.data.frame(xy)
    if(!is.matrix(xy) || ncol(xy) != 2 || !is.numeric(xy[, 1]) || !is.numeric(xy[, 2]))
    stop("xy must by nX2 numeric matrix or data.frame")
    #
    # Columns of poly are start.x, a, end.x, b
    xy <- as.matrix(xy)
    poly <- matrix(c(xy, xy[c(2:nrow(xy), 1),  ]), ncol = 4)
    poly <- poly[poly[, 1] != poly[, 3],  ]
    poly[, 4] <- (poly[, 4] - poly[, 2])/(poly[, 3] - poly[, 1])
    poly[, 2] <- poly[, 2] - poly[, 1] * poly[, 4]
    poly
}

	nr <- nrow(grid)
	if (nr > block) {
		keep <- rep(NA,nr)
		n <- ceiling(nr/block)
		for (i in 1:n) {
			inds <- (1 + (i-1)*block):(i*block)
			inds <- inds[inds<=nr]
			keep[inds] <- poly.in(grid[inds,], bound)
			cat(i,"")
		}
		cat("\n")
	}
	else {
		keep <- poly.in(grid, bound)
	}
	grid[keep,]
}
