makegrid.density <- function(data, long="long", lat="lat", eps=0.5, 
	use.line=1 ,size=0.1, block=1000, plt=TRUE) {

##
## function for generating grids with multiple areas based on spatial densities of points
## borders of grids are defined by a common (low) density boundary selected by the user
##
## "x,y": long and lat respectively
## "eps": controls the scale of smoothing -- try different values (range 0.1,1 ??) !!
## "use.lines": lines of the density smooth are labelled 1,2,... 
##    select which you want as your boundary
## "size": cell size of the grid; typically 0.01 for intial model selection, 
##    and 0.001 - 0.0005 for gridded outpus (ASCII & KMLs)
## 

	require(KernSmooth)
	x <- data[,long]
	y <- data[,lat]
	lst <- list()
	dens <- bkde2D(cbind(x,y),c(1,1)*eps)
	clines <- contourLines(dens$x1, dens$x2, dens$fhat)
	if (plt) {
		image(dens$x1, dens$x2, dens$fhat,xlab="Longitude",ylab="Latitude",asp=1,
		 col=rev(heat.colors(100)))
		contour(dens$x1, dens$x2, dens$fhat,add=T)
		points(x,y,pch=21,bg="lightblue")
		for (i in 1:length(use.line)) 
			lines(clines[[use.line[i]]],col="blue",lwd=2)
		box()
	}	
	for (i in 1:length(use.line)) {
		xlims <- range(clines[[use.line[i]]]$x)
		xlims <- c(floor(xlims[1]),ceiling(xlims[2]))
		ylims <- range(clines[[use.line[i]]]$y)
		ylims <- c(floor(ylims[1]),ceiling(ylims[2]))
		grid <- expand.grid(long=seq(xlims[1],xlims[2],by=size),
		lat=seq(ylims[1],ylims[2],by=size))
		xy <- cbind(long=clines[[use.line[i]]]$x,lat=clines[[use.line[i]]]$y)
		lst[[i]] <- makegrid.clip(as.matrix(grid),as.matrix(xy),block)
		cat("Data has",nrow(lst[[i]])," rows\n")
	}
	grid <- do.call("rbind",lst)
	print(summary(grid))
	attr(grid,"ndiv") <- round(1/size)
	grid
}
