makegrid.rect <- function(boundary, long="long", lat="lat", size=0.1, buffer=0.01, block = 1000, plt=TRUE) {

##
## function for generating a grid within a boundary
##
## "boundary": boundary coordinates in order "long" & "lat"  
## "size": cell size of the grid; typically 0.01 for intial model selection, 
##    and 0.001 - 0.0005 for gridded outpus (ASCII & KMLs)
## "plt" if TRUE plot the grid
## "data" a data frame or matrix with long, lat -- if not missing 
##    add the data points to the plot
##


	rx <- range(boundary[,long])
	ry <- range(boundary[,lat])

	rx2 <- ceiling((rx[2] + buffer)/size)*size
	rx1 <- floor((rx[1] - buffer)/size)*size
	ry2 <- ceiling((ry[2] + buffer)/size)*size
	ry1 <- floor((ry[1] - buffer)/size)*size

	grid <- expand.grid(long=seq(rx1,rx2,by=size),lat=seq(ry1,ry2,by=size))
	grid <- makegrid.clip(as.matrix(grid),as.matrix(boundary),block=1000)
	grid <- data.frame(na.omit(grid))
	if (plt) {
		plot(grid,pch=".",asp=1,xlab="Longitude",ylab="Latitude")
		lines(rbind(boundary,boundary[1,]), col="red")
	}
	attr(grid,"ndiv") <- round(1/size)
	grid
}
