makegrid.select <- function(data, long="long", lat="lat", size=0.1, buffer=0.01, block = 1000, plt=TRUE) {

##
## interactive function for defining a mask (boundary) and then 
## generating the grid within it.
## both grid and bundary are returned by the function
##
## the data points supplied in long-lat order in data are plotted
## then click to select the boundary points (in order!!)
##
##  arguments:
##  
## "data": data frame or matrix of coordinates in order "long" & "lat"  
## "size": cell size of the grid; typically 0.01 for intial model selection, 
##    and 0.001 - 0.0005 for gridded outpus (ASCII & KMLs)
## "plt" if TRUE plot the grid and boundary
## "block" size of data blocks passed to makegrid.clip
##
##    add the data points to the plot
##

	plot(data[,c(long,lat)],asp=1,xlab="Longitude",ylab="Latitude")
	boundary <- as.data.frame(locator(type="l"))
	names(boundary) <- c("long","lat")
	boundary

	rx <- range(boundary[,1])
	ry <- range(boundary[,2])

	rx2 <- ceiling((rx[2] + buffer)/size)*size
	rx1 <- floor((rx[1] - buffer)/size)*size
	ry2 <- ceiling((ry[2] + buffer)/size)*size
	ry1 <- floor((ry[1] - buffer)/size)*size

	grid <- expand.grid(long=seq(rx1,rx2,by=size),lat=seq(ry1,ry2,by=size))
	grid <- makegrid.clip(as.matrix(grid),as.matrix(boundary),block=1000)
	grid <- data.frame(na.omit(grid))
	if (plt) {
		plot(grid,pch=".",asp=1,xlab="Longitude",ylab="Latitude")
		points(data,pch=21,bg="lightblue")
		lines(rbind(boundary,boundary[1,]), col="red")
	}
	attr(grid,"ndiv") <- round(1/size)
	list(boundary=boundary, grid=grid)
}
